/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.client;

import javax.xml.namespace.QName;

import org.apache.axis.types.URI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import uk.org.ogsadai.dais.xml.service.core.*;
import uk.org.ogsadai.dais.xml.services.daix.*;
import uk.org.ogsadai.wsdaix.common.Serializer;

/**
 * An example of a client which utilises the WS-DAIX client toolkit.
 * It functions as an interface to the WS-DAIX client toolkit providing
 * XML representations of the results of operations.
 * 
 * @author Steven Lynden
 * 
 */
public class Client {

    /*
     * Abstract name of the data resource
     */
    private URI resourceName;
    
    /*
     * URL of the data service
     */
    private String serviceURL;
    
    /**
     * WS-DAI namespace
     */
    public static final String WS_DAI_NS = "http://www.ggf.org/namespaces/2005/12/WS-DAI";
    
    /*
     * Message QNames
     */
    
    /**
     * Data resource property document QName
     */
    public static final QName DATA_RESOURCE_PROPERTY_DOCUMENT = 
        new QName(WS_DAI_NS,"PropertyDocument");
    
    /**
     * Data resource address document QName
     */
    public static final QName DATA_RESOURCE_ADDRESS = 
        new QName(WS_DAI_NS,"DataResourceAddress");
    
    /**
     * Remove documents response QName
     */
    public static final QName REMOVE_DOCUMENTS_RESPONSE = 
        new QName(WS_DAI_NS,"RemoveDocumentsResponse");
    
    /**
     * Add documents response QName
     */
    public static final QName ADD_DOCUMENTS_RESPONSE = 
        new QName(WS_DAI_NS,"AddDocumentsResponse");
    
    /**
     * Remove collection response QName
     */
    public static final QName REMOVE_COLLECTION_RESPONSE = 
        new QName(WS_DAI_NS,"RemoveCollectioResponse");
    
    /**
     * Create collection response QName
     */
    public static final QName CREATE_COLLECTION_RESPONSE = 
        new QName(WS_DAI_NS,"CreateCollectionResponse");
    
    /**
     * Destroy data resource QName
     */
    public static final QName DESTROY_RESPONSE = 
        new QName(WS_DAI_NS,"DestroyDataResourceResponse");
    
    /**
     * Create a new client.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     */
    public Client(String serviceURL, URI resourceName) {
    	this.serviceURL = serviceURL;
        this.resourceName = resourceName;
    }
   
    /**
     * Retrieve a data resource property document
     */
    public Element getDataResourcePropertyDocument() throws Exception {
    	PropertyDocumentType properties = ClientToolkit.getDataResourcePropertyDocument(serviceURL,resourceName);
        return Serializer.objectToDOM(properties,DATA_RESOURCE_PROPERTY_DOCUMENT);
    }
    
    /**
     * Retrieve an XML collection property document
     */
    public Element getCollectionPropertyDocument() throws Exception {
        PropertyDocumentType properties = ClientToolkit.getCollectionPropertyDocument(serviceURL,resourceName);
        return Serializer.objectToDOM(properties,DATA_RESOURCE_PROPERTY_DOCUMENT);
    }
    
    /**
     * Retrieve an XML sequence property document
     */
    public Element getXMLSequencePropertyDocument() throws Exception {
        PropertyDocumentType properties = ClientToolkit.getXMLSequencePropertyDocument(serviceURL,resourceName);
        return Serializer.objectToDOM(properties,DATA_RESOURCE_PROPERTY_DOCUMENT);
    }
    
    /**
     * Resolve the data resource abstract name. Returns a list of
     * endpoints through which the resource may be accessed
     */
    public Element[] resolve() throws Exception {
        DataResourceAddressType[] addr = ClientToolkit.resolve(serviceURL,resourceName);
        return dataResourceAddressesToElements(addr);
    }
    
    /**
     * Destroys the data resource
     */
    public void destroy() throws Exception {
        DestroyDataResourceResponse r = ClientToolkit.destroyDataResource(serviceURL,resourceName);
    }
    
    /**
     * Returns the list of resources that can be addressed
     * through the data service
     */
    public Element[] listResources() throws Exception {
        DataResourceAddressType[] addr = ClientToolkit.getResourceList(serviceURL);
        return dataResourceAddressesToElements(addr);
    }
    
    /**
     * Executes the document selection factory operation.
     * Collection URI, portyTypeQName and config document parameters are
     * optional (may be set to NULL).
     */
    public Element documentSelectionFactory(URI collectionURI, String documentName, QName portTypeQName, ConfigurationDocumentType config) throws Exception {
        DataResourceAddressType r = ClientToolkit.documentSelectionFactory(serviceURL,resourceName,collectionURI,documentName,portTypeQName,config);
        return Serializer.objectToDOM(r,DATA_RESOURCE_ADDRESS);
    }
    
    /**
     * Executes the collection selection factory operation.
     * Collection URI, portyTypeQName and config document parameters are
     * optional (may be set to NULL).
     */
    public Element collectionSelectionFactory(URI collectionURI, QName portTypeQName, XMLCollectionConfigurationDocumentType config) throws Exception {
        DataResourceAddressType r = ClientToolkit.collectionSelectionFactory(serviceURL,resourceName,collectionURI,portTypeQName,config);
        return Serializer.objectToDOM(r,DATA_RESOURCE_ADDRESS);
    }
    
    /**
     * Executes the remove documents operation.
     * Collection URI parameter is optional (may be set to NULL).
     */
    public Element removeDocuments(URI collectionURI, String[] documentNames) throws Exception {
        RemoveDocumentsResponse r = ClientToolkit.removeDocuments(serviceURL,resourceName,collectionURI,documentNames);
        return Serializer.objectToDOM(r,REMOVE_DOCUMENTS_RESPONSE);
    }
    
    /**
     * Executes the add documents operation.
     * Collection URI parameter is optional (may be set to NULL).
     */
    public Element addDocuments(URI collectionURI, String[] documentNames, Document[] documents) throws Exception {
        AddDocumentsResponse r = ClientToolkit.addDocuments(serviceURL,resourceName,collectionURI,documentNames,documents);
        return Serializer.objectToDOM(r,ADD_DOCUMENTS_RESPONSE);
    }
    
    /**
     * Executes the remove collection operation.
     * Collection URI parameter is optional (may be set to NULL).
     */
    public Element removeCollection(URI collectionURI, URI subcollectionURI) throws Exception {
        RemoveSubcollectionResponse r = ClientToolkit.removeCollection(serviceURL,resourceName,collectionURI,subcollectionURI);
        return Serializer.objectToDOM(r,REMOVE_COLLECTION_RESPONSE);
    }
    
    /**
     * Executes the create collection operation.
     * Collection URI parameter is optional (may be set to NULL).
     */
    public Element createCollection(URI collectionURI, URI subcollectionURI) throws Exception {
        CreateSubcollectionResponse r = ClientToolkit.createCollection(serviceURL,resourceName,collectionURI,subcollectionURI);
        return Serializer.objectToDOM(r,CREATE_COLLECTION_RESPONSE);
    }
    
    /**
     * Executes the XQuery statement operation.
     * The expression parameter must be a valid XQueryX formatted
     * query.
     */
    public Element xQueryStatement(Element expression) throws Exception {
        XQueryExecuteResponse r = ClientToolkit.XQueryExecute(serviceURL,resourceName,expression);
        return r.getDataset().getDatasetData().get_any()[0].getAsDOM();
    }
    
    /**
     * Executes the XUpdate statement operation.
     * The expression parameter must be a valid XUpdate formatted
     * expression.
     */
    public long xUpdateStatement(Element expression) throws Exception {
        XUpdateExecuteResponse r = ClientToolkit.XUpdateExecute(serviceURL,resourceName,expression);
        return r.getCount().longValue();
    }
    
    /**
     * Retrieves the specified items from an XML sequence data
     * resource.
     */
    public Element getItems(long startPosition, long count) throws Exception {
        GetItemsResponse r = ClientToolkit.getItems(serviceURL,resourceName,startPosition,count);
        return r.getDataset().getDatasetData().get_any()[0].getAsDOM();
    }
    
    /**
     * Executes the XPath statement operation.
     * The expression parameter must be a valid XQueryX formatted
     * XPath query.
     */
    public Element xPathStatement(Element expression) throws Exception {
        XPathExecuteResponse r = ClientToolkit.XPathExecute(serviceURL,resourceName,expression);
        return r.getDataset().getDatasetData().get_any()[0].getAsDOM();
    }
    
    /**
     * Executes the XPath factory operation.
     * The expression parameter must be a valid XQueryX formatted
     * XPath query.
     * Config document is an optional parameter (may be set to NULL).
     */
    public Element xPathFactory(Element expression, ConfigurationDocumentType config) throws Exception {
        DataResourceAddressType addr = ClientToolkit.XPathExecuteFactory(serviceURL,resourceName,expression,null,null,config,null);
        return Serializer.objectToDOM(addr,DATA_RESOURCE_ADDRESS);
    }
    
    /**
     * Executes the XQuery factory operation.
     * The expression parameter must be a valid XQueryX formatted
     * XQuery query.
     * Config document is an optional parameter (may be set to NULL).
     */
    public Element xQueryFactory(Element expression, ConfigurationDocumentType config) throws Exception {
        DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(serviceURL,resourceName,expression,null,null,config,null);
        return Serializer.objectToDOM(addr,DATA_RESOURCE_ADDRESS);
    }
    
    /*
     * Utility method to convert a list of data resource addresses
     * to XML elements
     */
    private static Element[] dataResourceAddressesToElements(DataResourceAddressType[] addr) throws Exception {
        Element[] elem = new Element[addr.length];
        for(int i=0;i<addr.length;i++)
            elem[i] = Serializer.objectToDOM(addr[i],DATA_RESOURCE_ADDRESS);
        return elem;
    }

}
