/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import junit.framework.TestCase;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentResponseWrapperResponse;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.CollectionAlreadyExistsFaultType;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentResponseWrapperResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType;
import uk.org.ogsadai.wsdaix.client.ClientToolkit;
import uk.org.ogsadai.wsdaix.test.properties.TestDocuments;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the CollectionAccess port type.
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 * 
 * TODO Schema addition/removal tests to be defined
 *
 */
public class XMLCollectionAccessTest extends TestCase {

    private static Logger mLog = Logger.getLogger(XMLCollectionAccessTest.class);
    
    /*
     * URL of the service to be tested. The assumption is made
     * that all XMLCollectionAccess operations can be tested 
     * through the same service.
     */
    private String serviceURL;
    
    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;
    
    private URI invalidResourceName;
    private URI invalidSubcollectionURI,epccSubcollectionURI;
    private URI ibmSubcollectionURI,almadenSubcollectionURI;
    
    /**
     * Set up the test suite
     */
    public XMLCollectionAccessTest() throws Exception {
        setUp();
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public XMLCollectionAccessTest(String name) {
        super(name);
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(XMLCollectionAccessTest.class);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        serviceURL =  p.getProperty(TestProperties.COLLECTION_ACCESS_TEST_ENDPOINT);
        resourceName = new URI("daix:testresource");
        invalidResourceName = new URI("daix:invalidresource");
        ibmSubcollectionURI = new URI("daix:/db/contacts/ibm");
        almadenSubcollectionURI = new URI("daix:/db/contacts/ibm/almaden");
        epccSubcollectionURI = new URI("daix:/db/contacts/epcc");
        invalidSubcollectionURI = new URI("daix:/db/contacts/invalidcollection");
    }
    
    public void tearDown() {
        
    }
    
    /**
     * Tests the operation GetCollectionPropertyDocument.
     * The test is passed if the correct number of documents
     * and sub-collections exists. This may be different depending
     * on whether the resource supports nested collections or not.
     * If nested collections are supported, the test checks that
     * 5 documents exist and 2 sub-collections exist.
     * If nested collections are unsupported, the test checks that
     * 1 document exists.
     * Whether or not nested collections are supported, the test 
     * checks that the root collection contains one document  
     * named "Sugden" 
     */
    public void testGetCollectionPropertyDocument() {
        try {
            XMLCollectionPropertyDocumentType p = (XMLCollectionPropertyDocumentType) ClientToolkit.getCollectionPropertyDocument(serviceURL,resourceName);
            if ( p.isSupportsCollectionNesting() ) {
                assertEquals("Number of documents", 5, p.getNumberOfDocuments() );
                assertEquals("Number of documents in root collection",1,p.getTopLevelCollection().getDocument().length);
                assertEquals("Document name", "Sugden", p.getTopLevelCollection().getDocument(0).getName() );
                assertEquals( "Number of sub-collections ", 2, p.getTopLevelCollection().getCollection().length );
            } else {
                assertEquals("Number of documents", 1, p.getNumberOfDocuments() );
                assertEquals("Document name", "Sugden", p.getTopLevelCollection().getDocument(0).getName() );
            }
        } catch(Exception e) {
            fail(e.toString());
        }
    }

    /**
     * Tests the AddDocuments operation.
     * Attempts to add two documents: "Sugden" and "Jackson".
     * Checks that "Jackson" is added successfully and that
     * the other document is not added because it already exists. 
     */
    public void testAddDocuments() {
        try {
            AddDocumentsResponse r = ClientToolkit.addDocuments(serviceURL,resourceName,new String[] { "Jackson", "Sugden" }, new Document[] {TestDocuments.jackson,TestDocuments.sugden} );
            assertEquals("Number of response wrappers", 2, r.getAddDocumentResponseWrapper().length );
            String jacksonResponse = null,sugdenResponse = null;
            for(int i=0;i<r.getAddDocumentResponseWrapper().length;i++) {
                if ( r.getAddDocumentResponseWrapper()[i].getDocumentName().equals("Jackson") )
                    jacksonResponse = r.getAddDocumentResponseWrapper()[i].getResponse().toString();
                if ( r.getAddDocumentResponseWrapper()[i].getDocumentName().equals("Sugden") )
                    sugdenResponse = r.getAddDocumentResponseWrapper()[i].getResponse().toString();
            }
            assertEquals("Add Jackson response",AddDocumentResponseWrapperResponse._value1.toString(),jacksonResponse);
            assertEquals("Add Sugden response",AddDocumentResponseWrapperResponse._value4.toString(),sugdenResponse);
        } catch (Exception e ) {
            fail(e.toString());    
        }
    }
    
    /**
     * Tests the GetDocuments operation.
     * Checks that the "Sugden" document is successfully retrieved.
     */
    public void testGetDocuments() {
        try {
            GetDocumentsResponse r = ClientToolkit.getDocuments(serviceURL,resourceName,new String[] { "Sugden" } );
            assertEquals("Number of response wrappers", 1, r.getGetDocumentResponseWrapper().length );
            //check the "Sugden" attribute of the document element
            String s = ((Document) r.getGetDocumentResponseWrapper(0).getData().get_any()[0].getAsDocument() ).getDocumentElement().getAttribute(("id"));
            assertEquals("Document id", "Sugden", s);
        } catch (Exception e ) {
            e.printStackTrace();
            fail(e.toString());    
        }
    }
    
    /**
     * Tests the CreateSubcollection operation.
     * Creates a subcollection named "Almaden" in the "IBM"
     * collection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testCreateSubcollection() {
        try {
            ClientToolkit.createCollection(serviceURL,resourceName,ibmSubcollectionURI,almadenSubcollectionURI);
        } catch (Exception e ) {
            fail(e.getClass().getName());
        }
    }
    
    /**
     * Tests the CreateSubcollection operation.
     * Checks that a CollectionAlreadyExistsFault is generated
     * when attempt is made to create the "EPCC" subcollection,
     * which already exists within the root collection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testCreateSubcollection_CollectionAlreadyExistsFault() {
        try {
            ClientToolkit.createCollection(serviceURL,resourceName,epccSubcollectionURI);
            fail("Invalid subcollection created");
        } catch ( CollectionAlreadyExistsFaultType f) {
             
        } catch (Exception e ) {
            fail(e.toString());    
        }
    }
    
    /**
     * Tests CreateSubcollection operation.
     * Tests that a InvalidCollectionNameFault is generated when
     * an attempt to create a subcollection inside a non-existent
     * collection is made.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testCreateSubcollection_InvalidCollectionNameFault() {
        try {
            ClientToolkit.createCollection(serviceURL,resourceName,invalidSubcollectionURI,invalidSubcollectionURI);
            fail("Invalid subcollection URI accepted");
        } catch ( InvalidCollectionNameFaultType f) {
             
        } catch (Exception e ) {
            fail(e.toString());    
        }
    }
    
    /**
     * Tests the AddDocuments operation.
     * Adds the document "Krause" to the "Almaden" collection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testAddDocumentsToSubcollection() {
        try {
            AddDocumentsResponse r = ClientToolkit.addDocuments(serviceURL,resourceName,almadenSubcollectionURI,new String[] {"Krause"},new Document[] {TestDocuments.krause});
            assertEquals("Number of response wrappers", 1, r.getAddDocumentResponseWrapper().length );
            assertEquals("Add Krause response",AddDocumentResponseWrapperResponse._value1.toString(),r.getAddDocumentResponseWrapper()[0].getResponse().toString());
        } catch (Exception e ) {
            fail(e.getClass().getName());    
        }
    }
    
    /**
     * Tests the RemoveDocuments operation.
     * Removes the "Jackson" document from the root collection
     * and checks that a response indicating success if received.
     */
    public void testRemoveDocuments() {
        try {
            RemoveDocumentsResponse r = ClientToolkit.removeDocuments(serviceURL,resourceName,new String[] {"Jackson"});
            assertEquals( "Number of response wrappers", 1, r.getRemoveDocumentResponseWrapper().length );
            assertEquals( "Remove Jackson response", RemoveDocumentResponseWrapperResponse.value1.toString(), r.getRemoveDocumentResponseWrapper(0).getResponse().getValue().toString() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the RemoveDocuments operation.
     * Tests that a DocumentDoesNotExistFault is generated
     * when an attempt is made to remove the document named
     * "Lynden", which doesn't exist.
     */
    public void testRemoveDocuments_DocumentDoesNotExist() {
        try {
            RemoveDocumentsResponse r = ClientToolkit.removeDocuments(serviceURL,resourceName,new String[] {"Lynden"});
            assertEquals( "Number of response wrappers", 1, r.getRemoveDocumentResponseWrapper().length );
            assertEquals( "Remove Lynden response", RemoveDocumentResponseWrapperResponse.value3.toString(), r.getRemoveDocumentResponseWrapper(0).getResponse().getValue().toString() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the RemoveDocuments operation.
     * Removes the document named "Jackson" from the "Almaden" collection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testRemoveDocumentsFromSubcollection() {
        try {
            RemoveDocumentsResponse r = ClientToolkit.removeDocuments(serviceURL,resourceName,almadenSubcollectionURI,new String[] {"Jackson"});
            assertEquals( "Number of response wrappers", 1, r.getRemoveDocumentResponseWrapper().length );
            assertEquals( "Remove Jackson response", RemoveDocumentResponseWrapperResponse.value3.toString(), r.getRemoveDocumentResponseWrapper(0).getResponse().getValue().toString() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    } 
    
    /**
     * Tests the RemoveDocuments operation.
     * Checks that an InvalidCollectionNameFault is generated when
     * an attempt is made to remove documents from a collection that
     * doesn't exist.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testRemoveDocumentsFromSubcollection_InvalidCollectionNameFault() {
        try {
            RemoveDocumentsResponse r = ClientToolkit.removeDocuments(serviceURL,resourceName,invalidSubcollectionURI,new String[] {"Jackson"});
            fail("Allowed the removal of an invalid collection");
        } catch ( InvalidCollectionNameFaultType f) {
        
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    } 
    
    /**
     * Tests the RemoveSubcollection operation.
     * Removes the "IBM" collection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testRemoveSubcollection() {
        try {
            ClientToolkit.removeCollection(serviceURL,resourceName,ibmSubcollectionURI,almadenSubcollectionURI);
        } catch ( Exception e ) {
            fail(e.getClass().getName());
        }
    }
    
    /**
     * Tests the RemoveSubcollection operation.
     * Checks that an InvalidCollectionNameFault is generated
     * when an attempt is made to remove from a collection that
     * doesn't exist.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testRemoveSubcollection_InvalidCollectionNameFault() {
        try {
            ClientToolkit.removeCollection(serviceURL,resourceName,invalidSubcollectionURI,almadenSubcollectionURI);
            fail("Allowed removal of collection from invalid subcollection");
        } catch ( InvalidCollectionNameFaultType f ) {
        
        } catch ( Exception e ) {
            fail(e.getClass().getName());
        }
    }

    /**
     * Tests the RemoveSubcollection operation.
     * Checks that an InvalidCollectionNameFault is generated
     * when an attempt is made to remove a subcollection that
     * doesn't exist.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testRemoveSubcollection_CollectionDoesNotExistFault() {
        try {
            ClientToolkit.removeCollection(serviceURL,resourceName,invalidSubcollectionURI);            
            fail("Allowed removal of non-existent collection");
        } catch ( InvalidCollectionNameFaultType f ) {
        
        } catch ( Exception e ) {
            fail(e.getClass().getName());
        }
    }
    
    public void addSchema() {
        
    }
    
    public void removeSchema() {
     
    }
    
    public void getSchema() {
        
    }
    
    
}
