/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;
import junit.framework.*;
import uk.org.ogsadai.dais.xml.service.core.*;
import uk.org.ogsadai.dais.xml.services.daix.*;
import uk.org.ogsadai.wsdaix.client.*;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the CollectionFactory port type.
 * Also contains a test for the Destroy operation of the 
 * CoreDataAccess port type. This is tested here because factory
 * operations are required in order to created a resource which
 * can subsequently be sucessfully destroyed to test this operation.
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 *
 */
public class XMLCollectionFactoryTest extends TestCase {

    private static Logger mLog = Logger.getLogger(XMLCollectionFactoryTest.class);
    
    /*
     * URL of the service to be tested. The assumption is made
     * that all XMLCollectionFactory operations can be tested
     * through the same service.
     */
    private String serviceURL;
    
    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;

    /*
     * The URL of the service through which CoreDataAccess
     * operations are tested.
     */
    private String collectionTestCoreDataAccessURL;

    /*
     * The URI of the "EPCC" collection.
     */
    private URI epccSubcollectionURI;
    
    /*
     * The URI of a collection that does not exist.
     */
    private URI invalidSubcollectionURI;
    
    /**
     * Set up the test suite
     */
    public XMLCollectionFactoryTest() throws Exception {
        setUp();
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public XMLCollectionFactoryTest(String name) {
        super(name);
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(XMLCollectionFactoryTest.class);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        serviceURL =  p.getProperty(TestProperties.COLLECTION_FACTORY_TEST_ENDPOINT);
        collectionTestCoreDataAccessURL = p.getProperty(TestProperties.COLLECTION_FACTORY_TEST_CORE_DATA_ACCESS_ENDPOINT);
        resourceName = new URI("daix:testresource");
        epccSubcollectionURI = new URI("daix:/db/contacts/epcc");
        invalidSubcollectionURI = new URI("daix:invalidcollectionuri");
    }
    
    public void tearDown() {
        
    }
    
    /**
     * Tests the DocumentSelectionFactory operation
     * Selects the document named "Sugden" and checks that
     * a data resource address is received in response.
     */
    public void testDocumentSelectionFactory() {
        try {
            DataResourceAddressType addr = ClientToolkit.documentSelectionFactory(serviceURL,resourceName,"Sugden");
        } catch (Exception e) {
            fail(e.getClass().getName());
            
        }
    }
    
    /**
     * Tests the DocumentSelectionFactory operation.
     * Selects the document named "Hume" from the "EPCC" collection
     * and checks that a data resource address is received in response.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testDocumentSelectionFactoryFromSubcollection() {
        try {
            ClientToolkit.documentSelectionFactory(serviceURL,resourceName,epccSubcollectionURI,"Hume");
        } catch (Exception e) {
            fail(e.getClass().getName());
            
        }
    }
    
    /**
     * Tests the DocumentSelectionFactory operation.
     * Checks that a DocumentDoesNotExistFault is generated when an attempt
     * is made to remove the document named "Sugden" from the "EPCC" collection.
     * "Sugden" exists in the root collection only.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testDocumentSelectionFactoryFromSubcollection_DocumentDoesNotExistFault() {
        try {
            ClientToolkit.documentSelectionFactory(serviceURL,resourceName,epccSubcollectionURI,"Sugden");
            fail("Allowed selection of invalid resource");
        } catch ( DocumentDoesNotExistFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
    }
    
    /**
     * Tests the DocumentSelectionFactory operation.
     * Checks that a DocumentDoesNotExistFault is generated when an attempt
     * is made to remove a document named "Hume", which does not exist.
     */
    public void testDocumentSelectionFactory_DocumentDoesNotExistFault() {
        try {
            ClientToolkit.documentSelectionFactory(serviceURL,resourceName,"Hume");
            fail("Allowed selection of invalid resource");
        } catch ( DocumentDoesNotExistFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
    }
    
    /**
     * Tests the DocumentSelectionFactory operation.
     * Checks that a DocumentDoesNotExistFault is generated when an attempt
     * is made to remove a document named "Hume", which does not exist.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testDocumentSelectionFactory_InvalidCollectionNameFault() {
        try {
            ClientToolkit.documentSelectionFactory(serviceURL,resourceName,invalidSubcollectionURI,"Sugden");
            fail("Allowed selection of invalid resource");
        } catch ( InvalidCollectionNameFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
    }

    /**
     * Tests the CollectionSelectionFactory operation.
     * Selects the root collection and checks that
     * a data resource address is received in response.
     */
    public void testCollectionSelectionFactory() {
        try {
            DataResourceAddressType addr = ClientToolkit.collectionSelectionFactory(serviceURL,resourceName);
        } catch (Exception e) {
            fail(e.getClass().getName());            
        }
    }
    
    /**
     * Tests the CollectionSelectionFactory operation.
     * Selects the "EPCC" collection and checks that
     * a data resource address is received in response.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testCollectionSelectionFactoryWithSubcollectionParameter() {
        try {
            ClientToolkit.collectionSelectionFactory(serviceURL,resourceName,epccSubcollectionURI);            
        } catch (Exception e) {
            fail(e.getClass().getName());            
        }
    }
    
    /**
     * Tests the CollectionSelectionFactory operation.
     * Checks that an InvalidCollectionNameFault is generated
     * when an attempt is made to select an invalid subcollection.
     * Note: This test applies to resources supporting nested
     * collections only.
     */
    public void testCollectionSelectionFactory_InvalidCollectionNameFault() {
        try {
            ClientToolkit.collectionSelectionFactory(serviceURL,resourceName,invalidSubcollectionURI);
            fail("Allowed selection of invalid resource");
        } catch ( InvalidCollectionNameFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
    }
    
    /**
     * Primary objective is to test the Destroy operation.
     * DocumentSelectionFactory and GetDataResourcePropertyDocument
     * are tested, but they also tested elsewhere in this test suite
     * and the CoreDataAccess tests respectively.  
     * 
     * Destroy is tested here, and not
     * with the CoreDataAccess operations, as might be expected, 
     * because a service managed resource must be created in order
     * to be able to have something to destroy. The DocumentSelectionFactory
     * should be supported by most implementation as it does not required
     * nested collections of query interfaces. If the factory operation
     * is not supported, an alternative test must be implemented to
     * test the Destroy operation. 
     * 
     * Creates a new resource to represent the "Sugden" document and
     * destroys it. Checks that the resource has been created by accessing
     * its property document. 
     * 
     * Checks that the resource has been destroyed by
     * attempting to retrieve its property document.
     * 
     * Note: this test assumes that the created resource is accessible through
     * the service URL used to test the CoreDataAccess port type. If this is not
     * the case then the test must be modified to take this into account. 
     */
    public void testDestroy() {
        URI newResource = null;
        try {
            DataResourceAddressType addr = ClientToolkit.documentSelectionFactory(serviceURL,resourceName,"Sugden");
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            newResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
        //access the created resource
        try {
            ClientToolkit.getDataResourcePropertyDocument(collectionTestCoreDataAccessURL,newResource);
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
        //destroy the resource
        try {
            ClientToolkit.destroyDataResource(collectionTestCoreDataAccessURL,newResource);   
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
        //check that the resource has been destroyed
        try {
            ClientToolkit.getDataResourcePropertyDocument(collectionTestCoreDataAccessURL,newResource);
            fail("Able to access data resource that should have been destroyed");
        } catch ( InvalidResourceNameFaultType f ) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());
        }
    }
    
}
