/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import javax.xml.namespace.QName;
import junit.framework.TestCase;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.GetItemsResponse;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCountFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidStartPositionFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XMLSequencePropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteResponse;
import uk.org.ogsadai.wsdaix.client.ClientToolkit;
import uk.org.ogsadai.wsdaix.test.properties.TestDocuments;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the XQuery port types. Tests both 
 * factory and non-factory operations.
 * 
 * Also tests the XMLSequence port type. It is assumed that
 * the factory operations expose results (by default) through
 * this port type. If this is not the case the tests must be 
 * modified.
 * 
 * All query operations are tested assuming that the OGSA-DAI
 * resourceset dataset format is used to return results. If other
 * dataset formats are used by default or this format is not 
 * supported the tests must be modified accordingly.
 * 
 * The same XQuery query, formulated using XQueryX, is used in
 * all cases. See client/systemtest/xqueryx-sugden.xml for the
 * query. 
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 *
 */
public class XQueryTest extends TestCase {

    private static Logger mLog = Logger.getLogger(XQueryTest.class);
    
    /* 
     * The assumption is made that all factory
     * operations can be tested through the xqueryAccessServiceURL
     * and that all non-factory operations can be tested through
     * the xqueryFactoryServiceURL.
     */
    private String xqueryAccessServiceURL,xqueryFactoryServiceURL;

    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;
    
    /*
     * Abstract name of a resource that does not exist
     */
    private URI invalidResourceName;
    
    /*
     * URI of the OGSA-DAI resourceset dataset format.
     */
    private URI datasetFormatURI;
    
    /*
     * A URI that is not supported as a dataset format
     * or language URI. 
     */
    private URI invalidURI;
    
    /*
     * QNames of the XMLSequence port type, which can be
     * used to access the results of queries. The invalid
     * QName is used to test InvalidPortTypeQNameFault
     * generation.
     */
    private QName xmlSequencePT,invalidPTQName;
    
    /*
     * XQueryX URI, used as the language URI for XPath/XQuery
     * operations.
     */
    private URI xQueryXURI;

    /**
     * Set up the test suite
     */
    public XQueryTest() throws Exception {
        setUp();
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public XQueryTest(String name) {
        super(name);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        xqueryAccessServiceURL =  p.getProperty(TestProperties.XQUERY_ACCESS_TEST_ENDPOINT);
        xqueryFactoryServiceURL = p.getProperty(TestProperties.XQUERY_FACTORY_TEST_ENDPOINT);
        resourceName = new URI("daix:testresource");
        datasetFormatURI = new URI("ogsadai:resourceset");
        invalidURI = new URI("invalid:invalid");
        xmlSequencePT = new QName("http://www.ggf.org/namespaces/2005/12/WS-DAIX","XMLSequenceAccessPT");
        invalidPTQName = new QName("http://invalid.org.uk","invalid");
        xQueryXURI = new URI("http://www.w3.org/2005/XQueryX");
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(XQueryTest.class);
    }
  
    /**
     * Tests the XQueryExecute operation.
     * Submits the XQuery query and checks the correct number of elements
     * are contained within the result set.
     */
    public void testXQuery() {
        try {
            XQueryExecuteResponse r = ClientToolkit.XQueryExecute(xqueryAccessServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            Element results = r.getDataset().getDatasetData().get_any()[0].getAsDOM();
            NodeList resources = results.getElementsByTagName("resource");
            assertEquals("Number of items in result set",1,resources.getLength());
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecute operation.
     * Specifies the dataset format URI.
     * Submits the XQuery query and checks the corrent number of elements
     * are contained within the result set.
     */
    public void testXQueryWithDatasetFormatURI() {
        try {
            XQueryExecuteResponse r = ClientToolkit.XQueryExecute(xqueryAccessServiceURL,resourceName,datasetFormatURI,TestDocuments.xQuerySelectSugdenQuery);
            Element results = r.getDataset().getDatasetData().get_any()[0].getAsDOM();
            NodeList resources = results.getElementsByTagName("resource");
            assertEquals("Number of items in result set",1,resources.getLength());
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecute operation.
     * Specifies an invalid dataset format URI and checks that
     * an InvalidDatasetFormatURIFault is generated.
     */
    public void testXQuery_InvalidDatasetFormatURIFault() {
        try {
            XQueryExecuteResponse r = ClientToolkit.XQueryExecute(xqueryAccessServiceURL,resourceName,invalidURI,TestDocuments.xQuerySelectSugdenQuery);
            fail("Accepted invalid dataset format URI");
        } catch ( InvalidDatasetFormatFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /*
     * Tests for factory operations 
     */
    
    /**
     * Tests the XQueryExecuteFactory operation.
     * Checks that a data resource address is received in response
     * when the operation is executed using the XQuery query. 
     */
    public void testXQueryFactory() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory operation.
     * Specifies the port type QName as that of the XMLSequence
     * port type. It is assumed that creation of resources with
     * this port type is supported.
     * Checks that a data resource address is received in response
     * when the operation is executed using the query. 
     */
    public void testXQueryFactoryWithPortTypeQName() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery,xmlSequencePT);
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory operation.
     * Specifies an invalid port type QName and checks that
     * an InvalidPortTypeQNameFault is generated. 
     */
    public void testXQueryFactory_InvalidPortTypeQNameFault() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery,invalidPTQName);
            fail("Factory operation succeeded with invalid port type QName");
        } catch (InvalidPortTypeQNameFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory operation.
     * Specifies the XQueryX URI as the expression URI.
     * It is assumed this expression format is supported.
     * Checks that a data resource address is received in response
     * when the operation is executed using the query. 
     */
    public void testXQueryFactoryWithExpressionURI() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery,xQueryXURI);
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory operation.
     * Specifies an invalid language URI.
     * Checks that an InvalidLanguageFault is generated. 
     */
    public void testXQueryFactory_InvalidLanguageFault() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery,invalidURI);
            fail("XQuery succeeded with invalid expression language URI");
        } catch (InvalidLanguageFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /*
     * The following tests test the XMLSequence port type in addition to
     * the XPath factory operations.
     */
    
    /**
     * Tests the XQueryExecuteFactory + GetXMLSequencePropertyDocument operations.
     * Creates a resource using the factory operation and then
     * checks that property document of the created resource
     * contains 1 item.
     */
    public void testGetXMLSequencePropertyDocument() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            XMLSequencePropertyDocumentType p = (XMLSequencePropertyDocumentType) ClientToolkit.getXMLSequencePropertyDocument(sURL,sResource);
            assertEquals( "Number of items", 1, p.getNumberOfItems() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory + GetItems operations.
     * Creates a resource using the factory operation and then
     * retrieves the first of the items maintained by the XMLSequence
     * resource.
     */
    public void testGetItems() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            GetItemsResponse r = ClientToolkit.getItems(sURL,sResource,0,1);
            Element results = r.getDataset().getDatasetData().get_any()[0].getAsDOM();
            NodeList resources = results.getElementsByTagName("resource");
            assertEquals("Number of items in result set",1,resources.getLength());
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory + GetItems operations.
     * Specifies the dataset format as the OGSA-DAI resource set
     * format URI.
     * Creates a resource using the factory operation and then
     * retrieves the first of the items maintained by the XMLSequence
     * resource.
     */
    public void testGetItemsWithDatasetFormatURI() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            GetItemsResponse r = ClientToolkit.getItems(sURL,sResource,this.datasetFormatURI,0,1);
            Element results = r.getDataset().getDatasetData().get_any()[0].getAsDOM();
            NodeList resources = results.getElementsByTagName("resource");
            assertEquals("Number of items in result set",1,resources.getLength());
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory + GetItems operations.
     * Creates a resource using the factory operation and checks
     * that an InvalidDatasetFormatURIFault is generated when executing
     * GetItems.
     */
    public void testGetItems_InvalidDatasetFormatURI() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            GetItemsResponse r = ClientToolkit.getItems(sURL,sResource,this.invalidURI,0,1);
            fail("GetItems succeeded with invalid dataset format URI");
        } catch (InvalidDatasetFormatFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory + GetItems operations.
     * Creates a resource using the factory operation and checks
     * that an InvalidStartPositionFault is generated when the
     * GetItems operation is executed, attempting to select 1 item
     * starting at position 2, where the should contain only 1 item.
     */
    public void testGetItems_InvalidStartPosition() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            GetItemsResponse r = ClientToolkit.getItems(sURL,sResource,2,1);
            fail("GetItems succeeded with invalid start position");
        } catch (InvalidStartPositionFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Tests the XQueryExecuteFactory + GetItems operations.
     * Creates a resource using the factory operation and checks
     * that an InvalidCountPositionFault is generated when the
     * GetItems operation is executed, attempting to select 5 items
     * starting at position 0, where the should contain only 1 item.
     */
    public void testGetItems_InvalidCountFault() {
        try {
            DataResourceAddressType addr = ClientToolkit.XQueryExecuteFactory(xqueryFactoryServiceURL,resourceName,TestDocuments.xQuerySelectSugdenQuery);
            NodeList n = addr.getReferenceParameters().get_any()[0].getAsDocument().getElementsByTagNameNS("http://www.ggf.org/namespaces/2005/12/WS-DAI","DataResourceAbstractName");
            URI sResource = new URI(n.item(0).getChildNodes().item(0).getNodeValue());
            String sURL = addr.getAddress().toString();
            GetItemsResponse r = ClientToolkit.getItems(sURL,sResource,0,5);
            fail("GetItems succeeded with invalid count parameter");
        } catch (InvalidCountFaultType f) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    
    
}
