/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import junit.framework.*;
import uk.org.ogsadai.dais.xml.service.core.*;
import uk.org.ogsadai.wsdaix.client.*;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;

import uk.org.ogsadai.wsdaix.test.properties.TestDocuments;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;
import uk.org.ogsadai.dais.xml.services.daix.*;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the XUpdate port type.
 * 
 * The same XUpdate expression is used in all test cases -
 * see client/systemtest/xupdate-sugden.xml
 * 
 * It is assumed that the XUpdate language URI is supported
 * by the tested data resource.
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 *
 */
public class XUpdateTest extends TestCase {

    private static Logger mLog = Logger.getLogger(XUpdateTest.class);
    
    /*
     * URL of the service to be tested
     */
    private String serviceURL;
    
    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;
    
    /*
     * A URI that is not supported as a dataset format
     * or language URI. 
     */
    URI invalidURI;
    
    /*
     * XUpdate URI used as a valid language URI parameter
     */
    URI xUpdateURI;
    
    /**
     * Set up the test suite
     */
    public XUpdateTest() throws Exception {
        setUp();   
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public XUpdateTest(String name) {
        super(name);   
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(XUpdateTest.class);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        serviceURL =  p.getProperty(TestProperties.XUPDATE_TEST_ENDPOINT);
        resourceName = new URI("daix:testresource");
        invalidURI = new URI("invalid:invalid");
        xUpdateURI = new URI("http://www.xmldb.org/xupdate");
    }
    
    public void tearDown() {
    
    }
    
    /**
     * Test the XUpdateExecute operation.
     * Checks that a response is received confirming that one item
     * has been updated.
     */
    public void testXUpdate() {
        try {
            XUpdateExecuteResponse r = ClientToolkit.XUpdateExecute(serviceURL,resourceName,TestDocuments.xUpdateSugden);
            assertEquals("Updated items", 1, r.getCount().intValue() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Test the XUpdateExecute operation.
     * Specifies the XUpdate language URI.
     * Checks that a response is received confirming that one item
     * has been updated.
     */
    public void testXUpdateWithLangaugeURI() {
        try {
            XUpdateExecuteResponse r = ClientToolkit.XUpdateExecute(serviceURL,resourceName,TestDocuments.xUpdateSugden,xUpdateURI);
            assertEquals("Updated items", 1, r.getCount().intValue() );
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    /**
     * Test the XUpdateExecute operation.
     * Specifies an invalid XUpdate language URI and checks that
     * an InvalidLanguageFault is generated.
     */
    public void testXUpdate_InvalidLangaugeFault() {
        try {
            XUpdateExecuteResponse r = ClientToolkit.XUpdateExecute(serviceURL,resourceName,TestDocuments.xUpdateSugden,invalidURI);
            fail("XUpdate operation succeeded with invalid expression langauge URI");
        } catch ( InvalidLanguageFaultType f ) {
            
        } catch (Exception e) {
            fail(e.getClass().getName());   
        }
    }
    
    
}
