//(c) International Business Machines Corporation, 2002 - 2005. 
//(c) University of Edinburgh, 2002 - 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.dataresource;

import java.io.File;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.XMLDBException;

import uk.org.ogsadai.common.files.DAIFileContentError;
import uk.org.ogsadai.common.properties.DynamicProperty;
import uk.org.ogsadai.common.properties.DynamicPropertyCreationException;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.rolemap.DatabaseRole;
import uk.org.ogsadai.common.rolemap.RoleMapAuthorizationException;
import uk.org.ogsadai.common.rolemap.RoleMapCannotAuthorizeException;
import uk.org.ogsadai.common.rolemap.RoleMapConfigurationException;
import uk.org.ogsadai.common.rolemap.RoleMapperCreationException;
import uk.org.ogsadai.common.rolemap.RoleMapper;
import uk.org.ogsadai.common.xml.XMLParseException;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dataresource.DataResourceAccessorConfigException;
import uk.org.ogsadai.dataresource.DataResourceAccessorMetaDataException;
import uk.org.ogsadai.dataresource.DataResourceAccessorSetupException;
import uk.org.ogsadai.dataresource.MetaDataExtractor;
import uk.org.ogsadai.dataresource.XMLDBMetaDataExtractor;
import uk.org.ogsadai.dataresource.XMLDBCloseCollectionException;
import uk.org.ogsadai.dataresource.XMLDBCollectionAccessException;
import uk.org.ogsadai.dataresource.XMLDBCollectionUseException;
import uk.org.ogsadai.dataresource.XMLDBUnknownCollectionException;
import uk.org.ogsadai.dataresource.XMLDBUnknownSubCollectionException;
import uk.org.ogsadai.dataresource.XMLDBRegistrationException;
import uk.org.ogsadai.dataresource.config.MetaData;
import uk.org.ogsadai.dataresource.config.ProductInfo;
import uk.org.ogsadai.dataresource.config.RoleMapperSpec;
import uk.org.ogsadai.exception.AuthorizationExecutionException;
import uk.org.ogsadai.exception.DAIClassCreateException;
import uk.org.ogsadai.exception.DAIClassMissingInterfaceException;
import uk.org.ogsadai.exception.DAIClassNotFoundException;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.common.msgs.DAILogger;

/**
 * A data resource accessor that provides access to XMLDB data
 * resources.
 * <p>
 * The configuration file is named <code>dataResourceConfig.xml</code> 
 * and is assumed to reside in the configuration files directory for
 * the data service resource that owns this data resource accessor.
 * 
 * @author The OGSA-DAI Project Team
 */
public class XMLDBDataResourceAccessor 
    implements DataResourceAccessor, PersistInFiles, XMLDBCollectionProvider
{
    // Copyright statement.
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) University of Edinburgh 2002 - 2005.";
  
    // Logger object for logging in this class
    private static DAILogger LOG = 
        DAILogger.getLogger(XMLDBDataResourceAccessor.class);
  
    // Data resource configuration file name.
    private static final String MAIN_CONFIG_FILE_NAME =
        "dataResourceConfig.xml";

    // Data service resource name
    protected String mResourceName;
    // Data resource accessor meta data.
    protected Property[] mProperties;
    // Role mapper for database access authorization.
    protected RoleMapper mRoleMapper;
    // Database driver class name.
    protected String mDriverClassName;
    // Database driver collection URI.
    protected String mCollectionURI;
    
    
    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#setResourceName(java.lang.String)
     */
    public void setResourceName(String resourceName)
    {
        if (resourceName == null)
        {
            throw new IllegalArgumentException("name must not be null");
        }
        mResourceName = resourceName;
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.PersistedDataResourceAccessor#restoreFromConfig(java.io.File)
     */
    public void restoreFromConfig(File directory) 
        throws 
            DataResourceAccessorConfigException, 
            DataResourceAccessorMetaDataException, 
            DataResourceAccessorSetupException, 
            IllegalArgumentException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Configuring XMLDBDataResourceAccessor");
        }
        if (directory == null)
        {
            throw new IllegalArgumentException("directory must not be null");
        }
        if (! directory.isDirectory())
        {
            throw new IllegalArgumentException("directory " + 
                                               directory.getAbsolutePath() +
                                               " must be a directory");
        }
        // Get data resource accessor configuration file as a DOM
        // object.
        Document configDocument = getConfigAsDOM(mResourceName, directory);
        // Extract meta data and convert to properties.
        Property productInfo = getProductInfo(configDocument);
        Property[] metaData = getMetaData(mResourceName, configDocument);
        Property collectionSchema = 
            getCollectionSchema(mResourceName, configDocument);
        mProperties = getProperties(productInfo, collectionSchema, metaData);
        // Get role mapper information.
        try 
        {
            mRoleMapper = getRoleMapper(configDocument);
        }
        catch (RoleMapperCreationException e) 
        {
            throw new DataResourceAccessorSetupException(mResourceName, e);
        }
        catch (RoleMapConfigurationException e) 
        {
            throw new DataResourceAccessorSetupException(mResourceName, e);
        }
        // Get driver class name and load driver
        try
        {
            mDriverClassName = getAndLoadDriverClassName(configDocument);
        } 
        catch (DAIException e)
        {
            throw new DataResourceAccessorSetupException(mResourceName, e);
        }
        // Get the collection URI
        mCollectionURI = getCollectionURI(configDocument);
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Finished configuring XMLDBDataResourceAccessor");
        }
    }
      
    /**
     * Reads the data resource accessor configuration file and
     * converts it to DOM.
     * 
     * @param name
     *     Data service resource name.
     * @param dir
     *     Data service resource configuration files directory.
     * @return DOM representation of data resource accessor
     * configuration file.
     * @throws DataResourceAccessorConfigException if there is an
     * error in reading the configuration file.
     */
    private Document getConfigAsDOM(String name, File dir) 
          throws DataResourceAccessorConfigException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Reading data resource accessor configuration file");
        }
        final boolean VALIDATE = true;
        File file = new File(dir, MAIN_CONFIG_FILE_NAME);
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Directory is: " + dir.getAbsolutePath());
            LOG.debug("File name is: " + file.getAbsolutePath());
        }
        Document result;
        try
        {
            // Read the file into a DOM object.
            result = XMLUtilities.xmlFileToDOM(file.getAbsolutePath(),
                                               VALIDATE);
        }
        catch (XMLParseException e) 
        {
            DAIFileContentError error = new DAIFileContentError(file, e);
            throw new DataResourceAccessorConfigException(name, error);
        }
        catch (DAIException e) 
        {
            throw new DataResourceAccessorConfigException(name, e);
        }     
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Read data resource accessor configuration file");
        }
        return result;
    }
    
    /**
     * Extract product meta data from the data resource accessor
     * configuration and convert to property.
     * 
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return product meta data property or <code>null</code> if none.
     */
    private Property getProductInfo(Document configDoc)
    {
        if (LOG.isDebugEnabled()) 
        {
            LOG.debug("Extracting product meta data");
        }
        Property productInfoProperty = null;
        Element elt = XMLUtilities.getElement(configDoc, "productInfo");
        if (elt != null)
        {
            ProductInfo productInfo = new ProductInfo();
            productInfo.configure(elt);
            productInfoProperty = productInfo.getProperty();
        }
        if (LOG.isDebugEnabled()) 
        {
            LOG.debug("Extracted product meta data");
        }
        return productInfoProperty;
    }

    /**
     * Extract meta data from the data resource accessor
     * configuration and convert to properties.
     * 
     * @param name
     *     Data service resource name.
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return meta data properties.
     * @throws DataResourceAccessorMetaDataException if an error
     * occurs when constructing data resource accessor meta data
     * properties. 
     */
    private Property[] getMetaData(String name, Document configDoc)
        throws DataResourceAccessorMetaDataException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracting meta data from configuration");
        }
        MetaData metaData = new MetaData();
        Element metaDataElement = XMLUtilities.getElement(configDoc, 
                                                          "metaData");
        try
        {
            metaData.configure(metaDataElement);
        } catch (DynamicPropertyCreationException e) {
            
            throw new DataResourceAccessorMetaDataException(name, e);
        }
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracted meta data");
        }
        return metaData.getProperties();
    }
   
    /**
     * Gets data resource accessor properties - formed from aggregation
     * of all meta data properties.
     *
     * @param productInfo
     *     Product meta data property.
     * @param collectionSchema
     *     Collection schema property.
     * @param metaData
     *     Other meta data properties.
     * @return data resource accessor properties.
     */
    private Property[] getProperties(Property productInfo, 
                                     Property collectionSchema,
                                     Property[] metaData)
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Constructing properties");
        }
        // Construct properties array.
        Property[] properties;
        int index = 0;
        int numProperties = metaData.length;
        if (productInfo != null)
        {
            numProperties++;
        }
        if (collectionSchema != null)
        {
            numProperties++;
        }
        properties = new Property[numProperties];
        if (productInfo != null)
        {
            properties[index++] = productInfo;
        }
        if (collectionSchema != null)
        {
            properties[index++] = collectionSchema;
        }
        for (int i = 0; i < metaData.length; ++i)
        {
            properties[index++] = metaData[i];
        }
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Constructed properties");
        }
        return properties;
    }

    /**
     * Extract collection schema meta data from the data 
     * resource accessor configuration and converts to a property.
     * <p>
     * Extracts a collection schema meta data extractor class from
     * the value of a <code>callback</code> attribute of
     * a <code>collectionSchema</code> child element of 
     * <code>metaData</code>. From this a dynamic property is
     * constructed. The dynamic property callback callsback in turn to 
     * this class.
     * 
     * @param name
     *     Data service resource name.
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return Collection schema meta data propert.
     * @throws DataResourceAccessorMetaDataException if an error
     * occurs when constructing data resource accessor meta data
     * properties. 
     */
    private Property getCollectionSchema(String name, Document configDoc)
        throws DataResourceAccessorMetaDataException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracting collectionSchema meta data from configuration");
        }
        // Find the callback attribute of the collectionSchema element
        String className =
            XMLUtilities.getAttributeValue(configDoc, "collectionSchema", 
                                           "callback");
        if (LOG.isDebugEnabled())
        {
            LOG.debug("collectionSchema meta data extactor class name is: " + 
                      className );
        }
        if (className == null)
        {
            return null;
        }
        DynamicProperty property = null;
        try
        {
            property = 
                DynamicProperty.createDynamicProperty(DataResourcePropertyNames.COLLECTION_SCHEMA, className);
        } catch (DynamicPropertyCreationException e) {
            
            throw new DataResourceAccessorMetaDataException(name, e);
        }
        if (property.getCallback() instanceof XMLDBMetaDataExtractor)
        {
            ((XMLDBMetaDataExtractor)property.getCallback()).setCollectionProvider(this);
        }
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracted collectionSchema meta data");
        }
        return property;
    }

    /**
     * Extracts the role mapper implementation class, creates an
     * instance of it and initialises it with a role map file.
     * <p>
     * The information is held within the <code>roleMap</code> element 
     * of a data resource accessor configuration file.
     *
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return role mapper.
     * @throws RoleMapperCreationException if the role mapper cannot
     * be created.
     * @throws RoleMapConfigurationException if the role mapper cannot
     * be configured with a role map file.
     */
    private RoleMapper getRoleMapper(Document configDoc) 
        throws RoleMapperCreationException,
               RoleMapConfigurationException 
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Creating and initialising role mapper");
        }
        // Get the roleMap element.
        NodeList elts = 
            configDoc.getDocumentElement().getElementsByTagName("roleMap");
        // Parse the element.
        RoleMapperSpec spec = new RoleMapperSpec();
        spec.configure((Element)(elts.item(0)));
        RoleMapper roleMapper;
        String className = spec.getImplementation();
        try
        {
            // Now create a role mapper.
            Class roleMapperClass = Class.forName(className);
            roleMapper = (RoleMapper)roleMapperClass.newInstance();
        }
        catch (ClassNotFoundException e) 
        {
            Exception ex = new DAIClassNotFoundException(className);
            throw new RoleMapperCreationException(ex);
        } 
        catch (InstantiationException e) 
        {
            Exception ex = new DAIClassCreateException(className, e);
            throw new RoleMapperCreationException(ex);
        } 
        catch (IllegalAccessException e) 
        {
            Exception ex = new DAIClassCreateException(className, e);
            throw new RoleMapperCreationException(ex);
        }
        catch (ClassCastException e)
        {
            Exception ex = new DAIClassMissingInterfaceException(className,
                                                                 RoleMapper.class.getName());
            throw new RoleMapperCreationException(ex);
        }
        roleMapper.setConfiguration(spec.getConfiguration());
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Created and initialised role mapper");
        }
        return roleMapper;
    }

    /**
     * Returns the name of the XMLDB driver class and loads the class
     * and registers it with the XMLDB database manager.
     * 
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return name of JDBC driver class.
     * @throws DAIClassNotFoundException if the driver class cannot be 
     * loaded.
     * @throws DAIClassCreateException if an instance of the driver
     * cannot be created.
     * @throws DAIClassMissingInterfaceException if the driver class
     * does not implement interfaces required by the XMLDB driver
     * manager. 
     * throws XMLDBRegistrationException if the driver instance cannot
     * be registered with the XMLDB driver manager.
     */
    private String getAndLoadDriverClassName(Document configDoc) 
        throws DAIClassNotFoundException, DAIClassCreateException,
               DAIClassMissingInterfaceException,
               XMLDBRegistrationException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracting and registering database driver class");
        }
        String className = null;
        try 
        {
            className = XMLUtilities.getAttributeValue(configDoc,
                                                       "driver",
                                                       "implementation");
            // Load the class
            Class c = Class.forName(className);            
            // Register the class
            Database db = (Database)c.newInstance();
            DatabaseManager.registerDatabase(db);
        } 
        catch (ClassNotFoundException e) 
        {
            throw new DAIClassNotFoundException(className);
        }
        catch (IllegalAccessException e)
        {
            throw new DAIClassCreateException(className);
        }
        catch (InstantiationException e)
        {
            throw new DAIClassCreateException(className);
        }
        catch (ClassCastException e)
        {
            throw new DAIClassMissingInterfaceException(className,
                                                        Database.class.getName());
        }
        catch (XMLDBException e)
        {
            throw new XMLDBRegistrationException(className, e);
        }
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracted and registered database driver class: " + 
                      className);
        }
        return className;
    }
    
    /**
     * Returns the URI of the database.
     * 
     * @param configDoc 
     *     Data resource accessor configuration document.
     * @return URI of database.
     */
    private String getCollectionURI(Document configDoc)
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracting database URI");
        }
        String collectionURI = XMLUtilities.getElementValue(configDoc, "uri" );
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Extracted database URI: " + collectionURI);
        }
        return collectionURI;
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResource#getProperties()
     */
    public Property[] getProperties()
    {
        return mProperties;
    }
    
    /* (non-Javadoc)
     *
     * @see uk.org.ogsadai.dataresource.XMLDBCollectionProvider#getBaseCollectionURI()
     */
    public String getBaseCollectionURI() 
    {
        return mCollectionURI;
    }

    /* (non-Javadoc)
     *
     * @see uk.org.ogsadai.dataresource.XMLDBCollectionProvider#getXMLDBCollection(java.lang.String)
     */
    public Collection getXMLDBCollection(String userCredentials) 
        throws DataResourceAuthorizationException, 
               XMLDBCollectionUseException,
               AuthorizationExecutionException
    {
        Collection result = null;
        try
        {
            result = getXMLDBCollection(null, userCredentials);
        }
        catch (XMLDBUnknownSubCollectionException e)
        {
            // Will not occur because there is no sub-collection 
            // specified.
        }
        return result;
    }

    /* (non-Javadoc)
     *
     * @see uk.org.ogsadai.dataresource.XMLDBCollectionProvider#getXMLDBCollection(java.lang.String, java.lang.String)
     */
    public Collection getXMLDBCollection(String subCollection, 
                                         String userCredentials) 
        throws DataResourceAuthorizationException, 
               XMLDBUnknownSubCollectionException,
               XMLDBCollectionUseException,
               AuthorizationExecutionException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Getting collection from XMLDBDataResourceAccessor");
        }
        String collectionURI = mCollectionURI;
        if (! (subCollection == null || subCollection.length() == 0))
        {
            collectionURI = collectionURI + "/" + subCollection;
        }
        DatabaseRole role = null;
        try 
        {
            role = (DatabaseRole)mRoleMapper.map(userCredentials,
                                                 mCollectionURI);
        }
        catch (RoleMapAuthorizationException e)
        {
            throw new DataResourceAuthorizationException(userCredentials, e);
        }
        catch (RoleMapCannotAuthorizeException e)
        {
            throw new AuthorizationExecutionException(e);
        }
        Collection collection = getCollection(collectionURI, 
                                              role.getUserId(), 
                                              role.getPassword());
        if (collection == null)
        {
            if ((subCollection == null) || (subCollection.length() == 0))
            {
                // Could not find the base collection.
                XMLDBUnknownCollectionException ex =
                    new XMLDBUnknownCollectionException(collectionURI, 
                                                        role.getUserId());
                throw new XMLDBCollectionUseException(mCollectionURI, ex);
            }
            else
            {
                // We were looking for a subcollection which did not exist. 
                throw new XMLDBUnknownSubCollectionException(subCollection);
            }
        }
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Have collection from XMLDBDataResourceAccessor");
        }
        return collection;
    }

    /* (non-Javadoc)
     *
     * @see uk.org.ogsadai.dataresource.XMLDBCollectionProvider#releaseXMLDBCollection(org.xmldb.api.base.Collection)
     */
    public void releaseXMLDBCollection(Collection collection) 
        throws XMLDBCollectionUseException 
    {
        try 
        {
            if ((collection != null) && (collection.isOpen()))
            {
                collection.close();
            }
        } 
        catch (XMLDBException e) 
        {
            Exception ex = new XMLDBCloseCollectionException(e);
            throw new XMLDBCollectionUseException(mCollectionURI, ex);
        }
    }
    
    /**
     * Obtains a collection from the database.
     * 
     * @param collectionURI 
     *     URI of the database collection.
     * @param user          
     *     User name used to access database
     * @param password      
     *     Password used to access database.
     * @return the collection or <code>null</code> if there is no
     * such collection.
     * @throws XMLDBCollectionUseException if the collection cannot
     * be accessed.  
     */ 
    protected Collection getCollection(String collectionURI, String user,
                                       String password)
        throws XMLDBCollectionUseException
    {
        Collection collection = null;
        try
        {
            collection = DatabaseManager.getCollection(collectionURI, user, 
                                                       password); 
        }
        catch (XMLDBException e) 
        {
            XMLDBCollectionAccessException ex = 
                new XMLDBCollectionAccessException(collectionURI, user, e);
            throw new XMLDBCollectionUseException(collectionURI, ex);
        } 
        return collection;
    }
}
