// (c) International Business Machines Corporation, 2005.
// (c) University of Edinburgh 2005-2006.
// See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.service.resource;

import org.w3c.dom.Document;

import uk.org.ogsadai.activity.ActivityExecutionException;
import uk.org.ogsadai.activity.ActivityFactory;
import uk.org.ogsadai.activity.ActivityFactoryImpl;
import uk.org.ogsadai.activity.ActivityRequestFactory;
import uk.org.ogsadai.activity.ResponseDocumentBuilder;
import uk.org.ogsadai.common.properties.Properties;
import uk.org.ogsadai.common.properties.PropertiesImpl;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.engine.ConcurrencySettings;
import uk.org.ogsadai.engine.Engine;
import uk.org.ogsadai.engine.EngineConfiguration;
import uk.org.ogsadai.engine.EngineConfigurationImpl;
import uk.org.ogsadai.engine.Request;
import uk.org.ogsadai.engine.RequestContext;
import uk.org.ogsadai.engine.RequestAuthorizationException;
import uk.org.ogsadai.engine.RequestExecutionException;
import uk.org.ogsadai.engine.RequestSpecificationException;
import uk.org.ogsadai.engine.RequestEngine;
import uk.org.ogsadai.engine.RequestFactory;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.common.msgs.DAILogger;
import uk.org.ogsadai.service.authorization.AccessAuthorizer;
import uk.org.ogsadai.service.authorization.ActivityAccessAuthorizationException;
import uk.org.ogsadai.service.resource.ResourceCreationException;
import uk.org.ogsadai.service.resource.ResourceRequestAuthorizationException;
import uk.org.ogsadai.service.resource.ResourceRequestExecutionException;
import uk.org.ogsadai.service.resource.ResourceRequestSpecificationException;
import uk.org.ogsadai.service.resource.config.ActivitiesConfig;
import uk.org.ogsadai.sessions.SessionComponentFactory;
import uk.org.ogsadai.sessions.SessionCoordinator;
import uk.org.ogsadai.sessions.SessionLookupException;
import uk.org.ogsadai.sessions.SessionNotFoundException;
import uk.org.ogsadai.sessions.SessionStreamClosedException;
import uk.org.ogsadai.sessions.SessionStreamID;
import uk.org.ogsadai.sessions.SessionStreamIOException;
import uk.org.ogsadai.sessions.SessionStreamManager;
import uk.org.ogsadai.sessions.SessionStreamNotFoundException;
import uk.org.ogsadai.sessions.SessionStreamRequestNotProcessingException;
import uk.org.ogsadai.sessions.SimpleSessionStreamManager;

/**
 * A data service resource class.
 * <p>
 * Manages the interaction between the presentation (service) layer
 * and the core (engine) layer in OGSA-DAI.
 * <p>
 * A data service resource includes a data resource accessor (used
 * to access any underlying data resource)  and an OGSA-DAI engine
 * (as represented by a <code>uk.org.ogsadai.engine.engine</code>).
 *
 * @author The OGSA-DAI Project Team
 */
public class DataServiceResource 
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE = 
        "(c) University of Edinburgh 2005-2006.";
    
    /** Logger object for logging in this class */
    private static DAILogger LOG = 
        DAILogger.getLogger(DataServiceResource.class);
    
    /** Data service resource ID */
    private Object mID = null;
    /** Data service resource properties */
    private Properties mProperties;
    /** Session stream manager */
    private final SessionStreamManager mSessionStreamManager;
    /** Factory for creating activities supported by the resource */
    private final ActivityFactory mActivityFactory;
    /** Manages access to data resource for the data service resource */
    private final DataResourceAccessor mDataResourceAccessor;
    /** Manages joining/releasing requests to/from sessions */
    private final SessionCoordinator mSessionCoordinator;
    /** Factory to create requests from perform documents */
    private final RequestFactory mRequestFactory;
    /** Request execution engine */
    private Engine mEngine;    
    /** Request execution engine configuration */
    private EngineConfiguration mEngineConfiguration;
    
	public DataResourceAccessor getDataResourceAccessor() 
	{
		return this.mDataResourceAccessor;   
	}

    /**
     * Create a new data service resource.
     * 
     * @param id
     *     Resource ID.
     * @param activitiesConfig
     *     Details of the activities associated with the data service
     *     resource.
     * @param dataResourceAccessor
     *     The data resource acccessor that gives access to any underlying
     *     data resource.
     * @param sessionComponentFactory
     *     A factory to create the session components to be used by the
     *     data service resource.
     * @param concurrencySettings 
     *     Default concurrency settings for the resource.
     * @param accessAuthorizer 
     *     Grants or denies access to activites.
     * @throws IllegalArgumentException
     *     If any of the arguments is <code>null</code> or the id "".
     * @throws ResourceCreationException
     *     If the data service resource cannot be created for some reason.
     */
    public DataServiceResource(
        Object                   id,
        ActivitiesConfig         activitiesConfig,
        DataResourceAccessor     dataResourceAccessor, 
        SessionComponentFactory  sessionComponentFactory, 
        ConcurrencySettings      concurrencySettings,
        AccessAuthorizer         accessAuthorizer) 
    throws ResourceCreationException
    {
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Creating new DataServiceResource " + id);
        }
        if ((id == null) || (id.equals("")))
        {
            throw new IllegalArgumentException("id must not be null or empty");
        }
        if (activitiesConfig == null)
        {
            throw new IllegalArgumentException(
                "activitiesConfig must not be null");
        }
        if (dataResourceAccessor == null)
        {
            throw new IllegalArgumentException(
                "dataResourceAccessor must not be null");
        }
        if (sessionComponentFactory == null)
        {
            throw new IllegalArgumentException(
                "sessionComponentFactory must not be null");
        }
        if (concurrencySettings == null)
        {
            throw new IllegalArgumentException(
                "concurrencySettings must not be null");
        }
        
        mID = id;

        // Store data resource accessor
        mDataResourceAccessor = dataResourceAccessor;
        // Create new set of data service resource properties.
        mProperties = new PropertiesImpl();
        // Create an activity factory using the activity configuration.
        mActivityFactory = new ActivityFactoryImpl(activitiesConfig);
        // Create a request factory using information on supported 
        // activities from the activity factory.
        mRequestFactory = new ActivityRequestFactory(mActivityFactory, accessAuthorizer);
        
        // Create a session coordinator using the data resource
        // accessor. 
        try
        {
            sessionComponentFactory.setResourceProperties(mProperties);
            sessionComponentFactory.setDataResourceAccessor(
                mDataResourceAccessor);
            mSessionCoordinator = 
                sessionComponentFactory.getSessionCoordinator();
        }
        catch (DAIException e)
        {
            throw new ResourceCreationException(id.toString(), e);
        }

        // Create a new engine configured to use the session
        // coordinator and with the given default concurrency
        // settings and data resource accessor.
        mEngineConfiguration = 
            new EngineConfigurationImpl(mSessionCoordinator,
                                        concurrencySettings);
        mEngineConfiguration.setDataResourceAccessor(mDataResourceAccessor);
        mEngine = new RequestEngine(mEngineConfiguration);
        // Create a session stream manager for the session
        // coordinator. 
        mSessionStreamManager = 
            new SimpleSessionStreamManager(mSessionCoordinator.getSessionManager());
        // Configure the resource properties.
        addProperties();
    }

    /**
     * Adds all the properties associated with the data resource
     * accessor and activity factory to the properties of the
     * resource.
     */
    private void addProperties()
    {
        if (LOG.isDebugEnabled()) 
        {            
            LOG.debug("Adding data resource accessor properties...");
        }
        Property[] drProps = mDataResourceAccessor.getProperties();
        for (int i = 0; i < drProps.length; i++)
        {
            mProperties.put(drProps[i]);
        }
        if (LOG.isDebugEnabled()) 
        {            
            LOG.debug("Adding activity factory properties...");
        }
        Property[] activityProperties = mActivityFactory.getProperties(); 
        for (int i = 0; i < activityProperties.length; i++)
        {
            mProperties.put(activityProperties[i]);
        }
        if (LOG.isDebugEnabled()) 
        {            
            LOG.debug("Properties added");
        }
    }

    /**
     * Gets the resource ID.
     *
     * @return the resource ID.
     */
    public Object getID() 
    {    
        return mID;
    }
    
    /**
     * Gets the data service resource properties for the resource.
     *
     * @return the properties.
     */
    public Properties getProperties()
    {
        return mProperties;
    }

    /**
     * Gets the session stream manager for the resource.
     *
     * @return the session stream manager.
     */
    public SessionStreamManager getSessionStreamManager() 
    {    
        return mSessionStreamManager;
    }

    /**
     * Execute a perform document.
     * 
     * @param performDocument
     *     The perform document.
     * @param requestContext
     *     Information about the client and the request e.g. client
     *     security credentials.
     * @return response document describing the status and possibly
     * the results of the request.
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unsupported activity or a
     *     query expression is has incorrect syntax.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource e.g. a database connection disappears or a file is
     *     corrupted. 
     */
    public Document perform(final Document performDocument,
                            final RequestContext requestContext)
        throws ResourceRequestAuthorizationException,
               ResourceRequestSpecificationException,
               ResourceRequestExecutionException
    {
        Document response = null;
        // Create request.
        ResponseDocumentBuilder builder = new ResponseDocumentBuilder();
        Request request = null;
        // Set up request context.
        requestContext.setResourceID(mID.toString());
        try
        {
            request = mRequestFactory.createRequest(performDocument, requestContext);
        }
        catch (RequestSpecificationException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (RequestExecutionException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        } catch (ActivityAccessAuthorizationException e) 
        {
            throw new ResourceRequestAuthorizationException(mID, e);
        }
        // Execute request.
        try
        {
            mEngine.perform(request, requestContext, builder);
        }
        catch (RequestAuthorizationException e)
        {
            throw new ResourceRequestAuthorizationException(mID, e);
        }
        catch (RequestSpecificationException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (RequestExecutionException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
        // Create Response.
        response = builder.getResponseDocument();
        // If response contains an error then wrap the response document
        // in an exception.
        if (requestContext.hasError())
        {
            Throwable cause = requestContext.getCauseOfError();
            if (cause instanceof RequestExecutionException)
            {
                throw new ResourceRequestExecutionException(mID, cause);
            }
            else if (cause instanceof ActivityExecutionException)
            {
                throw new ResourceRequestExecutionException(mID, 
                    new RequestExecutionException(cause));
            }
            else
            {
                throw new ResourceRequestSpecificationException(mID, 
                    new RequestSpecificationException(cause,
                                                      XMLUtilities.xmlDOMToString(response)));
            }
        }
        return response;
    }
    
    /**
     * Terminate a current request.
     *
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. the session to which the request belongs does not 
     *     exist.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     */
    public void terminate() 
        throws ResourceRequestAuthorizationException,
               ResourceRequestSpecificationException,
               ResourceRequestExecutionException
    {
        mEngine.terminate();
    }

    /** 
     * Get all the data from an output stream of a session.
     *
     * @param id
     *     Session and stream identifier.
     * @return data
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unknown session or stream.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     * @see uk.org.ogsadai.sessions.SessionStreamManager#getFully(uk.org.ogsadai.sessions.SessionStreamID)
     */
    public Object getFully(SessionStreamID id)
        throws ResourceRequestAuthorizationException,
               ResourceRequestExecutionException,
               ResourceRequestSpecificationException
    {
        try
        {
            return mSessionStreamManager.getFully(id);
        }
        catch (SessionNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamRequestNotProcessingException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionLookupException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
        catch (SessionStreamIOException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
    }

    /** 
     * Get a specific number of blocks of data from an output stream
     * of a session. 
     *
     * @param id
     *     Session and stream identifier.
     * @param numberOfBlocks
     *     Number of blocks to get.
     * @return data
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unknown session or stream.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     * @see uk.org.ogsadai.sessions.SessionStreamManager#getNBlocks(uk.org.ogsadai.sessions.SessionStreamID, int)
     */
    public Object getNBlocks(SessionStreamID id, int numberOfBlocks)
        throws ResourceRequestAuthorizationException,
               ResourceRequestExecutionException,
               ResourceRequestSpecificationException
    {
        try
        {
            return mSessionStreamManager.getNBlocks(id, numberOfBlocks);
        }
        catch (SessionNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamRequestNotProcessingException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionLookupException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
        catch (SessionStreamIOException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
    }

    /** 
     * Get one block of data from an output stream of a session.
     *
     * @param id
     *     Session and stream identifier.
     * @return data
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unknown session or stream.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     * @see uk.org.ogsadai.sessions.SessionStreamManager#getNBlocks(uk.org.ogsadai.sessions.SessionStreamID, int)
     */
    public Object getBlock(SessionStreamID id) 
        throws ResourceRequestAuthorizationException,
               ResourceRequestExecutionException,
               ResourceRequestSpecificationException
    {
        return getNBlocks(id, 1);
    }

    /** 
     * Put a block of data onto an input stream of a session.
     *
     * @param id
     *     Session and stream identifier.
     * @param data
     *     Single block of data.
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unknown session or stream
     *     or the stream was closed.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     * @see uk.org.ogsadai.sessions.SessionStreamManager#putBlock(uk.org.ogsadai.sessions.SessionStreamID, java.lang.Object)
     */
    public void putBlock(SessionStreamID id, Object data)
        throws ResourceRequestAuthorizationException,
               ResourceRequestExecutionException,
               ResourceRequestSpecificationException
    {
        try
        {
            mSessionStreamManager.putBlock(id, data);
        }
        catch (SessionNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamClosedException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamRequestNotProcessingException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionLookupException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
    }

    /** 
     * Put a complete set of data onto an input stream of a session.
     *
     * @param id
     *     Session and stream identifier.
     * @param data 
     *     Single block of data.
     * @throws ResourceRequestAuthorizationException 
     *     if the request fails because of an authorization problem.
     * @throws ResourceRequestSpecificationException 
     *     if the request fails to execute due to a problem in the
     *     request e.g. it specifies an unknown session or stream
     *     or the stream was closed.
     * @throws ResourceRequestExecutionException 
     *     if the request fails to execute due to a problem in the
     *     resource.
     *
     * @see uk.org.ogsadai.sessions.SessionStreamManager#putFully(uk.org.ogsadai.sessions.SessionStreamID, java.lang.Object)
     */
    public void putFully(SessionStreamID id, Object data)
        throws ResourceRequestAuthorizationException,
               ResourceRequestExecutionException,
               ResourceRequestSpecificationException
    {
        try
        {
            mSessionStreamManager.putFully(id, data);
        }
        catch (SessionNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamNotFoundException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamClosedException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionStreamRequestNotProcessingException e)
        {
            throw new ResourceRequestSpecificationException(mID, e);
        }
        catch (SessionLookupException e)
        {
            throw new ResourceRequestExecutionException(mID, e);
        }
    }
}
