/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import java.util.Vector;

import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.properties.PropertyValueException;
import uk.org.ogsadai.dais.xml.services.daix.CollectionType;
import uk.org.ogsadai.dais.xml.services.daix.DocumentDescriptionType;
import uk.org.ogsadai.dataresource.DataResourcePropertyNames;


/**
 * @author Steven Lynden
 * 
 * Facilitates access to XML collections. This class contains methods to
 * initialise the mapping between collection URIs and collection paths in
 * the XML database. Subsequently it is used to retrieve collection schema
 * and provides access to the collection mapping.
 *
 */
public class CollectionAccessor {

    private static Logger mLog = Logger.getLogger(CollectionAccessor.class);
    
    private Collections collectionMapping;

    /**
     * Initialise the collection accessor. The property argument must be the
     * database schema resource property of the initial XML collection data 
     * service resource. 
     */
    public CollectionAccessor(Property collectionSchema) throws PropertyValueException {
        collectionMapping = new Collections();
        Element pValue = (Element) collectionSchema.getValue();
        initialiseCollectionMapping(pValue);   
    }
    
    /**
     * Retrieves the collection mapping object
     */
    public Collections getCollectionMapping() {
        return collectionMapping;   
    }
    
    /*
     * Initialise the collection mapping by retrieving each collection
     * and invoking mapCollection
     */
    private void initialiseCollectionMapping(Element ogsaDaiSchema) {
        NodeList nodes = ogsaDaiSchema.getElementsByTagNameNS(DataResourcePropertyNames.NAMESPACE,"logicalSchema").item(0).getChildNodes();
        for(int i=0;i<nodes.getLength();i++) {
            if ( ! (nodes.item(i) instanceof Element )) continue;
            if ( nodes.item(i).getLocalName().equals("collection"))
                mapCollection((Element)nodes.item(i));
        }   
    }
    
    /*
     * Maps a specific collection path, retrieved through the OGSA-DAI interface,
     * to a URI that will be used to identify the collection through WS-DAIX.
     * Adds each mapping to the collectionMapping.
     */
    private void mapCollection(Element ogsaDaiCollection) {
        String colName = ogsaDaiCollection.getAttributeNS(DataResourcePropertyNames.NAMESPACE,"name");
        try {
            collectionMapping.addMapping(colName);
        } catch ( URI.MalformedURIException e ) {
            mLog.error("Collection " + colName + " could not be mapped to " +
                    "a URI because: " + e.toString() );
        }
        NodeList nodes = ogsaDaiCollection.getChildNodes();
        /* process any sub-collections */
        for(int i=0;i<nodes.getLength();i++) {
            if ( ! (nodes.item(i) instanceof Element) ) continue;
            if ( nodes.item(i).getLocalName().equals("subCollections")) {
                NodeList collections = nodes.item(i).getChildNodes();
                for(int j=0;j<collections.getLength();j++) {
                    if ( ! (collections.item(j) instanceof Element) ) continue;
                    if ( collections.item(j).getLocalName().equals("collection"))
                        mapCollection((Element)collections.item(j));
                }
            }
        }
    }
    
    /**
     * Retrieves the collection schema from an OGSA-DAI data service resource property.
     */
    public CollectionSchema getCollectionSchema(Property property) throws PropertyValueException {
        return CollectionSchema.dataResourcePropertyToCollectionSchema(collectionMapping,property);   
    }
   
}

/**
 * This class is used to represent the collection schema generated
 * from the OGSA-DAI database schema data service resource property
 * 
 * @author Steven Lynden
 * 
 */
class CollectionSchema {

    private long numberOfDocuments;
    
    private CollectionType topLevelCollection;
    
    private Collections collectionMapping;
    
    public static CollectionSchema dataResourcePropertyToCollectionSchema(Collections collectionMapping, Property property) throws PropertyValueException {
        Element pValue = (Element) property.getValue();
        CollectionSchema schema = new CollectionSchema(pValue,collectionMapping);
        return schema;
    }
    
    /**
     * Create an instance using an OGSA-DAI XML schema element
     */
    private CollectionSchema(Element ogsaDaiSchema, Collections collectionMapping) {
        this.collectionMapping = collectionMapping;
        numberOfDocuments = 0;
        NodeList nodes = ogsaDaiSchema.getElementsByTagNameNS(DataResourcePropertyNames.NAMESPACE,"logicalSchema").item(0).getChildNodes();
        for(int i=0;i<nodes.getLength();i++) {
            if ( ! (nodes.item(i) instanceof Element )) continue;
            if ( nodes.item(i).getLocalName().equals("collection"))
                topLevelCollection = createCollection((Element)nodes.item(i));
        }
    }
    
    /**
     * Returns the number of documents in the collections. 
     */
    public long getNumberOfDocuments() {
        return this.numberOfDocuments;   
    }
    
    /**
     * Returns the WS-DAIX topLevelCollection element
     */
    public CollectionType getTopLevelCollection() {
        return this.topLevelCollection;   
    }
    
    /*
     * Transforms an OGSA-DAI data service resource property XML collection
     * element into a WS-DAIX collection type
     */
    private CollectionType createCollection(Element ogsaDaiCollection) {
        CollectionType collection = new CollectionType();
        String colName = ogsaDaiCollection.getAttributeNS(DataResourcePropertyNames.NAMESPACE,"name");
        collection.setName( collectionMapping.pathToURI(colName) );
        NodeList nodes = ogsaDaiCollection.getChildNodes();
        for(int i=0;i<nodes.getLength();i++) {
            if ( ! (nodes.item(i) instanceof Element) ) continue;
            if ( nodes.item(i).getLocalName().equals("resources")) {    
                NodeList resources = ((Element)nodes.item(i)).getElementsByTagNameNS(DataResourcePropertyNames.NAMESPACE,"resource");
                collection.setDocument(new DocumentDescriptionType[resources.getLength()]);
                numberOfDocuments+=resources.getLength();
                for(int j=0;j<resources.getLength();j++) {
                    Element r = (Element) resources.item(j);
                    String name = r.getAttributeNS(DataResourcePropertyNames.NAMESPACE,"id");
                    DocumentDescriptionType d = new DocumentDescriptionType();
                    d.setName(name);
                    collection.setDocument(j,d);
                }
            }
            if ( nodes.item(i).getLocalName().equals("subCollections")) {
                NodeList collections = nodes.item(i).getChildNodes();
                Vector cList = new Vector();
                for(int j=0;j<collections.getLength();j++) {
                    if ( ! (collections.item(j) instanceof Element) ) continue;
                    if ( collections.item(j).getLocalName().equals("collection"))
                        cList.add(createCollection((Element)collections.item(j)));
                }
                collection.setCollection((CollectionType[])cList.toArray(new CollectionType[cList.size()]));
            }
        }
        return collection;
    }
    
}

