/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import java.util.*;

import org.apache.axis.types.URI;
import org.apache.log4j.Logger;

/**
 * @author Steven Lynden
 * 
 * In WS-DAIX collections are identified using URIs whereas OGSA-DAI uses
 * String paths. There therefore needs to be a 1:1 mapping between these two 
 * representations. This class maintains this mapping and is shared by all
 * classes that access/update collections. 
 * 
 * TODO Upon initialisation, existing collections in the database are mapped
 * (string to URI) as follows: collectionPath -> daix:collectionPath
 * e.g. the collection /db/macbeth is mapped to the URI daix:/db/macbeth.
 * The user currently has no control over the initial mapping, so this should
 * be rectified by allowing the user to specify an initial mapping to the
 * URIs that should be used to represent the collections that are already in
 * the database when WS-DAIX is initialised. This could possibly be done
 * by reading from the deployment properties file.
 * 
 */
public class Collections {

    private static Logger mLog = Logger.getLogger(Collections.class);
    
    private Hashtable pathToURI,URIToPath;
    
    /**
     * Initialises the mapping with the initial collections contained
     * within the XML DB. This will be performed that first time that
     * the service processes an operation, and the collections will be 
     * retrieved from the database collection schema. 
     */
    public Collections()  {
        pathToURI = new Hashtable();
        URIToPath = new Hashtable();
    }
    
    public synchronized void addMapping(String collectionPath) throws URI.MalformedURIException {
        addMapping(collectionPath,new URI("daix",collectionPath));
    }
    
    public synchronized void addMapping(String collectionPath, URI collectionURI) {
        pathToURI.put(collectionPath,collectionURI);
        URIToPath.put(collectionURI,collectionPath);
        mLog.debug("Adding mapping: " + collectionURI.toString() + " <-> " + collectionPath);
    }
    
    public synchronized URI pathToURI(String path) {
        return (URI) pathToURI.get(path);   
    }
    
    public synchronized String URIToPath(URI uri) {
        return (String) URIToPath.get(uri);   
    }
    
    /**
     * This method returns the collection path mapped to a URI, relative
     * to a parent collection. Returns null if the URI is not mapped or
     * the mapped collection path does not contain the specified path
     * parameter 
     */
    public synchronized String URIToRelativePath(URI uri, String path) {
        String collection = (String) URIToPath.get(uri);
        if ( collection == null ) return null;
        if ( collection.indexOf(path) != 0 ) return null;
        return collection.substring(path.length()+1,collection.length());
    }
    
    public synchronized boolean collectionExists(URI uri) {
        return URIToPath.containsKey(uri);   
    }
    
    public synchronized void removeMapping(String collectionPath) {
        URI uri = (URI) pathToURI.get(collectionPath);
        pathToURI.remove(collectionPath);
        URIToPath.remove(uri);
    }
    
    public synchronized void removeMapping(URI collectionURI) {
        String path = (String) URIToPath.get(collectionURI);
        pathToURI.remove(path);
        URIToPath.remove(collectionURI);
    }
        
}
