/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.properties.PropertyValueException;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dais.xml.service.core.ChildSensitiveToParent;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapTypeDefaultConfigurationDocument;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType;
import uk.org.ogsadai.dais.xml.service.core.ParentSensitiveToChild;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.service.core.TransactionInitiation;
import uk.org.ogsadai.dais.xml.service.core.TransactionIsolation;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.service.addressing.AttributedURIType;
import uk.org.ogsadai.service.addressing.ReferenceParametersType;


/**
 * @author Steven Lynden
 * 
 * Contains common utility methods used by the WS-DAIX implementation
 *
 */
public class DAIXUtil {

    private static Logger mLog = Logger.getLogger(DAIXUtil.class);

    /*
     * Utility methods used by data resource accessors to create property
     * documents
     */
    
    /**
     * Construct a property document with some default parameters supported
     * by all data resources exposed by this implementation of WS-DAIX:
     * - ConcurrentAccess = true
     * - Readable = true
     * - Transaction properties not supported
     */
    public static PropertyDocumentType getPropertyDocument() {
        PropertyDocumentType properties = new PropertyDocumentType();
        properties.setConcurrentAccess(true);
        properties.setReadable(true);
        properties.setTransactionInitiation(TransactionInitiation.NotSupported);
        properties.setTransactionIsolation(TransactionIsolation.NotSupported);
        return properties;
    }
    
    /**
     * Creates a CollectionPropertyDocument from a DataResourceProperty
     * document. The extra information needed to populate XML collection
     * specific properties is provided by the collection schema property
     * passed as an argument to this method. The CollectionAccessor 
     * parameter is used to access the mapping between WS-DAIX collection
     * URIs and the collectionNames provided by OGSA-DAI.
     */
    public static XMLCollectionPropertyDocumentType propertyDocumentToCollectionPropertyDocument(PropertyDocumentType propertyDoc, Property collectionSchemaProperty, CollectionAccessor collectionAccessor) {
        XMLCollectionPropertyDocumentType xmlProperties = new XMLCollectionPropertyDocumentType();
        xmlProperties.setChildSensitiveToParent(propertyDoc.getChildSensitiveToParent());
        xmlProperties.setConcurrentAccess(propertyDoc.isConcurrentAccess());
        xmlProperties.setConfigurationMap(propertyDoc.getConfigurationMap());
        xmlProperties.setDataResourceManagement(propertyDoc.getDataResourceManagement());
        xmlProperties.setDatasetMap(propertyDoc.getDatasetMap());
        xmlProperties.setLanguageMap(propertyDoc.getLanguageMap());
        xmlProperties.setParentDataResource(propertyDoc.getParentDataResource());
        xmlProperties.setParentSensitiveToChild(propertyDoc.getParentSensitiveToChild());
        xmlProperties.setReadable(propertyDoc.isReadable());
        xmlProperties.setTransactionInitiation(propertyDoc.getTransactionInitiation());
        xmlProperties.setTransactionIsolation(propertyDoc.getTransactionIsolation());
        xmlProperties.setWriteable(propertyDoc.isWriteable());
        /*
         * Retrieve the XML collection schema
         */
        try {
            CollectionSchema schema = collectionAccessor.getCollectionSchema(collectionSchemaProperty);
            xmlProperties.setTopLevelCollection(schema.getTopLevelCollection());
            xmlProperties.setNumberOfDocuments(schema.getNumberOfDocuments());            
        } catch ( PropertyValueException e ) {
            mLog.error("Unable to retrieve the collection schema " + e.toString() );   
        }
        xmlProperties.setSupportsCollectionNesting(true);
        xmlProperties.setSupportsCollections(true);
        xmlProperties.setSupportsSchemas(false);
        return xmlProperties; 
    }
    
    /**
     * Returns the configuration map for an XML Collection resource.
     * This maps:
     * {FactoryOperation, PortTypeQName} -> DefaultConfigurationDocument 
     */
    public static ConfigurationMapType[] createCollectionResourceConfigurationMap() {
        ConfigurationMapType[] configurationMap = new ConfigurationMapType[14];
        //xpathfactory.sequence
        configurationMap[0] = getConfigurationMap(DAIXProperties.XPathExecuteFactory,DAIXProperties.XMLSequenceAccessPT,getConfigurationDocument(false,false,false));
        //xqueryfactory.sequence
        configurationMap[1] = getConfigurationMap(DAIXProperties.XQueryExecuteFactory,DAIXProperties.XMLSequenceAccessPT,getConfigurationDocument(false,false,false));
        //collectionSelectionFactory.collectionAccess
        configurationMap[2] = getXMLCollectionConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XMLCollectionAccessPT,getXMLCollectionConfigurationDocument(true,true,true));
        //collectionSelectionFactory.collectionFactory
        configurationMap[3] = getXMLCollectionConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XMLCollectionFactoryPT,getXMLCollectionConfigurationDocument(true,true,true));
        //collectionSelectionFactory.xpathAccess
        configurationMap[4] = getConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XPathAccessPT,getConfigurationDocument(true,true,true));
        //collectionSelectionFactory.xpathFactory
        configurationMap[5] = getConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XPathFactoryPT,getConfigurationDocument(true,true,true));
        //collectionSelectionFactory.xqueryAccess
        configurationMap[6] = getConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XQueryAccessPT,getConfigurationDocument(true,true,true));
        //collectionSelectionFactory.xqueryFactory
        configurationMap[7] = getConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XQueryFactoryPT,getConfigurationDocument(true,true,true));
        //collectionSelectioNFactory.xUpdateAccess
        configurationMap[8] = getConfigurationMap(DAIXProperties.CollectionSelectionFactory,DAIXProperties.XUpdateAccessPT,getConfigurationDocument(true,true,true));
        //documentSelectionFactory.xpathAccess
        configurationMap[9] = getConfigurationMap(DAIXProperties.DocumentSelectionFactory,DAIXProperties.XPathAccessPT,getConfigurationDocument(true,true,true));
        //documentSelectionFactory.xpathFactory
        configurationMap[10] = getConfigurationMap(DAIXProperties.DocumentSelectionFactory,DAIXProperties.XPathFactoryPT,getConfigurationDocument(true,true,true));
        //documentSelectionFactory.xQueryAccess
        configurationMap[11] = getConfigurationMap(DAIXProperties.DocumentSelectionFactory,DAIXProperties.XQueryAccessPT,getConfigurationDocument(true,true,true));
        //documentSelectionFactory.xQueryFactory
        configurationMap[12] = getConfigurationMap(DAIXProperties.DocumentSelectionFactory,DAIXProperties.XQueryFactoryPT,getConfigurationDocument(true,true,true));
        //documentSelectionFactory.xUpdateAccess
        configurationMap[13] = getConfigurationMap(DAIXProperties.DocumentSelectionFactory,DAIXProperties.XUpdateAccessPT,getConfigurationDocument(true,true,true));
        return configurationMap;
    }
    
    /*
     * Utility methods for converting to/from XQueryX
     */    
    
    /**
     * Convert XQueryX to XQuery
     */    
    public static String XQueryXToXQuery(Document xQueryX, String xsltDocFilename) throws TransformerException, TransformerConfigurationException, FileNotFoundException, ParserConfigurationException, SAXException, IOException, DAIException {
        Document xsltDoc = XMLUtilities.xmlFileToDOM(xsltDocFilename,false);
        return XQueryXToXQuery(xQueryX,xsltDoc);
    }
    
    /**
    * Convert an XQueryX XPath statement to XPath 
    */
    public static String XPathXToXQuery(Document xPathExpr, String xsltDocFilename) throws TransformerException, TransformerConfigurationException, FileNotFoundException, ParserConfigurationException, SAXException, IOException, DAIException {
        Document xsltDoc = XMLUtilities.xmlFileToDOM(xsltDocFilename,false);
        Document xQueryExpr = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        Element root = xQueryExpr.createElementNS(DAIXProperties.XQUERYX_NS,"module");
        xQueryExpr.appendChild(root);
        Node n = xQueryExpr.importNode(xPathExpr.getDocumentElement(),true);
        root.appendChild(n);
        if ( mLog.isDebugEnabled() ) {
            mLog.debug("Wrapped XPath expression as XQueryX module: " + XMLUtilities.xmlDOMToString(xQueryExpr) );   
        }
        return XQueryXToXQuery(xQueryExpr,xsltDoc);
    }
    
    /**
     * Convert XQueryX to XQuery
     */ 
    public static String XQueryXToXQuery(Document xQueryX, Document XSLTDoc) throws TransformerException, TransformerConfigurationException, FileNotFoundException, ParserConfigurationException, SAXException, IOException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        if (tFactory.getFeature(DOMSource.FEATURE) && tFactory.getFeature( DOMResult.FEATURE )) {
            DocumentBuilderFactory dFactory = DocumentBuilderFactory.newInstance();
            dFactory.setNamespaceAware(true);
            DocumentBuilder dBuilder = dFactory.newDocumentBuilder();
            DOMSource xslDomSource = new DOMSource(XSLTDoc);
            Transformer transformer = tFactory.newTransformer(xslDomSource);
            DOMSource xmlDomSource = new DOMSource(xQueryX);
            DOMResult domResult = new DOMResult();
            StringWriter sw = new StringWriter();
            StreamResult s = new StreamResult(sw);
            transformer.transform(xmlDomSource, s);
            return sw.getBuffer().toString();
        } else {
            throw new org.xml.sax.SAXNotSupportedException("DOM node processing " + "not supported!");
        }
    }
    
    /*
     * Utility methods for extracting results from messages
     */
    
    /**
     * Returns a message element containing the results if the dataset format URI
     * is valid
     */
    public static MessageElement getResultsAsMessageElement(URI datasetFormatURI, Document results) throws InvalidDatasetFormatFaultType {
        if ( datasetFormatURI.equals(DAIXProperties.defaultDataSetFormatURI))
            return new MessageElement(DAIXProperties.OGSA_DAI_RESULTS_QNAME,results);
        else throw new InvalidDatasetFormatFaultType();
    }
    
    /*
     * Utility methods for constructing configuration documents
     */
    
    /**
     * Returns an XML collection configuration document given parameters specifying:
     * - whether the resource is by default writeable
     * - whether the resource is by default sensitive to children
     * - whether the resource is by default sensitive to its parent
     */
    public static XMLCollectionConfigurationDocumentType getXMLCollectionConfigurationDocument(boolean writeable, boolean childSensitiveToParent, boolean parentSensitiveToChild) {
        XMLCollectionConfigurationDocumentType c = new XMLCollectionConfigurationDocumentType();
        c.setSupportsCollectionNesting(true);
        c.setSupportsCollections(true);
        c.setWriteable(Boolean.valueOf(writeable));
        if ( childSensitiveToParent )
            c.setChildSensitiveToParent(ChildSensitiveToParent.Sensitive);
        else 
            c.setChildSensitiveToParent(ChildSensitiveToParent.Insensitive);
        if ( parentSensitiveToChild )
            c.setParentSensitiveToChild(ParentSensitiveToChild.Sensitive);
        else
            c.setParentSensitiveToChild(ParentSensitiveToChild.Insensitive);
        c.setReadable(Boolean.TRUE);
        c.setTransactionInitiation(TransactionInitiation.NotSupported);
        c.setTransactionIsolation(TransactionIsolation.NotSupported);
        return c;
    }
    
    /**
     * Returns a data resource configuration document given parameters specifying:
     * - whether the resource is by default writeable
     * - whether the resource is by default sensitive to children
     * - whether the resource is by default sensitive to its parent
     */
    public static ConfigurationDocumentType getConfigurationDocument(boolean writeable, boolean childSensitiveToParent, boolean parentSensitiveToChild) {
        ConfigurationDocumentType c = new ConfigurationDocumentType();   
        c.setWriteable(Boolean.valueOf(writeable));
        if ( childSensitiveToParent )
            c.setChildSensitiveToParent(ChildSensitiveToParent.Sensitive);
        else 
            c.setChildSensitiveToParent(ChildSensitiveToParent.Insensitive);
        if ( parentSensitiveToChild )
            c.setParentSensitiveToChild(ParentSensitiveToChild.Sensitive);
        else
            c.setParentSensitiveToChild(ParentSensitiveToChild.Insensitive);
        c.setReadable(Boolean.TRUE);
        c.setTransactionInitiation(TransactionInitiation.NotSupported);
        c.setTransactionIsolation(TransactionIsolation.NotSupported);
        return c;
    }
    
    /**
     * Returns a configuration map element wrapping a configuration document.
     * Each entry in a configuration map is one of these elements.
     */
    public static ConfigurationMapType getConfigurationMap(QName messageQName, QName portTypeQName, ConfigurationDocumentType configurationDocument ) {
        ConfigurationMapType map = new ConfigurationMapType();
        map.setConfigurationDocumentQName(DAIXProperties.ConfigurationDocumentType);
        map.setMessageQName(messageQName);
        map.setPortTypeQName(portTypeQName);
        ConfigurationMapTypeDefaultConfigurationDocument d = new ConfigurationMapTypeDefaultConfigurationDocument();
        d.setConfigurationDocument(configurationDocument);
        map.setDefaultConfigurationDocument(d);
        return map; 
    }
    
    /**
     * Returns an XML collection configuration map element wrapping an
     * XML configuration document.
     * Each entry in an XML collection configuration map is one of these elements.
     */
    public static ConfigurationMapType getXMLCollectionConfigurationMap(QName messageQName, QName portTypeQName, XMLCollectionConfigurationDocumentType configurationDocument ) {
        ConfigurationMapType map = new ConfigurationMapType();
        map.setConfigurationDocumentQName(DAIXProperties.XMLCollectionConfigurationDocumentType);
        map.setMessageQName(messageQName);
        map.setPortTypeQName(portTypeQName);
        ConfigurationMapTypeDefaultConfigurationDocument d = new ConfigurationMapTypeDefaultConfigurationDocument();
        d.setConfigurationDocument(configurationDocument);
        map.setDefaultConfigurationDocument(d);
        return map; 
    }
    
    /*
     * Utility methods for creating data resource addresses
     * 
     * - OGSA-DAI respresents resource IDs as Strings.
     *  
     * - WS-DAI represents resource IDs (abstract names) as URIs.
     * 
     * These methods construct an address (endpoint) using a service
     * URL + either of these resource identifiers.
     */
    
    /**
     * Constructs a data resource address
     * The resourceID in string format; this therefore assumes
     * that it is an OGSA-DAI resource ID
     */
    public static DataResourceAddressType constructDataResourceAddressType(String url, String resourceID) {
        DataResourceAddressType addr = new DataResourceAddressType();
        ReferenceParametersType rp = new ReferenceParametersType();
        addr.setAddress(new AttributedURIType(url.trim()));
        rp.set_any( new MessageElement[] { new MessageElement(new QName(DAIXProperties.WS_DAI_NS,"DataResourceAbstractName"),"daix:" + resourceID) } );
        addr.setReferenceParameters(rp);
        return addr;
    }
    
    /**
     * Constructs a data resource address
     * The resource ID in URI format; this therefore assumes
     * its a DAIS resource ID
     */
    public static DataResourceAddressType constructDataResourceAddressType(String url, URI resourceID) {
        DataResourceAddressType addr = new DataResourceAddressType();
        ReferenceParametersType rp = new ReferenceParametersType();
        addr.setAddress(new AttributedURIType(url.trim()));
        rp.set_any( new MessageElement[] { new MessageElement(new QName(DAIXProperties.WS_DAI_NS,"DataResourceAbstractName"),resourceID.toString())} );
        addr.setReferenceParameters(rp);
        return addr;
    }
    
    /*
     * Other misc. methods
     */
    
    /**
     * Retrieves the collection path from an absoluate collection URI
     * e.g. xmldb:exist://localhost:8080/exist/xmlrpc/db/contacts
     * will return db/contacts
     * @deprecated
     */
    public static String getPathFromCollectionURI(String collectionURI) {
        int i = collectionURI.indexOf("exist/xmlrpc/");
        return collectionURI.substring(i+12,collectionURI.length());
    }   //collection.getName() ?
    
}

