/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import javax.xml.namespace.QName;
import org.apache.log4j.Logger;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.service.resource.config.FileBasedActivitiesConfig;
import uk.org.ogsadai.sessions.SessionConfigurationLoader;

/**
 * 
 * Loads deployment properties from the wsdaix-deployment.properties file
 * 
 * Maintains a mapping of port types + messages to end points.
 * 
 * @author Steven Lynden
 *
 */
public class DeploymentProperties {

    private static Logger mLog = Logger.getLogger(DeploymentProperties.class);
    
    /*
     * The properties object loaded from the properties file.
     */
    private Properties properties;
    
    /*
     * The service path e.g. ogsadai/service of the OGSA-DAI service
     * used as a back-end by the WS-DAIX implementation.  
     */
    private String servicePath;
    
    /*
     * Keys in the properties file
     */
    public static String COLLECTION_CORE_RESOURCE_LIST_ENDPOINT = "collection.CoreResourceList.url";
    public static String COLLECTION_XPATH_ACCESS_ENDPOINT = "collection.XPathAccess.url";
    public static String COLLECTION_CORE_DATA_ACCESS_ENDPOINT = "collection.CoreDataAccess.url";
    public static String COLLECTION_XQUERY_ACCESS_ENDPOINT = "collection.XQueryAccess.url";
    public static String COLLECTION_XPATH_FACTORY_ENDPOINT = "collection.XPathFactory.url";
    public static String COLLECTION_XQUERY_FACTORY_ENDPOINT = "collection.XQueryFactory.url";
    public static String COLLECTION_COLLECTION_ACCESS_ENDPOINT = "collection.CollectionAccess.url";
    public static String COLLECTION_XUPDATE_ACCESS_ENDPOINT = "collection.XUpdateAccess.url";
    public static String COLLECTION_COLLECTION_FACTORY_ENDPOINT = "collection.CollectionFactory.url";
    public static String SEQUENCE_SEQUENCE_ACCESS_ENDPOINT = "sequence.SequenceAccess.url";
    public static String SEQUENCE_CORE_DATA_ACCESS_ENDPOINT = "sequence.CoreDataAccess.url";
    public static String SEQUENCE_CORE_RESOURCE_LIST_ENDPOINT = "sequence.CoreResourceList.url";
    public static String DOCUMENT_XPATH_FACTORY_ENDPOINT = "document.XPathFactory.url";
    public static String DOCUMENT_XPATH_ACCESS_ENDPOINT = "document.XPathAccess.url";
    public static String DOCUMENT_XQUERY_FACTORY_ENDPOINT = "document.XQueryFactory.url";
    public static String DOCUMENT_XQUERY_ACCESS_ENDPOINT = "document.XQueryAccess.url";
    public static String DOCUMENT_CORE_RESOURCE_LIST_ENDPOINT = "document.CoreResourceList.url";
    public static String DOCUMENT_XUPDATE_ACCESS_ENDPOINT = "document.XUpdate.url";
    public static String DOCUMENT_CORE_DATA_ACCESS_ENDPOINT = "document.CoreDataAccess.url";
    
    /*
     * Indicates whether the deployed externally managed resource
     * is writeable or not.
     */
    private static String WRITEABLE = "writeable";

    /*
     * The XSLT file used to transform XQueryX -> XQuery/XPath
     */
    private static final String XQUERYX_TRANSFORM = "xquery.transform.file";

    //mapping of services, port types -> endpoint key
    
    /*
     * A mapping of { service, port type } -> endpoint
     * WS-DAIX needs this information to determine the endpoint through
     * which resources can be accessed when they are created via
     * factory operations. This mapping is accessed by the getEndpoint
     * method.
     */
    private String[][] servicePortTypeToEndpointMap = new String[][] {
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.CoreResourceListPT.toString(), COLLECTION_CORE_RESOURCE_LIST_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XPathAccessPT.toString(), COLLECTION_XPATH_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.CoreDataAccessPT.toString(), COLLECTION_CORE_DATA_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XQueryAccessPT.toString(), COLLECTION_XQUERY_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XPathFactoryPT.toString(), COLLECTION_XPATH_FACTORY_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XQueryFactoryPT.toString(), COLLECTION_XQUERY_FACTORY_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XMLCollectionAccessPT.toString(), COLLECTION_COLLECTION_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.COLLECTION_SERVICE, DAIXProperties.XMLCollectionFactoryPT.toString(), COLLECTION_COLLECTION_FACTORY_ENDPOINT },
        new String[] { DAIXProperties.SEQUENCE_SERVICE, DAIXProperties.XMLSequenceAccessPT.toString(), SEQUENCE_SEQUENCE_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.SEQUENCE_SERVICE, DAIXProperties.CoreDataAccessPT.toString(), SEQUENCE_CORE_DATA_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.SEQUENCE_SERVICE, DAIXProperties.CoreResourceListPT.toString(), SEQUENCE_CORE_RESOURCE_LIST_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.XPathFactoryPT.toString(), DOCUMENT_XPATH_FACTORY_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.XPathAccessPT.toString(), DOCUMENT_XPATH_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.XQueryFactoryPT.toString(), DOCUMENT_XQUERY_FACTORY_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.XQueryAccessPT.toString(), DOCUMENT_XQUERY_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.CoreResourceListPT.toString(), DOCUMENT_CORE_RESOURCE_LIST_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.XUpdateAccessPT.toString(), DOCUMENT_XUPDATE_ACCESS_ENDPOINT },
        new String[] { DAIXProperties.DOCUMENT_SERVICE, DAIXProperties.CoreDataAccessPT.toString(), DOCUMENT_CORE_DATA_ACCESS_ENDPOINT }
    };
    
    /**
     * Given a service type and port type, returns the 
     * endpoint through which access it provided by WS-DAIX
     * Used by factory operations to determine the
     * address returned to the client 
     */
    public String getEndpoint(String serviceType, QName portType) throws DataResourceUnavailableFaultType {
        mLog.debug("Looking up endpoint: " + serviceType + " " + portType.toString() );
        for(int i=0;i<servicePortTypeToEndpointMap.length;i++) {
            if ( ( servicePortTypeToEndpointMap[i][0].equals(serviceType) ) && servicePortTypeToEndpointMap[i][1].equals(portType.toString()) )
                return getProperty( servicePortTypeToEndpointMap[i][2] );    
        }
        //should never happen unless a port type is missing in the map
        mLog.debug("Unexpected FATAL error looking up endpoint:");
        mLog.debug("Endpoint for " + serviceType + " " + portType.toString() + " not found");
        mLog.debug("... throwing DataResourceUnavailableException.");
        throw new DataResourceUnavailableFaultType();
    }
    
    /**
     * Initialise the deployment properties by loading the file
     */
    public DeploymentProperties(String filename) throws IOException {
        this(filename,null);
    }
    
    /**
     * Initialise the deployment properties by loading the file and
     * setting the OGSA-DAI service path
     */
    public DeploymentProperties(String filename, String servicePath) throws IOException {
        properties = new Properties();
        properties.load( new FileInputStream(filename));
        this.servicePath = servicePath;
    }
    
    /**
     * Returns the XSLT file used to convert XQueryX
     * to XQuery. WS-DAIX query interfaces receivee
     * XQueryX and must convert it to a String which is
     * passed to OGSA-DAI through the perform document. 
     */
    public String getXQueryXTransformFilename() {
        return getProperty(XQUERYX_TRANSFORM);    
    }
    
    /**
     * Retrieve a property with the given name (key).
     */
    public String getProperty(String name) {
        return properties.getProperty(name).trim();
    }
    
    /**
     * Returns the service path of the OGSA-DAI data
     * service used to implement WS-DAIX
     */
    public String getServicePath() {
        return this.servicePath;   
    }
    
    /**
     * Indicates whether modification of the underlying XML database 
     * (through the externally managed resource) is allowed.
     */
    public boolean getWriteAccess() {
        return Boolean.valueOf(properties.getProperty(WRITEABLE)).booleanValue();   
    }
    
    /**
     * Retrieves the activity config info from the data resource directory.
     */
    public static FileBasedActivitiesConfig getActivityConfig(String directory, String dsrType) throws DAIException {
        File childActivityConfigFile = 
            new File(directory,"activityConfig.xml");
        return new FileBasedActivitiesConfig(childActivityConfigFile); 
    }
    
    /**
     * Retrieves the session config info from the data resource directory.
     */
    public static SessionConfigurationLoader getSessionConfig(String directory, String dsrType) throws DAIException {
        File childSessionConfigFile = 
            new File(directory,"sessionConfig.xml");
        return new SessionConfigurationLoader(childSessionConfigFile);
    }
    
}
