//(c) International Business Machines Corporation, 2002 - 2006. 
//(c) University of Edinburgh 2002 - 2006.
//See OGSA-DAI-Licence.txt for licencing information.
package uk.org.ogsadai.wsdaix.core;

import java.io.File;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;
import java.util.Vector;
import javax.xml.namespace.QName;
import org.apache.axis.MessageContext;
import org.apache.axis.message.MessageElement;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.client.toolkit.soap.AttachmentManager;
import uk.org.ogsadai.common.SecurityContext;
import uk.org.ogsadai.common.SecurityContextProvider;
import uk.org.ogsadai.common.msgs.DAILogger;
import uk.org.ogsadai.common.msgs.MessageID;
import uk.org.ogsadai.common.properties.Properties;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.serializer.DAISerializationException;
import uk.org.ogsadai.common.serializer.SerializerUtils;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.engine.ConcurrencySettings;
import uk.org.ogsadai.engine.IllegalConcurrencySettingException;
import uk.org.ogsadai.engine.RequestContext;
import uk.org.ogsadai.exception.AuthorisationFaultException;
import uk.org.ogsadai.exception.DAIAuthorizationException;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.exception.DAIExceptionInformation;
import uk.org.ogsadai.exception.InternalFaultException;
import uk.org.ogsadai.exception.NonOGSADAIExceptionWrapper;
import uk.org.ogsadai.exception.ServerException;
import uk.org.ogsadai.service.DeploymentParamParseException;
import uk.org.ogsadai.service.MessageContextException;
import uk.org.ogsadai.service.MissingDeploymentParamException;
import uk.org.ogsadai.service.MissingPerformDocException;
import uk.org.ogsadai.service.OGSADAIConstants;
import uk.org.ogsadai.service.ResourceFileException;
import uk.org.ogsadai.service.authorization.AccessAuthorizer;
import uk.org.ogsadai.service.authorization.AuthorizationFactory;
import uk.org.ogsadai.service.authorization.AuthorizationInitialisationException;
import uk.org.ogsadai.service.authorization.ResourceAuthorizationException;
import uk.org.ogsadai.service.faults.AuthorisationFaultType;
import uk.org.ogsadai.service.faults.ComplexFaultType;
import uk.org.ogsadai.service.faults.InternalFaultType;
import uk.org.ogsadai.service.faults.RequestFaultType;
import uk.org.ogsadai.service.faults.ResourceBusyFaultType;
import uk.org.ogsadai.service.faults.ResourceUnknownFaultType;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.NoSuchResourceIDException;
import uk.org.ogsadai.service.resource.ResourceCreationException;
import uk.org.ogsadai.service.resource.ResourceIDAlreadyAssignedException;
import uk.org.ogsadai.service.resource.ResourceRequestAuthorizationException;
import uk.org.ogsadai.service.resource.ResourceRequestExecutionException;
import uk.org.ogsadai.service.resource.ResourceRequestSpecificationException;
import uk.org.ogsadai.service.resource.ResourceUnknownException;
import uk.org.ogsadai.service.resource.config.ActivitiesConfig;
import uk.org.ogsadai.service.resource.config.DSRComponentFactory;
import uk.org.ogsadai.service.resource.config.FileBasedDSRComponentFactory;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactoryUtil;
import uk.org.ogsadai.service.resource.factory.DynamicDataServiceResourceCreationException;
import uk.org.ogsadai.service.resource.factory.DynamicDataServiceResourceRemovalException;
import uk.org.ogsadai.service.resource.factory.ResourceFactoryUtilCreateException;
import uk.org.ogsadai.service.wsi.dataservice.DataServicePortType;
import uk.org.ogsadai.service.wsi.handler.ResourceIDHandler;
import uk.org.ogsadai.service.wsrp.GetMultipleResourcePropertiesRequest;
import uk.org.ogsadai.service.wsrp.GetMultipleResourcePropertiesResponse;
import uk.org.ogsadai.service.wsrp.GetResourcePropertyResponse;
import uk.org.ogsadai.service.wsrp.InvalidResourcePropertyQNameFaultType;
import uk.org.ogsadai.sessions.SessionComponentFactory;
import uk.org.ogsadai.sessions.SessionStreamID;
import uk.org.ogsadai.types.ExtensibilityType;

/**
* This class provides an OGSA-DAI data service which exposes 0 or
* more data service resources. Extra methods have been added to 
* implement the WS-DAIX port types.
* 
* @author The OGSA-DAI Project Team, modified by Steven Lynden
* 
* Modified class contains the WS-DAIX operations and some initialisation
* code to prevent multiple instance of this class being created every
* time one of the three WS-DAIX services is first accessed.
* 
* Only one instance of this class is created.
* 
* This class has been modified to implement WS-DAIX; invoking the OGSA-DAI
* operations is not safe.
* 
*/
public class DataServiceImpl implements SecurityContextProvider,
                                     DataServicePortType,
                                     DataServiceResourceFactory
{        
 /** Copyright statement */
 private static final String COPYRIGHT_NOTICE =
     "(c) International Business Machines Corporation, 2002-2006. (c) University of Edinburgh 2002 - 2006.";
 
 /** Logger object for logging in this class */
 private static DAILogger LOG = DAILogger.getLogger(DataServiceImpl.class);
 
 /** 
  * Maps resource IDs (java.lang.String) to data service resources
  * (uk.org.ogsadai.service.resource.DataServiceResource)
  */
 private Map mDataServiceResources = 
     Collections.synchronizedMap(new HashMap());
 
 /** Has service been initialised? */
 private static boolean mInitialised = false;
 
 /**
  * Deployment descriptor parameter - name of data service
  * resources file holding names of resources exposed by the
  * service
  */
 private static final String RESOURCES_FILE = 
     "dai.data.service.resources.file";
 
 /** Name of data service resources file */
 protected String mResourcesFile = null;
 
 /**
  * Deployment descriptor parameter - path to OGSA-DAI
  * configuration files directory
  */
 private static final String DAI_CONFIG_DIR = "dai.config.files.root";
 
 /**
  * Path to OGSA-DAI configuration files directory
  */
 private String mConfigDir = null;
 
 /**
  * Deployment descriptor parameter - path to OGSA-DAI
  * XML Schema directory
  */
 private static final String DAI_SCHEMA_DIR = "dai.schema.files.root";
 
 /**
  * Deployment descriptor parameter - OGSA-DAI version used to
  * implement service
  */
 private static final String VERSION = "dai.version";
 
 /**
  * Deployment descriptor parameter - timeout for all Perform
  * requests submitted to each resource
  */
 private static final String REQUEST_TIMEOUT = 
     "dai.request.timeout";
 
 /**
  * Deployment descriptor parameter - maximum number of Perform 
  * requests that can be submitted to each resource
  */
 private static final String MAX_REQUESTS = "dai.max.requests";
 
 /**
  * Deployment descriptor parameter - maximum number of Perform
  * requests that can be queued by each resource
  */
 private static final String MAX_QUEUE_LENGTH = 
     "dai.max.queue.length";
 
 /** Concurrency settings for resources */
 private ConcurrencySettings mConcurrencySettings = 
     new ConcurrencySettings(10, 20, 0);
 
 /** Configuration file for resource authorization */
 private static final String AUTHORIZATION_CONFIG = "dai.authorization.config";
 
 /** Authorizes access to resources */
 private AccessAuthorizer mAuthority = null;
 
 /** Data service resources file root element name */
 protected static final String RESOURCE_ELT = "dataServiceResource";
 
 /** Data service resources file root element attribute name */
 protected static final String RESOURCE_ATTR = "name";
 
 /** OGSA-DAI service version */
 private String mVersion = null;
 
 /** Local service URL */
 protected String mURL = null;
 
 /** The main implementation class of WS-DAIX */
 private static DAIXImpl daixImpl = null;
 
 /** 
  * Data service resource factory utility object used to assist in
  * the dynamic creation of new data service resources.
  */
 protected DataServiceResourceFactoryUtil mResourceFactoryUtil;
 
 /**
  * Create a new data service.
  * <p>
  * The service will initialise itself using information within its 
  * deployment descriptor if it has not already done so.
  * 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  */
 public DataServiceImpl() throws InternalFaultType
 {
    if (! mInitialised) 
    {
        try 
        {
            //NOTE: THIS PREVENTS THE USE OF THE
            //OGSA-DAI OPERATIONS.
            
            synchronized(DataServiceImpl.class) {
                //Although multiple instances of this class exist, the same daixImpl object
                //is used by all instances of subclasses
                //This allows the data services resources of a single OGSA-DAI service
                //to be shared by the various WS-DAIX port types.
                if ( mInitialised ) return;
                initialise();
                daixImpl = new DAIXImpl(mDataServiceResources,mConfigDir,this,mURL);
                mInitialised = true;
            }
        }
        catch (Exception e)
        {
            throwInternalFault(e, true);
        }
    }
 }

 /**
  * Extracts from the message context and the service deployment
  * descriptor information on the service URL, version,
  * configuration files directory, schemae
  * directory and data service resource configuration file.
  * <p>
  * Sets up the initial set of data service resources to be
  * exposed.
  *  
  * @throws MissingDeploymentParamException 
  *     if the OGSA-DAI configuration files directories are not
  *     specified in the deployment descriptor. 
  * @throws DeploymentParamParseException 
  *     if any concurrency setting is not parsable into a number. 
  * @throws IllegalConcurrencySettingException 
  *     if any concurrency setting is illegal.
  * @throws ResourceFactoryUtilCreateException 
  *     if there is a problem encountered when creating a data
  *     service resource factory utility object.
  * @throws ResourceFileException 
  *     if there is a problem encountered when loading the data
  *     service resources file. 
  * @throws MessageContextException 
  *     if the current MessageContext cannot be determined. 
  * @throws AuthorizationInitialisationException
  *     if there is a problem encountered when initialising the authorization
  */
 private void initialise()
     throws MissingDeploymentParamException,
            MessageContextException,
            ResourceFileException,
            DeploymentParamParseException,
            IllegalConcurrencySettingException,
            ResourceFactoryUtilCreateException,
            AuthorizationInitialisationException
 {        
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Entered initialise...");
     }
     // Get the current message context.
     MessageContext context = MessageContext.getCurrentContext();
     if (context == null) 
     {            
         throw new MessageContextException();
     }
     // Get the service URL and OGSA-DAI version.
     mURL = (String)context.getTargetService();
     mVersion = (String)context.getProperty(VERSION);
     // Get OGSA-DAI configuration files root.
     mConfigDir = (String)context.getProperty(DAI_CONFIG_DIR);
     if (mConfigDir == null) 
     {    
         throw new MissingDeploymentParamException(DAI_CONFIG_DIR);
     }
     // Get OGSA-DAI schema root.
     String schemaDir = (String)context.getProperty(DAI_SCHEMA_DIR);
     if (schemaDir == null) 
     {   
         throw new MissingDeploymentParamException(DAI_SCHEMA_DIR);
     }
     // Get path to the data service resources file.
     String resourcesFileStr = (String)context.getProperty(RESOURCES_FILE);
     if ((resourcesFileStr == null) || (resourcesFileStr.equals(""))) 
     {
         throw new MissingDeploymentParamException(RESOURCES_FILE);
     }
     if (LOG.isDebugEnabled()) 
     {            
         LOG.debug("Service URL: " + mURL);
         LOG.debug("OGSA-DAI version: " + mVersion);
         LOG.debug("OGSA-DAI configuration file directory: " + mConfigDir);
         LOG.debug("OGSA-DAI schema directory: " + schemaDir);
         LOG.debug("OGSA-DAI data service resources file: " +
                   resourcesFileStr); 
     }
     // Get concurrency settings.
     configureConcurrencySettings(context);
     
     // Create the authorization class.
     String authorizationFile = (String)context.getProperty(AUTHORIZATION_CONFIG);
     loadAuthorization(authorizationFile);
     
     // Create the utility data resource factory.            
     try 
     {
         mResourceFactoryUtil = 
             new DataServiceResourceFactoryUtil(new File(mConfigDir),
                                                new File(schemaDir));
     }
     catch (DAIException e)
     {
         throw new ResourceFactoryUtilCreateException(e);
     }
     // Get the initial set of data service resources.
     File resourcesFile = null;
     if (! mConfigDir.equals("")) 
     {
         resourcesFile = new File(mConfigDir);
         resourcesFile = new File(resourcesFile, resourcesFileStr);
     }
     else
     {
         resourcesFile = new File(resourcesFileStr);
     }
     mResourcesFile = resourcesFile.toString();
     // Load the data service resources file.
     Document resourcesDoc = null;
     try
     {
         resourcesDoc =
             XMLUtilities.xmlFileToDOM(mResourcesFile, false);
     }
     catch (DAIException e)
     {
         throw new ResourceFileException(resourcesFile, e);
     } 
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Data service resources file loaded.");
     }
     // Get the number of resources.
     NodeList resources = 
         resourcesDoc.getDocumentElement().getElementsByTagName(RESOURCE_ELT);
     int numResources = resources.getLength();
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Number of resources: " + numResources);
     }
     // Create DataServiceResources for each of the data service
     // resource exposed by this service. 
     String name;
     for (int i = 0; i < numResources; i++) 
     {
         name = ((Element)(resources.item(i))).getAttribute(RESOURCE_ATTR);
         if (LOG.isDebugEnabled()) 
         {
             LOG.debug("Creating resource: " + i + " = " + name);
         }
         try
         {
             createResourceFromConfigDirectory(name);
         }
         catch (DAIException e) 
         {
             // Problem creating one? Just gulp the exception so that
             // the service can still create and be used with the
             // non-problematic resources.
             LOG.error(e);
         }
     }
     
     mInitialised = true;
 }
 
 /**
  * Extract the concurrency settings from the deployment descriptor.
  *
  * @param context 
  *     Current message context.
  * @throws DeploymentParamParseException 
  *     if any concurrency setting is not parsable into a number. 
  * @throws IllegalConcurrencySettingException 
  *     if any concurrency setting is illegal.
  */
 private void configureConcurrencySettings(MessageContext context)
     throws DeploymentParamParseException, 
            IllegalConcurrencySettingException
 {
     // Get concurrency settings.
     String maxRequestsStr =
         (String)context.getProperty(MAX_REQUESTS);
     String maxQueueLengthStr =
         (String)context.getProperty(MAX_QUEUE_LENGTH);
     String requestTimeoutStr =
         (String)context.getProperty(REQUEST_TIMEOUT);
     // Set defaults.
     int maxRequests =
         mConcurrencySettings.getMaxRequests();
     int maxQueueLength = 
         mConcurrencySettings.getMaxQueueLength();
     long requestTimeout = 
         mConcurrencySettings.getRequestProcessingTimeout();
     // Extract and parse.
     if (maxRequestsStr != null)
     {
         try
         {
             maxRequests = Integer.valueOf(maxRequestsStr).intValue();
         }
         catch (NumberFormatException e)
         {
             throw new
                 DeploymentParamParseException(MAX_REQUESTS,
                                               maxRequestsStr,
                                               java.lang.Integer.class);
         }
     }
     if (maxQueueLengthStr != null) 
     {
         try
         {
             maxQueueLength = 
                 Integer.valueOf(maxQueueLengthStr).intValue();
         }
         catch (NumberFormatException e) 
         {
             throw new 
                 DeploymentParamParseException(MAX_QUEUE_LENGTH,
                                               maxQueueLengthStr,
                                               java.lang.Integer.class);
         }
     }
     if (requestTimeoutStr != null)
     {
         try 
         {
             requestTimeout = Long.valueOf(requestTimeoutStr).longValue();
         }
         catch (NumberFormatException e)
         {
             throw new DeploymentParamParseException(REQUEST_TIMEOUT,
                                                     requestTimeoutStr,
                                                     java.lang.Long.class);
         }
     }
     try 
     {
         mConcurrencySettings = new ConcurrencySettings(maxRequests,
                                                        maxQueueLength,
                                                        requestTimeout);
     }
     catch (IllegalArgumentException e)
     {
         throw new IllegalConcurrencySettingException(maxRequests,
                                                      maxQueueLength,
                                                      requestTimeout);
     }
     if (LOG.isDebugEnabled())
     {
         LOG.debug("OGSA-DAI request timeout: " + requestTimeout);
         LOG.debug("OGSA-DAI maximum number of requests: " + maxRequests);
         LOG.debug("OGSA-DAI maximum queue length: " + maxQueueLength);
     }
 }   

 // PERFORM and TERMINATE

 /**
  * Submit an OGSA-DAI perform document to a data service resource. 
  * 
  * @param performDocument
  *     OGSA-DAI perform document wrapped in service-layer bean.
  * @return OGSA-DAI response document wrapped in service-layer
  * bean. 
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to submit a request to this
  *     resource or to execute some action within their request.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the request does not validate against the perform
  *     document schema, contains an unsupported activity, contains
  *     an unconnected activity stream, contains an activity the
  *     user is not allowed to authorise or contains an
  *     incorrectly-specified activity.
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot 
  *     service this request.
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#perform(uk.org.ogsadai.types.ExtensibilityType)
  */
 public ExtensibilityType perform(ExtensibilityType performDocument)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType
 { 
     // Get the DataServiceResource to handle the request.
     DataServiceResource resource = getDataServiceResource();
     // Unpack the actual perform document from the operation
     // wrappers. 
     MessageElement[] elts = performDocument.get_any();
     if (elts.length < 1) 
     {
         throwRequestFault(new MissingPerformDocException());
     }
     Document performDoc = null;
     try 
     {
         performDoc = elts[0].getAsDocument();
     }
     catch (Exception e)
     {           
         throwInternalFault(e, false);
     }
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Extracted Perform document...");
     }
     // For SOAP Attachments to work properly in Axis you need to add them
     // just before returning response message, so the correct message
     // context will be used. Thats why this method needs to know about the
     // reference to the attachment manager that is forwarded to the
     // DataServiceResource.
     AttachmentManager attMgr = new AttachmentManager();

     // Now forward to the DataServiceResource.        
     Document responseDoc = perform(resource, performDoc, attMgr);
     
     // Add attachments to the response message that Axis engine is about
     // to send back to a client.
     attMgr.addAttachments(MessageContext.getCurrentContext().getResponseMessage());
     
     // Wrap up Response document for return to client.
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Wrapping Response document....");
     }
     ExtensibilityType responseType = new ExtensibilityType();
     MessageElement[] responseElts = new MessageElement[1];
     responseElts[0] = 
         new MessageElement(responseDoc.getDocumentElement());
     responseType.set_any(responseElts);
     return responseType;
 }
 
 /**
  * Submit an OGSA-DAI perform document to a data service resource. 
  * 
  * @param resource
  *     Targetted data service resource.
  * @param performDoc
  *     OGSA-DAI perform document.
  * @param attMgr
  *     Attachment manager.
  * @return OGSA-DAI response document.
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to submit a request to this
  *     resource or to execute some action within their request.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the request does not validate against the perform
  *     document schema, contains an unsupported activity, contains
  *     an unconnected activity stream, contains an activity the
  *     user is not allowed to authorise or contains an
  *     incorrectly-specified activity.
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot 
  *     service this request.
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  */
 private Document perform(DataServiceResource resource, 
                          Document performDoc, AttachmentManager attMgr)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType
 { 
     Document responseDoc = null;
     // Set up the context for the submission.
     SecurityContext securityContext = getSecurityContext();
     RequestContext context = new RequestContext(securityContext);
     context.setDataServiceResourceFactory(this);
     context.setURL(mURL);
     context.setResourceID(resource.getID().toString());
     
     // Add attachment manager
     context.setAttachmentManager(attMgr);
     context.setAuthorizer(mAuthority);

     if (LOG.isInfoEnabled())
     {
         LOG.info(MessageID.MSG_RECEIVED_CLIENT_RESOURCE_REQUEST,
                  new Object[]{securityContext.getDN(), 
                               resource.getID()});
     }
     // Submit the Perform document to the resource.
     try
     {
         responseDoc = resource.perform(performDoc, context);
     }
     catch (ResourceRequestAuthorizationException e)
     {
         throwAuthorisationFault(e);
     }
     catch (ResourceRequestExecutionException e)
     {
         throwInternalFault(e, false);
     }
     catch (ResourceRequestSpecificationException e)
     {
         throwRequestFault(e);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     if (LOG.isInfoEnabled())
     {
         LOG.info(MessageID.MSG_FORWARDED_CLIENT_RESOURCE_RESPONSE,
                  new Object[]{securityContext.getDN(), 
                               resource.getID()});
     }
     return responseDoc;
 }

 /**
  * Terminate the current request running on the resource.
  * <p>
  * <b>This operation currently does nothing</p>
  *  
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to submit a request to this
  *     resource or to execute some action within their request.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot 
  *     service this request.
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#terminate()
  */
 public void terminate() throws RemoteException,
     ResourceBusyFaultType, ResourceUnknownFaultType,
     InternalFaultType, AuthorisationFaultType
 { 
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Entered terminate...");
     }
 }

 // GET PROPERTIES
 
 /**
  * Obtain the value of a single property of a data service resource.
  * 
  * @param propertyName
  *     the resource property name in a bean.
  * @return resource property value in a bean.
  * @throws InvalidResourcePropertyQNameFaultType
  *     if the resource does not have property of the given name.
  * @throws ResourceUnknownFaultType
  *     if the resource is unknown to this service.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RemoteException
  *     Never.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getResourceProperty(javax.xml.namespace.QName)
  */
 public GetResourcePropertyResponse getResourceProperty(QName propertyName)
     throws RemoteException,
     uk.org.ogsadai.service.wsrp.ResourceUnknownFaultType,
     InvalidResourcePropertyQNameFaultType
 {
     if (LOG.isDebugEnabled()) 
     {            
         LOG.debug("Entered getResourceProperty...");
         LOG.debug("Requested property: " + propertyName);
     }        
     DataServiceResource resource = null;
     try 
     {
         resource = getDataServiceResource();
     } 
     catch (ResourceUnknownFaultType e) 
     {
         Calendar timestamp = 
             Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         throw new uk.org.ogsadai.service.wsrp.ResourceUnknownFaultType(
             timestamp, null, null, null);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }        
     GetResourcePropertyResponse resp = new GetResourcePropertyResponse();
     Properties properties = resource.getProperties();
     Property property = null;
     if (properties.containsKey(propertyName)) 
     {            
         property = (Property)properties.get(propertyName);
     }
     if (property == null) 
     {                
         // The property does not exist.
         Calendar timestamp =
             Calendar.getInstance(TimeZone.getTimeZone("GMT")); 
         throw new InvalidResourcePropertyQNameFaultType(
             timestamp, null, new String[] {propertyName.toString()}, null);
     }
     property.setSecurityContext(getSecurityContext());
     Object value = null;
     MessageElement[] any = null;
     // Process the activity value.
     try
     {
         value = property.getValue();
         if (value instanceof Collection)
         {
             // Create an element for each entry in the collection.
             any = new MessageElement[((Collection)value).size()];
             Object[] obj = ((Collection)value).toArray();
             int len = obj.length;
             any = new MessageElement[len];
             for (int i = 0; i < len; i++)
             {
                 any[i] = new MessageElement(propertyName, obj[i]);
             }
         }
         else if (value instanceof Object[])
         {
             // Create an element for each entry in the array.
             Object[] obj = (Object[])value;
             int len = obj.length;
             any = new MessageElement[len];
             for (int i = 0; i < len; i++)
             {
                 any[i] = new MessageElement(propertyName, obj[i]);
             }
         } 
         else if (value instanceof Element)
         {
             Element element = (Element)value;
             QName valueName = new QName(element.getNamespaceURI(),
                                         element.getLocalName());
             // If the property name matches the root element name
             // then avoid a doubly nested element with the same name.
             if (valueName.equals(propertyName))
             {
                 any = new MessageElement[]{new MessageElement(element)};
             }
             else 
             {
                 any = new MessageElement[]{
                     new MessageElement(propertyName, value)};
             }
         }
         else if (value instanceof Document)
         {
             Element element = ((Document)value).getDocumentElement();
             QName valueName = new QName(element.getNamespaceURI(),
                                         element.getLocalName());
             // If the property name matches the root element name
             // then avoid a doubly nested element with the same name.
             if (valueName.equals(propertyName))
             {
                 any = new MessageElement[]{new MessageElement(element)};
             }
             else 
             {
                 any = new MessageElement[]{
                     new MessageElement(propertyName, value)};
             }
         }
         else
         {
             any = new MessageElement[]{
                 new MessageElement(propertyName, value)};
         }
     }
     catch (Exception e) 
     {            
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     resp.set_any(any);
     return resp;
 }
 
 /**
  * Obtain the value of multiple properties of a data service
  * resource. 
  *
  * @param getMultipleResourcePropertiesRequest 
  *     Resource property names in a bean.
  * @return resource property values in a bean.
  * @throws InvalidResourcePropertyQNameFaultType 
  *     if the resource does not a have property of a given name.
  * @throws ResourceUnknownFaultType
  *     if the resource is unknown to this service.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RemoteException
  *     Never.
  *  @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getMultipleResourceProperties(uk.org.ogsadai.service.wsrp.GetMultipleResourcePropertiesRequest)
  */
 public GetMultipleResourcePropertiesResponse
     getMultipleResourceProperties(GetMultipleResourcePropertiesRequest
                                   getMultipleResourcePropertiesRequest)
     throws RemoteException,
            uk.org.ogsadai.service.wsrp.ResourceUnknownFaultType,
            InvalidResourcePropertyQNameFaultType 
 {
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered getResourceProperties...");
     }
     QName[] names = 
         getMultipleResourcePropertiesRequest.getResourceProperty();
     int numNames = names.length;
     List messageElements = new ArrayList();
     for (int i = 0; i < numNames; i++) 
     {
         GetResourcePropertyResponse answer =
             getResourceProperty(names[i]); 
         messageElements.addAll(Arrays.asList(answer.get_any()));
     }         
     MessageElement[] allAnswers = (MessageElement[])
         messageElements.toArray(new MessageElement[] {}); 
     // Now throw them back at the user.
     GetMultipleResourcePropertiesResponse resp = new
         GetMultipleResourcePropertiesResponse(); 
     resp.set_any(allAnswers);
     return resp;
 }

 // SECURITY CONTEXT
         
 /** 
  * Return a security context.
  * <p>
  * Provides a default with no security information.
  *
  * @return context.
  * @see uk.org.ogsadai.common.SecurityContextProvider#getSecurityContext()
  */
 public SecurityContext getSecurityContext()
 {
     return new SecurityContext(null, null);
 }

 // SERVICE INFORMATION
 
 /**
  * Return a list of the IDs of the data service resources
  * currently known to this service.
  *
  * @param listResourcesVoid 
  *     ignored. 
  * @return list of valid data service resource IDs.
  * @throws RemoteException 
  *     never.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  *  @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#listResources(java.lang.String)
  */
 public String[] listResources(String listResourcesVoid)
     throws RemoteException, InternalFaultType
 {
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered listResources...");
     }
     String[] names = null;
     try
     {
         // Convert keys to strings and fill response.
         Object[] keys    = mDataServiceResources.keySet().toArray();
         int      numKeys = keys.length;
         names   = new String[numKeys];
         for (int i = 0; i < numKeys; i++)
         {
             names[i] = keys[i].toString();
         }
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     return names;
 }
 
 /**
  * Return the OGSA-DAI version.
  *
  * @return OGSA-DAI version.
  * @throws RemoteException 
  *     never.
  * @throws InternalFaultType 
  *     never.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getVersion()
  */
 public String getVersion() throws RemoteException, InternalFaultType
 {
     if (LOG.isDebugEnabled()) 
     {
         LOG.debug("Entered getVersion...");
     }
     return mVersion;
 }

 // DATA TRANSPORT OPERATIONS
 
 /**
  * Put some data into a stream known to an active session of a
  * specific data service resource. 
  *
  * @param id
  *     Session ID and stream ID conjoined by ":".
  * @param data
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to access the resource or
  *     stream. 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the session or stream is unknown to the resource. 
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot
  *     service this request. 
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#putFully(java.lang.String, uk.org.ogsadai.types.ExtensibilityType)
  */
 public void putFully(String id, ExtensibilityType data)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType
 { 
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered putFully...");
     }
     SessionStreamID ssid = new SessionStreamID(id);
     DataServiceResource resource = getDataServiceResource();
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Put data on " + ssid.getStreamID()
                   + " stream of session " + ssid.getSessionID() 
                   + " of resource " + resource.getID());
     }
     Object block = null;
     try
     {
         block = SerializerUtils.toObject(data.get_any());
     }
     catch (DAISerializationException e)
     {
         throwInternalFault(e, false);
     }
     try
     {
         resource.putFully(ssid, block);                   
     }
     catch (ResourceRequestAuthorizationException e)
     {
         throwAuthorisationFault(e);
     }
     catch (ResourceRequestExecutionException e)
     {
         throwInternalFault(e, false);
     }
     catch (ResourceRequestSpecificationException e)
     {
         throwRequestFault(e);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Exiting putFully...");
     }
 }
 
 /**
  * Put part of a successive batch of data into a stream known to
  * an active session of a specific data service resource. 
  *
  * @param id
  *     Session ID and stream ID conjoined by ":".
  * @param data
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to access the resource or
  *     stream. 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the session or stream is unknown to the resource. 
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot
  *     service this request. 
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#putBlock(java.lang.String, uk.org.ogsadai.types.ExtensibilityType)
  */
 public void putBlock(String id, ExtensibilityType data)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType
 { 
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered putBlock...");
     }
     SessionStreamID ssid = new SessionStreamID(id);
     DataServiceResource resource = getDataServiceResource();
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Put data on " + ssid.getStreamID()
                   + " stream of session " + ssid.getSessionID() 
                   + " of resource " + resource.getID());
     }
     Object block = null;
     try
     {
         block = SerializerUtils.toObject(data.get_any());
     }
     catch (DAISerializationException e)
     {
         throwInternalFault(e, false);
     }
     try
     {
         resource.putBlock(ssid, block);                   
     }
     catch (ResourceRequestAuthorizationException e)
     {
         throwAuthorisationFault(e);
     }
     catch (ResourceRequestExecutionException e)
     {
         throwInternalFault(e, false);
     }
     catch (ResourceRequestSpecificationException e)
     {
         throwRequestFault(e);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Exiting putBlock...");
     }
 }
 
 /**
  * Get all the possible data from stream known to an active
  * session of a specific data service resource.
  * 
  * @param id 
  *     Session ID and stream ID conjoined by ":".
  * @return data
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to access the resource or
  *     stream. 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the session or stream is unknown to the resource. 
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot
  *     service this request. 
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getFully(java.lang.String)
  */
 public ExtensibilityType getFully(String id)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType
 { 
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered getFully...");
     }
     SessionStreamID ssid = new SessionStreamID(id);
     DataServiceResource resource = getDataServiceResource();
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Get data from " + ssid.getStreamID()
                   + " stream of session " + ssid.getSessionID() 
                   + " of resource " + resource.getID());
     }
     // Create response holder.
     ExtensibilityType result = new ExtensibilityType();        
     Object data = null;
     try
     {
         data = resource.getFully(ssid);
     }
     catch (ResourceRequestAuthorizationException e)
     {
         throwAuthorisationFault(e);
     }
     catch (ResourceRequestExecutionException e)
     {
         throwInternalFault(e, false);
     }
     catch (ResourceRequestSpecificationException e)
     {
         throwRequestFault(e);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     result.set_any(SerializerUtils.toMessage(OGSADAIConstants.DT_BLOCK_QNAME, data));
     data = null; // Garbage collect.
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Exiting getFully...");
     }
     return result;
 }
 
 /**
  * Get part of a batch of data from a stream known to an active
  * session of a specific data service resource.
  * 
  * @param id 
  *     Session ID and stream ID conjoined by ":".
  * @return the data
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to access the resource or
  *     stream. 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the session or stream is unknown to the resource. 
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot
  *     service this request. 
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getBlock(java.lang.String)
  */
 public ExtensibilityType getBlock(String id)
     throws RemoteException, ResourceBusyFaultType,
            ResourceUnknownFaultType, InternalFaultType,
            RequestFaultType, AuthorisationFaultType 
 { 
     return getNBlocks(id, 1);
 }

 /**
  * Get part of a batch of data from a stream known to an active
  * session of a specific data service resource.
  * 
  * @param id 
  *     Session ID and stream ID conjoined by ":".
  * @param numberOfBlocks 
  *     Number of blocks to retrieve.
  * @return the data.
  * @throws RemoteException 
  *     never.
  * @throws AuthorisationFaultType 
  *     if the client is not authorised to access the resource or
  *     stream. 
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws RequestFaultType 
  *     if the session or stream is unknown to the resource. 
  * @throws ResourceBusyFaultType 
  *     if the resource is processing a request and so cannot
  *     service this request. 
  * @throws ResourceUnknownFaultType 
  *     if the resource is unknown to this service.
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @see uk.org.ogsadai.service.wsi.dataservice.DataServicePortType#getNBlocks(java.lang.String, int)
  */
 public ExtensibilityType getNBlocks(String id, int numberOfBlocks)
     throws RemoteException, ResourceBusyFaultType,
     ResourceUnknownFaultType, InternalFaultType, RequestFaultType,
     AuthorisationFaultType
 { 
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Entered getNBlocks...");
     }
     SessionStreamID ssid = new SessionStreamID(id);
     DataServiceResource resource = getDataServiceResource();
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Get " + numberOfBlocks + " blocks of data from " + 
                   ssid.getStreamID() + " stream of session " + 
                   ssid.getSessionID() + " of resource " + 
                   resource.getID());
     }
     // Create response holder.
     ExtensibilityType result = new ExtensibilityType();        
     Object data = null;
     try
     {
         data = resource.getNBlocks(ssid, numberOfBlocks);
     }
     catch (ResourceRequestAuthorizationException e)
     {
         throwAuthorisationFault(e);
     }
     catch (ResourceRequestExecutionException e)
     {
         throwInternalFault(e, false);
     }
     catch (ResourceRequestSpecificationException e)
     {
         throwRequestFault(e);
     }
     catch (Exception e)
     {
         // Catch-all for any unanticipated problems.
         throwInternalFault(e, false);
     }
     result.set_any(SerializerUtils.toMessage(OGSADAIConstants.DT_BLOCK_QNAME, data));
     data = null; // Garbage collect.
     if (LOG.isDebugEnabled())
     {
         LOG.debug("Exiting getNBlocks...");
     }
     return result;
 }

 // FAULT PROCESSING

 /**
  * Throw information about a
  * <code>uk.org.ogsadai.exception.DAIException</code> 
  * back to the client in a <code>RequestFaultType</code>. If the
  * <code>DAIException</code> has child exceptions then these are
  * represented as child <code>ChainedFaultType</code>s.
  *
  * @param e 
  *    Server-side exception.
  * @throws RequestFaultType
  *    containing the information in the exception.
  */
 protected void throwRequestFault(DAIException e) throws
 RequestFaultType
 {
     // Log the exception.
     LOG.warn(e);
     // Create RequestFault.
     RequestFaultType fault = new RequestFaultType();
     // Add information about the top level exception.
     if (e instanceof DAIAuthorizationException) 
     { 
         addAuthzComplexFault(fault, (DAIAuthorizationException)e);
         // DAIAuthorizationException has been encountered so now
         // just exit.
         throw fault;
     }
     else 
     {
         addComplexFault(fault, e);
     }
     // Traverse child exceptions.
     Vector causesVector = new Vector();
     Throwable exception = e.getCause();
     while (exception != null)
     {
         ComplexFaultType complex = new ComplexFaultType();
         causesVector.add(complex);
         if (exception instanceof DAIAuthorizationException) 
         { 
             addAuthzComplexFault(complex, 
                                  (DAIAuthorizationException)exception);
             // DAIAuthorizationException has been encountered so now
             // just exit from the traversal.
             break;
         }
         else 
         {
             addComplexFault(complex, exception);
             // Get child exception.
             exception = exception.getCause();
         }
     }
     int numCauses = causesVector.size();
     ComplexFaultType[] causes = new ComplexFaultType[numCauses];
     for (int i = 0; i < numCauses; i++)
     {
         causes[i] = (ComplexFaultType)causesVector.get(i);
     }
     fault.setReason(causes);
     throw fault;
 }

 /** 
  * Adds information about an authorization exception to a
  * <code>ComplexFaultType</code> which is to be inserted into a 
  * <code>RequestFaultType</code>.
  *
  * @param complex
  *     <code>ComplexFaultType</code>
  * @param exception
  *     The authorization exception.
  */
 private void addAuthzComplexFault(ComplexFaultType complex,
                                   DAIAuthorizationException exception)
 {
     // Wrap so can internationalize in a standard way and hide
     // details of the exception from the client.
     AuthorisationFaultException authEx = 
         new AuthorisationFaultException((DAIAuthorizationException)exception);
     // Add the exception information.
     complex.setErrorID(authEx.getErrorID().toString());
     complex.setMsg(authEx.getLocalizedMessage());
     complex.setParams(new String[]{authEx.getParameters()[0].toString()});
 }

 /** 
  * Adds information about an exception to a
  * <code>ComplexFaultType</code> which is to be inserted into a 
  * <code>RequestFaultType</code>.
  *
  * @param complex
  *     <code>ComplexFaultType</code>
  * @param exception
  *     The exception.
  */
 private void addComplexFault(ComplexFaultType complex,
                              Throwable exception)
 {
     DAIExceptionInformation daiEx = null;
     if (exception instanceof DAIExceptionInformation)
     {
         daiEx = (DAIExceptionInformation)exception;
     }
     else
     {
         NonOGSADAIExceptionWrapper wrapper = 
             new NonOGSADAIExceptionWrapper(exception);
         daiEx = (DAIExceptionInformation)wrapper;
     }
     String errorID = daiEx.getErrorID().toString();
     Object[] params = daiEx.getParameters();
     String msg = exception.getLocalizedMessage();
     // Get the information about the exception...
     complex.setErrorID(errorID);
     complex.setMsg(msg);
     int numParams = 0;
     if (params != null)
     {
         numParams = params.length;
     }
     String[] stringParams = new String[numParams];
     for (int i = 0; i < numParams; i++)
     {
         stringParams[i] = params[i].toString();
     }
     complex.setParams(stringParams);
 }

 /**
  * Throw a <code>AuthorisationFaultType</code> back to the
  * client. 
  *
  * @param exception 
  *     Server-side exception with information about the
  *     authorisation fault. 
  * @throws AuthorisationFaultType 
  *     containing the unique ID of the exception so the client can
  *     contact the service deployer if required. 
  */
 protected void throwAuthorisationFault(DAIAuthorizationException exception)
     throws AuthorisationFaultType
 {
     // Log the authorisation exception.
     LOG.warn(exception);
     // Wrap so can internationalize in a standard way and hide
     // details of the exception from the client.
     AuthorisationFaultException authEx = 
         new AuthorisationFaultException(exception);
     // Throw the fault to the client.
     throw new AuthorisationFaultType(authEx.getErrorID().toString(), 
                                      authEx.getLocalizedMessage(),
                                      authEx.getParameters()[0].toString());
 }
 
 /**
  * Throw a <code>ResourceUnknownFaultType</code> back to the
  * client. 
  *
  * @param resourceID 
  *     ID of the unknown resource.
  * @throws ResourceUnknownFaultType
  *     containing the ID of the unknown resource.
  */
 protected void throwResourceUnknownFault(String resourceID) throws
 ResourceUnknownFaultType 
 {
     ResourceUnknownException ex = 
         new ResourceUnknownException(resourceID);
     // Log the unknown resource.
     LOG.warn(ex);
     // Create server-side log message.
     // Throw the fault to the client.
     throw new ResourceUnknownFaultType(ex.getErrorID().toString(), 
                                        ex.getLocalizedMessage(),
                                        resourceID);
 }

 /**
  * Throw an <code>InternalFaultType</code> - raised in response to 
  * a server-side problem - back to the client.
  *
  * @param exception 
  *     Server-side exception.
  * @param isFatal
  *     Is the error fatal? If so then log as <code>fatal</code> else
  *     log as <code>error</code>.
  * @throws InternalFaultType 
  *     containing the unique ID of the exception so the client can
  *     contact the service deployer if required. 
  */
 protected void throwInternalFault(Throwable exception, boolean isFatal)
     throws InternalFaultType 
 {
     // Wrap internal exception.
     ServerException serverEx = new ServerException(exception);
     if (isFatal)
     {
         LOG.fatal(serverEx, true);
     }
     else
     {
         LOG.error(serverEx, true);
     }
     // Wrap so can internationalize in a standard way and hide
     // details of the exception from the client.
     InternalFaultException internalEx = 
         new InternalFaultException(serverEx);
     // Throw the fault to the client.
     throw new InternalFaultType(internalEx.getErrorID().toString(), 
                                 internalEx.getLocalizedMessage(),
                                 internalEx.getParameters()[0].toString());

 }

 // DATA SERVICE RESOURCE CREATION / DESTRUCTION and CALLBACKS
 
 /**
  * From the current message and request context get the 
  * <code>DataServiceResource</code> specified by the client.
  *
  * @return DataServiceResource
  * @throws InternalFaultType 
  *     if some internal problem occurs that is not under the
  *     client's control.
  * @throws ResourceUnknownFaultType 
  * @throws AuthorisationFaultType
  *     if the client is not authorize to access the resource
  * if the resource is unknown to this service.
  */
 protected DataServiceResource getDataServiceResource() throws
 ResourceUnknownFaultType, InternalFaultType, AuthorisationFaultType
 {
     MessageContext msgContext = MessageContext.getCurrentContext();
     if (msgContext == null) 
     {
         MessageContextException e = new MessageContextException();
         throwInternalFault(e, true);
     }            
     String id = 
         (String)msgContext.getProperty(ResourceIDHandler.RESOURCE_ID_KEY);
     if ((id == null) || (! mDataServiceResources.containsKey(id)))
     {
         throwResourceUnknownFault(id);
     }
     DataServiceResource dataServiceResource = 
         (DataServiceResource)mDataServiceResources.get(id);

     authorizeResource(dataServiceResource, getSecurityContext());
     
     return dataServiceResource;
 }

 /**
  * Creates a data service resource using configuration details
  * from the resource's configuration directory.
  * 
  * @param id 
  *      The ID of the resource to create.  This ID should be unique
  *      within this container.  
  * 
  * @return The ID of the newly created resource.
  * 
  * @throws ResourceIDAlreadyAssignedException
  *          If the resource ID has already been assigned so
  *          a new resource with the same ID cannot be created.
  * 
  * @throws ResourceCreationException
  *           If an error occured while attempting to create the resource.
  */
 protected Object createResourceFromConfigDirectory(Object id)
     throws 
         ResourceIDAlreadyAssignedException,
         ResourceCreationException
 {
     DSRComponentFactory componentFactory;
     try
     {
         componentFactory = new FileBasedDSRComponentFactory(
             id.toString(), new File(mConfigDir));
     }
     catch(DAIException e)
     {
         throw new ResourceCreationException(id.toString(), e);
     }
     
     return createResourceFromComponentFactory(id,componentFactory);
 }
 
 /**
  * Creates a data service resource using a data service resource component
  * factory to generate the required components of the resource.
  * 
  * @param id 
  *      The ID of the resource to create.  This ID should be unique
  *      within this container.  
  * 
  * @param componentFactory
  *      A factory to create the require data service resource components.
  * 
  * @return The ID of the newly created resource.
  * 
  * @throws ResourceIDAlreadyAssignedException
  *          If the resource ID has already been assigned so
  *          a new resource with the same ID cannot be created.
  * 
  * @throws ResourceCreationException
  *           If an error occured while attempting to create the resource.
  */
 protected Object createResourceFromComponentFactory(
     Object id,
     DSRComponentFactory componentFactory)
     throws ResourceIDAlreadyAssignedException,
            ResourceCreationException
 {
     // Create the data service resource components
     ActivitiesConfig activitiesConfig;
     DataResourceAccessor dataResourceAccessor;
     SessionComponentFactory sessionComponentFactory;
     try
     {
         activitiesConfig = 
             componentFactory.getActivityConfiguration();
         dataResourceAccessor =
             componentFactory.getDataResourceAccessor();
         sessionComponentFactory =
             componentFactory.getSessionComponentFactory();
     }
     catch(DAIException e)
     {
         throw new ResourceCreationException(id.toString(), e);
     }

     return createResourceFromComponents(
         id, 
         dataResourceAccessor, 
         activitiesConfig,
         sessionComponentFactory);
 }
 
 /**
  * Creates a data service resource that is made up of the given components.
  * 
  * @param id 
  *      The ID of the resource to create.  This ID should be unique
  *      within this container.  
  * 
  * @param dataResourceAccessor
  *      The data resource accessor that gives access to any underlying data
  *      resource.
  * 
  * @param activitiesConfig
  *      Configuration details regarding the activity that can be used with
  *      the resource.
  * 
  * @param sessionComponentFactory
  *      A factory to create the session components required by a data 
  *      service resource.
  * 
  * @return The ID of the newly created resource.
  * 
  * @throws ResourceIDAlreadyAssignedException
  *          If the resource ID has already been assigned so
  *          a new resource with the same ID cannot be created.
  * 
  * @throws ResourceCreationException
  *           If an error occured while attempting to create the resource.
  */
 protected Object createResourceFromComponents(
     Object id,
     DataResourceAccessor dataResourceAccessor, 
     ActivitiesConfig activitiesConfig, 
     SessionComponentFactory sessionComponentFactory)
         throws 
             ResourceIDAlreadyAssignedException,
             ResourceCreationException
 {
     // Need to synchronize between reading that there is not a
     // resource with the same ID and inserting the resource.
     synchronized(mDataServiceResources)
     {
         // Check that there is not a data service resource with this ID. 
         if (mDataServiceResources.containsKey(id))
         {
             throw new ResourceIDAlreadyAssignedException(id);
         }
         
         // Create data service resource.
         DataServiceResource resource =
             new DataServiceResource(
                 id,
                 activitiesConfig, 
                 dataResourceAccessor,
                 sessionComponentFactory, 
                 mConcurrencySettings,
                 mAuthority);
         mDataServiceResources.put(id, resource);
         if (LOG.isInfoEnabled())
         {
             LOG.info(MessageID.MSG_DEPLOYED_DATA_SERVICE_RESOURCE,
                      new Object[]{id});
         }
         return id;
     }
 }

     

 /**
  * Remove a data service resource with the given ID
  * 
  * @param id 
  *     ID of the resource to remove
  * @throws NoSuchResourceIDException 
  *     if the resource ID is unknown
  */
 protected void removeResource(Object id) throws NoSuchResourceIDException
 {
     // Need to synchronize between reading that there is not a
     // resource with the same ID and removing the resource.
     synchronized(mDataServiceResources)
     {
         // If the resource exists, remove it 
         if (mDataServiceResources.containsKey(id)) 
         {
             mDataServiceResources.remove(id);            
             if (LOG.isInfoEnabled())
             {
                 LOG.info(MessageID.MSG_UNDEPLOYED_DATA_SERVICE_RESOURCE,
                          new Object[]{id});
             }
         }
         else
         {
             throw new NoSuchResourceIDException(id);
         }
     }
 }

 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#getNewResourceName()
  */
 public String getNewResourceName()
 {
     return mResourceFactoryUtil.getNewResourceName();
 }
 
 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#getNewResourceName(java.lang.String)
  */
 public String getNewResourceName(String prefix)
 {
     return mResourceFactoryUtil.getNewResourceName(prefix);
 }

 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#getResourceConfigDirectory(java.lang.String)
  */
 public File getResourceConfigDirectory(String resourceName) 
     throws DynamicDataServiceResourceCreationException
 {
     try
     {
         return mResourceFactoryUtil.createResourceConfigDirectory(resourceName);
     }
     catch (DAIException e) 
     {
         throw new DynamicDataServiceResourceCreationException(resourceName,
                                                               e);
     }
 }
 
 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#createResource(java.lang.String, java.lang.String)
  */
 public void createResource(String resourceName, 
                            String dataResourceClassName)
     throws DynamicDataServiceResourceCreationException
 {
     try
     {
         // Write the class config file to the config directory.
         mResourceFactoryUtil.writeClassConfigFile(resourceName,
                                                   dataResourceClassName);
         // Update all the tokens in the configuration files with the
         // appropriate strings.
         mResourceFactoryUtil.replaceTokensInAllConfigFiles(resourceName);
         // Now we have to add the data resource to the collection.
         createResourceFromConfigDirectory(resourceName);
         // Persist the resource by writing to file
         mResourceFactoryUtil.persistResource(resourceName,
                                              new File(mResourcesFile));
     }
     catch (DAIException e) 
     {
         throw new DynamicDataServiceResourceCreationException(resourceName,
                                                               e);
     }
 }

 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#removeResource(java.lang.String, boolean removeConfiguration)
  */
 public void removeResource(String resourceName, 
                            boolean removeConfiguration)
     throws DynamicDataServiceResourceRemovalException
 {
     try
     {
         // Remove the resource from the collection.
         removeResource(resourceName);
         // Remove resource from the persistent resources file.
         mResourceFactoryUtil.removePersistentResource(resourceName,
                                                       new File(mResourcesFile));
         if (removeConfiguration)
         {
             mResourceFactoryUtil.removeResourceConfigDirectory(resourceName);
         }
     }
     catch (DAIException e) 
     {
         throw new DynamicDataServiceResourceRemovalException(resourceName,
                                                              e);
     }
 }

 /* (non-Javadoc)
  * @see uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory#createResource(java.lang.String, uk.org.ogsadai.dataresource.DataResource, uk.org.ogsadai.service.resource.config.ActivitiesConfig, uk.org.ogsadai.sessions.SessionComponentFactory)
  */
 public String createResource(
     String namePrefix, 
     DataResourceAccessor dataResourceAccessor, 
     ActivitiesConfig activitiesConfig, 
     SessionComponentFactory sessionComponentFactory) 
        throws 
            ResourceCreationException
 {
     String name = mResourceFactoryUtil.getNewResourceName(namePrefix);
     try
     {
         Object id = createResourceFromComponents(
             name,
             dataResourceAccessor,
             activitiesConfig,
             sessionComponentFactory);
         
         dataResourceAccessor.setResourceName(id.toString());
         
         return id.toString();
     }
     catch(ResourceIDAlreadyAssignedException e)
     {
         // This error should not occur as we generate a unique name above.
         throw new ResourceCreationException(name, e);
     }
 }
 
 /**
  * Constructs a new authorization file.
  * 
  * @param configFile
  *            path of the configuration file
  * @throws AuthorizationInitialisationException
  *             if the authorization could not be created
  */
 private void loadAuthorization(String configFile)
 throws AuthorizationInitialisationException 
 {
     mAuthority = 
         AuthorizationFactory.createAuthorization(configFile);
 }
 
 /**
  * Authorizes a resource for access within the given security context.
  * 
  * @param resource
  *            data service resource to access
  * @param context
  *            security context of the request
  * @throws AuthorisationFaultType
  *             if the user is not authorized the data service resource
  */
 private void authorizeResource(
         DataServiceResource resource, 
         SecurityContext context) 
 throws AuthorisationFaultType 
 {
     if (!mAuthority.authorizeResource(
             resource.getID().toString(), context)) {
         throwAuthorisationFault(
                 new ResourceAuthorizationException(
                         context.getDN(),
                         resource.getID().toString()));
     }
 }
 
  
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType getDataResourcePropertyDocument(uk.org.ogsadai.dais.xml.service.core.GetDataResourcePropertyDocumentRequest getDataResourcePropertyDocumentRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType
 {
     return daixImpl.getDataResourcePropertyDocument(getDataResourcePropertyDocumentRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceResponse destroyDataResource(uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceRequest destroyDataResourceRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType
 {
     return daixImpl.destroyDataResource(destroyDataResourceRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.GenericQueryResponse genericQuery(uk.org.ogsadai.dais.xml.service.core.GenericQueryRequest genericQueryRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType
 {
     return daixImpl.genericQuery(genericQueryRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.GetResourceListResponse getCollectionServiceResourceList(uk.org.ogsadai.dais.xml.service.core.GetResourceListRequest getResourceListRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType {
     return daixImpl.getCollectionServiceResourceList(getResourceListRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.GetResourceListResponse getDocumentServiceResourceList(uk.org.ogsadai.dais.xml.service.core.GetResourceListRequest getResourceListRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType {
     return daixImpl.getDocumentServiceResourceList(getResourceListRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.GetResourceListResponse getSequenceServiceResourceList(uk.org.ogsadai.dais.xml.service.core.GetResourceListRequest getResourceListRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType {
     return daixImpl.getSequenceServiceResourceList(getResourceListRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.ResolveResponse resolve(uk.org.ogsadai.dais.xml.service.core.ResolveRequest resolveRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.resolve(resolveRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType getCollectionPropertyDocument(uk.org.ogsadai.dais.xml.service.core.GetDataResourcePropertyDocumentRequest getCollectionPropertyDocumentRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.getCollectionPropertyDocument(getCollectionPropertyDocumentRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.AddDocumentsResponse addDocuments(uk.org.ogsadai.dais.xml.services.daix.AddDocumentsRequest addDocumentsRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.addDocuments(addDocumentsRequest);
 }
 
 /** WS-DAIX method */
  public uk.org.ogsadai.dais.xml.services.daix.GetDocumentsResponse getDocuments(uk.org.ogsadai.dais.xml.services.daix.GetDocumentsRequest getDocumentsRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
    return daixImpl.getDocuments(getDocumentsRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsResponse removeDocuments(uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsRequest removeDocumentsRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.removeDocuments(removeDocumentsRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionResponse createSubcollection(uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionRequest createSubcollectionRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.services.daix.CollectionAlreadyExistsFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.createSubcollection(createSubcollectionRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionResponse removeSubcollection(uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionRequest removeSubcollectionRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.removeSubcollection(removeSubcollectionRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.AddSchemaResponse addSchema(uk.org.ogsadai.dais.xml.services.daix.AddSchemaRequest addSchemaRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.services.daix.SchemaInvalidFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.services.daix.SchemaAdditionMakesDocumentsInvalidFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.addSchema(addSchemaRequest);
 }
 
 public uk.org.ogsadai.dais.xml.services.daix.GetSchemaResponse getSchema(uk.org.ogsadai.dais.xml.services.daix.GetSchemaRequest getSchemaRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.services.daix.SchemaDoesNotExistFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType {
    return daixImpl.getSchema(getSchemaRequest);
}
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaResponse removeSchema(uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaRequest removeSchemaRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.services.daix.SchemaRemovalMakesDocumentsInvalidFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.services.daix.SchemaDoesNotExistFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.removeSchema(removeSchemaRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType collectionSelectionFactory(uk.org.ogsadai.dais.xml.services.daix.CollectionSelectionFactoryRequest collectionSelectionFactoryRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.collectionSelectionFactory(collectionSelectionFactoryRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType documentSelectionFactory(uk.org.ogsadai.dais.xml.services.daix.DocumentSelectionFactoryRequest documentSelectionFactoryRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.DocumentDoesNotExistFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.documentSelectionFactory(documentSelectionFactoryRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.XMLSequencePropertyDocumentType getXMLSequencePropertyDocument(uk.org.ogsadai.dais.xml.service.core.GetDataResourcePropertyDocumentRequest getXMLSequencePropertyDocumentRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.getXMLSequencePropertyDocument(getXMLSequencePropertyDocumentRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.GetItemsResponse getItems(uk.org.ogsadai.dais.xml.services.daix.GetItemsRequest getItemsRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.services.daix.InvalidStartPositionFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.services.daix.InvalidCountFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.getItems(getItemsRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.XPathExecuteResponse XPathExecute(uk.org.ogsadai.dais.xml.services.daix.XPathExecuteRequest XPathExecuteRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.services.daix.XPathFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.XPathExecute(XPathExecuteRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType XPathExecuteFactory(uk.org.ogsadai.dais.xml.services.daix.XPathExecuteFactoryRequest XPathExecuteFactoryRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType, uk.org.ogsadai.dais.xml.services.daix.XPathFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.XPathExecuteFactory(XPathExecuteFactoryRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteResponse XUpdateExecute(uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteRequest XUpdateExecuteRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.services.daix.XUpdateFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.XUpdateExecute(XUpdateExecuteRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteResponse XQueryExecute(uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteRequest XQueryExecuteRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.services.daix.XQueryFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.XQueryExecute(XQueryExecuteRequest);
 }
 
 /** WS-DAIX method */
 public uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType XQueryExecuteFactory(uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteFactoryRequest XQueryExecuteFactoryRequest) throws java.rmi.RemoteException, uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType, uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType, uk.org.ogsadai.dais.xml.services.daix.XQueryFaultType, uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType, uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType, uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType {
     return daixImpl.XQueryExecuteFactory(XQueryExecuteFactoryRequest);
 }
 
}
