/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.perform;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import uk.org.ogsadai.common.UniqueName;
import uk.org.ogsadai.common.xml.XMLParseException;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.service.OGSADAIConstants;

/**
 * 
 * Encapsulates the OGSA-DAI perform documents used
 * to execute WS-DAIX operations. 
 * 
 * This class is extended by activity specific perform
 * documents.  
 * 
 * @author Steven Lynden
 * 
 */
public class PerformDocument {

    /** The perform document XML document */
    Document perform;
    
    /** The root element of the perform document */
    Element root;
    
    /** The activity embedded in the perform document */
    Element activity;
    
    /**
     * Constructs a new perform document with the given name 
     */
    public PerformDocument(String name) throws ParserConfigurationException {
        perform = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        root = perform.createElement("perform");
        perform.appendChild(root);
        activity = createElement(name);
        activity.setAttribute("name",name+"-" + UniqueName.getInstance().getName());
        root.appendChild(activity);
    }
    
    /**
     * Creates an element with the given name
     */
    protected Element createElement(String name) {
        return perform.createElementNS( OGSADAIConstants.TYPES_NAMESPACE ,name );
    }
    
    /**
     * Constructs a text element with the given name and text content
     */
    protected Element createElementText(String name, String text) {
        Element e = createElement(name);
        Node n = perform.createTextNode(text);
        e.appendChild(n);
        return e;
    }
    
    /**
     * Creates a resourceSetStream element
     * with the given name
     */
    protected Element createResourceSetElement(String name) {
        Element e = createElement("resourceSetStream");
        e.setAttribute("name",name);        
        return e;
    }
    
    /**
     * Creates an outputStream element with the
     * given name
     */
    protected Element createOutputStreamElement(String name) {
        Element e = createElement("outputStream");
        e.setAttribute("name",name);        
        return e;
    }
    
    /**
     * Returns the perform document XML 
     */
    public Document getDocument() {
        return this.perform;   
    }
    
    /**
     * Return the results embedded within a response document as an XML document
     */
    public static Document getResultsAsDocument(Document responseDoc, String outputName) {
        String results = getResults(responseDoc,outputName);
        try {
            return XMLUtilities.xmlStringToDOM(results,false);
        } catch ( XMLParseException e ) { }
        return null;    
    }
    
    /**
     * Return the results embedded within a response document as a string
     */
    private static String getResults(Document responseDoc, String outputName) {
        NodeList results = responseDoc.getElementsByTagNameNS(OGSADAIConstants.TYPES_NAMESPACE,"result");
        for(int i=0;i<results.getLength();i++) {
            if ( ((Element)results.item(i)).getAttribute("name").equals(outputName) ) {
                return results.item(i).getFirstChild().getNodeValue(); 
            }
        }
        return null;
    }
    
}
