/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.porttypes;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import uk.org.ogsadai.common.SecurityContext;
import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.FactoryRequestType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.engine.RequestContext;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.config.ActivitiesConfig;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.sessions.SessionConfigurationLoader;
import uk.org.ogsadai.wsdaix.perform.PerformDocument;
import uk.org.ogsadai.wsdaix.resource.DAIXDataResourceAccessor;
import uk.org.ogsadai.wsdaix.resource.ParentXMLResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLDocumentResource;
import uk.org.ogsadai.wsdaix.resource.XMLCollectionResource;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;

/**
 * 
 * The base class for WS-DAIX port type implementations
 * 
 * @author Steven Lynden
 *
 */
public class PortTypeImpl {

    private static Logger mLog = Logger.getLogger(PortTypeImpl.class);
    
    /** Used to create new resources */
    protected DataServiceResourceFactory factory;
    
    /** Values loaded from the deployment properties file */
    protected DeploymentProperties deploymentProperties;
     
    /**
     * Invoked by sub-classes which implement specific port types
     */
    protected PortTypeImpl(DataServiceResourceFactory factory, DeploymentProperties deploymentProperties) {
        this.factory = factory;
        this.deploymentProperties = deploymentProperties;
    }
     
    /**
     * Expose an new OGSA-DAI data service resource
     */
    protected String exposeResource(DAIXDataResourceAccessor child, ActivitiesConfig activityConfig, SessionConfigurationLoader sessionConfig, DeploymentProperties deploymentProperties, DataResourceAddressType parentAddress) throws DAIException {
       child.setDeploymentProperties(deploymentProperties);
       child.setParentDataResource(parentAddress);
       return factory.createResource("ws",child,activityConfig,sessionConfig);
    }
     
    /**
     * Executes an OGSA-DAI perform document using the engine and
     * return a response document
     */
    protected Document perform(DataServiceResource resource, PerformDocument perform) throws DAIException {
       // Set up the context for the submission.
       if ( mLog.isDebugEnabled() )
           mLog.debug("Perform document: " + XMLUtilities.xmlDOMToString(perform.getDocument()));
       SecurityContext securityContext = new SecurityContext(null, null);
       RequestContext context = new RequestContext(securityContext);
       context.setDataServiceResourceFactory(factory);
       context.setURL(deploymentProperties.getServicePath());
       context.setResourceID(resource.getID().toString());   
       return resource.perform(perform.getDocument(), context);
    }
 
    /**
     * Used by factory operations.
     * Returns:
     * - The port type QName specified by the request (if valid & specified)
     * - The default port type QName if one isn't requested
     * Throws the relevant fault if an invalid port type QName is specified
     * by the request message
     */
    protected static ConfigurationMapType getConfigurationMap(ParentXMLResource resource, QName message, FactoryRequestType request) throws DataResourceUnavailableFaultType, InvalidPortTypeQNameFaultType {
       ConfigurationMapType[] configMap = null;
       if ( resource instanceof XMLCollectionResource ) {
           configMap = ((XMLCollectionResource)resource).getDataResourcePropertyDocument().getConfigurationMap();
       } else if ( resource instanceof TransientXMLCollectionResource ) {
           configMap = ((TransientXMLCollectionResource)resource).getDataResourcePropertyDocument().getConfigurationMap();
       } else if ( resource instanceof TransientXMLDocumentResource ) {
           configMap = ((TransientXMLDocumentResource)resource).getDataResourcePropertyDocument().getConfigurationMap();
       }
       //if the request specifies a port type qname, need to check that it exists in the config map
       if ( request.getPortTypeQName() != null  ) {
           for(int i=0;i<configMap.length;i++) {
               if ( configMap[i].getMessageQName().equals(message) )
                   if ( configMap[i].getPortTypeQName().equals( request.getPortTypeQName() ) )
                       return configMap[i];
           }
           throw new InvalidPortTypeQNameFaultType();
       }
       //otherwise, return the first entry in the config map
       for(int i=0;i<configMap.length;i++) {
           if ( configMap[i].getMessageQName().equals(message) )
               return configMap[i];
       }
       mLog.error("Unexpected FATAL error, unable to get configuration map for collection selection factory operation");
       throw new DataResourceUnavailableFaultType();
    }
        
}