/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.porttypes;

import java.rmi.RemoteException;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.common.UniqueName;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentRequestWrapper;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentResponseWrapper;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentResponseWrapperResponse;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.CollectionAlreadyExistsFaultType;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentResponseWrapper;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentResponseWrapperResponse;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentRequestWrapper;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentResponseWrapper;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentResponseWrapperResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.XMLWrapperType;
import uk.org.ogsadai.dais.xml.services.daix.XPathFaultType;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.wsdaix.common.CollectionAccessor;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;
import uk.org.ogsadai.wsdaix.perform.AddDocumentPerformDocument;
import uk.org.ogsadai.wsdaix.perform.CreateCollectionPerformDocument;
import uk.org.ogsadai.wsdaix.perform.PerformDocument;
import uk.org.ogsadai.wsdaix.perform.RemoveCollectionPerformDocument;
import uk.org.ogsadai.wsdaix.perform.RemoveDocumentPerformDocument;
import uk.org.ogsadai.wsdaix.perform.XPathExecutePerformDocument;
import uk.org.ogsadai.wsdaix.resource.DAIXXMLDBResource;

/**
 * 
 * Implementation of the WS-DAIX XMLCollectionAccess port type
 * 
 * @author Steven Lynden
 *
 */
public class XMLCollectionAccessImpl extends PortTypeImpl {

    private static Logger mLog = Logger.getLogger(XMLCollectionAccessImpl.class);
    
    private CollectionAccessor collectionAccessor;
    
    /**
     * Initialse the port type
     */
    public XMLCollectionAccessImpl(DataServiceResourceFactory factory, DeploymentProperties deploymentProperties, CollectionAccessor collectionAccessor) {
        super(factory,deploymentProperties);
        this.collectionAccessor = collectionAccessor;
    }   
    
    /*
     * Uses the mapping between OGSA-DAI and WS-DAIX collections
     * to return a collection path string given a URI
     */
    private String getCollectionPath(URI collectionURI) throws InvalidCollectionNameFaultType {
        String path = collectionAccessor.getCollectionMapping().URIToPath(collectionURI);
        if ( path == null ) throw new InvalidCollectionNameFaultType();
        return path;
    }
    
    /**
     * Implementation of CollectionAccess::GetDocuments
     */
    public GetDocumentsResponse getDocuments(GetDocumentsRequest getDocumentsRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType {
        DAIXXMLDBResource dra = (DAIXXMLDBResource) resource.getDataResourceAccessor();
        String collectionPath = null;
        if ( getDocumentsRequest.getCollectionName() != null ) {
            collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
                getDocumentsRequest.getCollectionName(),        
                dra.getCollectionName()
            );
            if ( collectionPath == null ) throw new InvalidCollectionNameFaultType();
        }
        int size = getDocumentsRequest.getGetDocumentRequestWrapper().length;
        GetDocumentResponseWrapper[] responseWrapper = new GetDocumentResponseWrapper[size];
        for(int i=0;i<size;i++) {
            String outputName = "XPathStatement-" + UniqueName.getInstance().getName();
            PerformDocument performDoc = null;
            String docName = getDocumentsRequest.getGetDocumentRequestWrapper(i).getDocumentName();
            String expression = "/";
            try {
                performDoc = new XPathExecutePerformDocument(collectionPath,docName,null,expression,outputName);
            } catch ( ParserConfigurationException ex ) {
                throw new XPathFaultType();   
            }
            try {
                Document responseDoc = PerformDocument.getResultsAsDocument(perform(resource,performDoc),outputName);
                Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
                NodeList ns = responseDoc.getElementsByTagName("resource").item(0).getChildNodes();
                //the retrieved document will be the the only Element object
                for(int j=0;j<ns.getLength();j++) {
                    if ( ns.item(j) instanceof Element ) {
                        doc.appendChild(doc.importNode(ns.item(j),true));
                        break;
                    }
                }
                responseWrapper[i] = new GetDocumentResponseWrapper();
                XMLWrapperType wrapper = new XMLWrapperType();
                MessageElement msgElem = new MessageElement(doc.getDocumentElement());
                wrapper.set_any(new MessageElement[] {msgElem} );
                responseWrapper[i].setData(wrapper);
                responseWrapper[i].setDocumentName(docName);
                responseWrapper[i].setResponse(GetDocumentResponseWrapperResponse.value1);
            } catch ( Exception e ) {
                mLog.error("Unable to execute perform document: " + e);
                responseWrapper[i].setDocumentName(docName);
                responseWrapper[i] = new GetDocumentResponseWrapper();
                responseWrapper[i].setResponse(GetDocumentResponseWrapperResponse.value2);
            }            
        }
        GetDocumentsResponse response = new GetDocumentsResponse(); 
        response.setGetDocumentResponseWrapper(responseWrapper);
        return response;    
    }
    
    /**
     * Implementation of CollectionAccess::AddDocuments
     */
    public AddDocumentsResponse addDocuments(AddDocumentsRequest addDocumentsRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType {
        mLog.debug("Adding documents to collection");
        DAIXXMLDBResource dra = (DAIXXMLDBResource) resource.getDataResourceAccessor();
        String collectionPath = null;
        if ( addDocumentsRequest.getCollectionName() != null ) {
            collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
                addDocumentsRequest.getCollectionName(),        
                dra.getCollectionName()
            );
            if ( collectionPath == null ) throw new InvalidCollectionNameFaultType();
        }
        mLog.debug("Collection path: " + collectionPath);
        AddDocumentRequestWrapper[] docs = addDocumentsRequest.getAddDocumentRequestWrapper();
        AddDocumentResponseWrapper[] responseWrappers = new AddDocumentResponseWrapper[docs.length]; 
        for(int i=0;i<docs.length;i++) {
            String docName = docs[i].getDocumentName();
            boolean exists = false;
            //check that the document exists, if it does then indicate that it has been overwritten
            if ( dra.resourceExists(collectionPath,docName) ) {
                mLog.debug("Resource " + docName + " already exists!");
                exists = true;
            }
            responseWrappers[i] = new AddDocumentResponseWrapper();
            responseWrappers[i].setDocumentName(docName);
            //repsponse wrapper values
            //1 - <xsd:enumeration value="Success"/> 
            //2 - <xsd:enumeration value="DocumentNotAdded-DocumentDoesNotValidate"/> 
            //3 - <xsd:enumeration value="DocumentNotAdded-NotAuthorized"/>      
            //4 - <xsd:enumeration value="DocumentOfSameNameOverwritten"/>
            String output = "AddDocument-" + UniqueName.getInstance().getName();
            MessageElement elem = docs[i].getData().get_any()[0];
            Object o = null;
            try { o = elem.getAsDocument(); } catch ( Exception e ) { }
            AddDocumentPerformDocument pd = null;
            if (! (o instanceof Document)) mLog.error("Unable to add document " + docName + " because of a problem deserializing the document");
            try {
                pd = new AddDocumentPerformDocument(collectionPath,docName,((Document)o).getDocumentElement(),output);
            } catch ( ParserConfigurationException e ) {
                mLog.error("Error creating perform document " + e);
                responseWrappers[i].setResponse(AddDocumentResponseWrapperResponse.value3);
                continue;
            }
            try {
                perform(resource,pd);
            } catch ( Exception e ) {
                mLog.error("Unable to execute perform document: " + e);
                responseWrappers[i].setResponse(AddDocumentResponseWrapperResponse.value3);
                continue;
            }
            if ( exists )
                responseWrappers[i].setResponse(AddDocumentResponseWrapperResponse.value4);
            else    
                responseWrappers[i].setResponse(AddDocumentResponseWrapperResponse.value1);
        }
        AddDocumentsResponse r = new AddDocumentsResponse();
        r.setAddDocumentResponseWrapper(responseWrappers);
        return r;
    }
    
    /**
     * Implementation of CollectionAccess::RemoveDocuments
     */
    public RemoveDocumentsResponse removeDocuments(RemoveDocumentsRequest removeDocumentsRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType {
        DAIXXMLDBResource dra = (DAIXXMLDBResource) resource.getDataResourceAccessor();
        String collectionPath = null;
        if ( removeDocumentsRequest.getCollectionName() != null ) {
            collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
                removeDocumentsRequest.getCollectionName(),        
                dra.getCollectionName()
            );
            if ( collectionPath == null ) throw new InvalidCollectionNameFaultType();
        } 
        RemoveDocumentsResponse response = new RemoveDocumentsResponse();
        RemoveDocumentRequestWrapper[] docs = removeDocumentsRequest.getRemoveDocumentRequestWrapper();
        RemoveDocumentResponseWrapper[] responseWrappers = new RemoveDocumentResponseWrapper[docs.length];
        for(int i=0;i<docs.length;i++) {
            String docName = docs[i].getDocumentName();
            boolean exists = true;
            if ( ! dra.resourceExists(collectionPath,docName) ) {
                mLog.debug("Resource " + docName + " does not exist!");
                exists = false;
            }
            responseWrappers[i] = new RemoveDocumentResponseWrapper();
            responseWrappers[i].setDocumentName(docName);
            if ( ! exists ) {
                responseWrappers[i].setResponse(RemoveDocumentResponseWrapperResponse.value3);
                continue;
            }
            RemoveDocumentPerformDocument pd = null;
            String output = "RemoveDocument-" + UniqueName.getInstance().getName();
            try {
                pd = new RemoveDocumentPerformDocument(collectionPath,docName,output);
            } catch ( ParserConfigurationException e ) {
                mLog.error("Error creating perform document " + e);
                responseWrappers[i].setResponse(RemoveDocumentResponseWrapperResponse.value2);
                continue;
            }
            try {
                perform(resource,pd);
            } catch ( Exception e ) {
                mLog.error("Unable to execute perform document: " + e);
                responseWrappers[i].setResponse(RemoveDocumentResponseWrapperResponse.value2);
                continue;
            }
            //v1 = Success
            //v2 = DocumentNotRemoved-NotAuthorized
            //v3 = documentDoesNotExist
            responseWrappers[i].setResponse(RemoveDocumentResponseWrapperResponse.value1);
        }
        response.setRemoveDocumentResponseWrapper(responseWrappers);
        return response;
    }
    
    /**
     * Implementation of CollectionAccess::createSubCollection
     */
    public CreateSubcollectionResponse createSubcollection(CreateSubcollectionRequest createSubcollectionRequest, DataServiceResource resource) throws RemoteException, CollectionAlreadyExistsFaultType, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        mLog.debug("Creating subcollection");
        DAIXXMLDBResource dra = (DAIXXMLDBResource) resource.getDataResourceAccessor();
        String collectionPath = null;
        if ( createSubcollectionRequest.getCollectionName() != null ) {
            collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
                createSubcollectionRequest.getCollectionName(),
                dra.getCollectionName()
            );
            if ( collectionPath == null ) throw new InvalidCollectionNameFaultType();
        } 
        mLog.debug("Collection path: " + collectionPath);
        /* check that the collection doesn't already exist */
        if ( collectionAccessor.getCollectionMapping().collectionExists(createSubcollectionRequest.getSubcollectionName())) {
            mLog.error("Collection already exists!");
            throw new CollectionAlreadyExistsFaultType();
        }
        /*
         * The WS-DAIX collection URI must be mapped to a collection path.
         * The URI is translated as follows:
         * - the uri is converted to a string
         * - all instances of ":" converted to "_"
         * - all instances of "/" converted to "_" 
         */
        String subcollectionPath = 
            createSubcollectionRequest.getSubcollectionName().getScheme() + ":" +
            createSubcollectionRequest.getSubcollectionName().getPath();
        subcollectionPath = subcollectionPath.replace(':','_');
        subcollectionPath = subcollectionPath.replace('/','_');
        /*  
         * even though the collection doesn't already exist, need to ensure
         * that the collection string to which it has been mapped does not
         * exist. If it does, simply try appending _1, _2, ... to the path
         */
        int c = 1;
        String initialPath = subcollectionPath;
        while ( dra.subcollectionExists(collectionPath,subcollectionPath) ) {
            subcollectionPath = initialPath + "_" + c++;
        }
        mLog.debug("Subcollection path" + subcollectionPath);
        /*
         * Finally we need to add the mapping of URI/collection path
         */
        String absolutePath = dra.getCollectionName();
        if ( collectionPath != null )
            absolutePath = absolutePath + "/" + collectionPath;
        absolutePath = absolutePath + "/" + subcollectionPath;
        collectionAccessor.getCollectionMapping().addMapping(
            absolutePath,
            createSubcollectionRequest.getSubcollectionName()
        );
        /*
         * Now create a perform document to create the collection
         */
        String output = "CreateCollection-" + UniqueName.getInstance().getName();        
        Document responseDoc = null;
        PerformDocument performDoc = null;
        try {
            performDoc = new CreateCollectionPerformDocument(collectionPath,subcollectionPath,output);
        } catch ( ParserConfigurationException ex ) {
            mLog.error("Error executing perform document " + ex);
            throw new DataResourceUnavailableFaultType();   
        }
        try {
            responseDoc = perform(resource,performDoc);
        } catch ( Exception ex ) {
            mLog.error("Error executing perform document  " + ex);
            throw new DataResourceUnavailableFaultType();
        }
        mLog.debug("Collection " + absolutePath + " created successfully");
        return new CreateSubcollectionResponse();
    }
    
    
    /**
     * Implementation of CollectionAccess::RemoveSubcollection
     */
    public RemoveSubcollectionResponse removeSubcollection(RemoveSubcollectionRequest removeSubcollectionRequest, DataServiceResource resource)throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        DAIXXMLDBResource dra = (DAIXXMLDBResource) resource.getDataResourceAccessor();
        /* need to retrieve the path of the subCollection */
        String collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
            removeSubcollectionRequest.getSubcollectionName(),        
            dra.getCollectionName()
        );
        //the path will be null if the specified collection does not exist
        if ( collectionPath == null ) throw new InvalidCollectionNameFaultType();
        String output = "RemoveCollection-" + UniqueName.getInstance().getName();        
        Document responseDoc = null;
        PerformDocument performDoc = null;
        try {
            performDoc = new RemoveCollectionPerformDocument(collectionPath, output);
        } catch ( ParserConfigurationException ex ) {
            mLog.error("Error executing perform document " + ex);
            throw new DataResourceUnavailableFaultType();   
        }
        try {
            responseDoc = perform(resource,performDoc);
        } catch ( Exception e ) {
            mLog.error("Error executing perform document " + e);
            throw new DataResourceUnavailableFaultType();
        }
        return new RemoveSubcollectionResponse();
    }
    
}
