/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.porttypes;

import java.rmi.RemoteException;

import org.apache.log4j.Logger;

import uk.org.ogsadai.dais.xml.service.core.ConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType;
import uk.org.ogsadai.dais.xml.services.daix.CollectionSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.DocumentDoesNotExistFaultType;
import uk.org.ogsadai.dais.xml.services.daix.DocumentSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionConfigurationDocumentType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.config.FileBasedActivitiesConfig;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.sessions.SessionConfigurationLoader;
import uk.org.ogsadai.wsdaix.common.CollectionAccessor;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;
import uk.org.ogsadai.wsdaix.resource.DAIXXMLDBResource;
import uk.org.ogsadai.wsdaix.resource.ParentXMLResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLDocumentResource;

/**
 * 
 * Implementation of the WS-DAIX XMLCollectionFactory port type
 * 
 * @author Steven Lynden
 *
 */
public class XMLCollectionFactoryImpl extends PortTypeImpl {

    private static Logger mLog = Logger.getLogger(XMLCollectionFactoryImpl.class);
    
    private CollectionAccessor collectionAccessor;
    
    /**
     * Initialse the port type
     */
    public XMLCollectionFactoryImpl(DataServiceResourceFactory factory, DeploymentProperties deploymentProperties, CollectionAccessor collectionAccessor) {
        super(factory,deploymentProperties);
        this.collectionAccessor = collectionAccessor;
    }   

    /**
     * Implementation of CollectionFactory::CollectionSelection
     */
    public DataResourceAddressType collectionSelectionFactory(CollectionSelectionFactoryRequest collectionSelectionFactoryRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, InvalidConfigurationDocumentFaultType, InvalidPortTypeQNameFaultType, DataResourceUnavailableFaultType {
        mLog.debug("Creating new collection resource");
        ParentXMLResource dra = (ParentXMLResource) resource.getDataResourceAccessor();
        /*
         * Lookup the configuration map, which will give the port type QName
         * of the resource to be created
         */        
        ConfigurationMapType configurationMap = getConfigurationMap(
                            dra,
                            DAIXProperties.CollectionSelectionFactory,
                            collectionSelectionFactoryRequest );
        /* by default writeable = true */
        boolean writeable = true;
        /*
         * Some initial parameters may be specified in the configuration document
         * Of these, only writeable is supported as a configurable property by this
         * implementation of WS-DAIX.
         */
        if ( collectionSelectionFactoryRequest.getConfigurationDocument() != null ) {
            try {
                XMLCollectionConfigurationDocumentType xmlConfigDoc = (XMLCollectionConfigurationDocumentType) collectionSelectionFactoryRequest.getConfigurationDocument();
                writeable = isWriteable(xmlConfigDoc);
            } catch ( ClassCastException cce ) {
                throw new InvalidConfigurationDocumentFaultType();
            }
        }
        /*
         * Get the collection URI; this is the URI used to access the database,
         * not the WS-DAIX URI used to identify the collection
         */
        String dbCollectionURI = dra.getCollectionURI();;
        if ( collectionSelectionFactoryRequest.getCollectionName() != null ) {
            /* if the request specifies a sub collection, need to get
             * the base collection and then add the collection path
             */    
            String collectionPath = collectionAccessor.getCollectionMapping().URIToPath(
                    collectionSelectionFactoryRequest.getCollectionName() );
            /* if the collection path doesn't exist ... */
            if ( collectionPath == null )
                throw new InvalidCollectionNameFaultType();
            String baseCollectionName = ((DAIXXMLDBResource)dra).getCollectionName();
            /*
             * Here, baseCollectionName = something like /db/collectionA
             * collectionPath = something like /db/collectionA/collectionB
             * and the getBaseCollectionURI we give something like
             * xmldb:exist:/db/collectionA
             * This would require xmldb:exist:/db/collectionA/collectionB, 
             * so need extract the extra path from collectionPath when compared
             * to baseCollectionName and add it to the baseCollectionURI
             */
            collectionPath = collectionPath.substring(baseCollectionName.length());
            dbCollectionURI = ((DAIXXMLDBResource)dra).getBaseCollectionURI() + collectionPath;
        }
        mLog.debug("Collection URI is " + dbCollectionURI);
        /*
         * The created resource will be represented by the collection service
         * - need to get the endpoint from the deployment properties 
         */
        String url = 
          deploymentProperties.getEndpoint( DAIXProperties.COLLECTION_SERVICE, configurationMap.getPortTypeQName() );
        DataResourceAddressType addr = null;
        try {
            TransientXMLCollectionResource childDRA = new TransientXMLCollectionResource(dra,dbCollectionURI,dra.getRootCollectionURI());
            childDRA.setCollectionAccessor(collectionAccessor);
            childDRA.setWriteable(writeable);
            FileBasedActivitiesConfig activityConfig = DeploymentProperties.getActivityConfig(dra.getDirectory(),"");
            SessionConfigurationLoader sessionLoader = DeploymentProperties.getSessionConfig(dra.getDirectory(),"");
            DataResourceAddressType parent = null;
            DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.COLLECTION_COLLECTION_FACTORY_ENDPOINT),resource.getID().toString());
            String resourceId = this.exposeResource(childDRA, activityConfig, sessionLoader, deploymentProperties,parent);
            addr = DAIXUtil.constructDataResourceAddressType(
                    url,
                    resourceId );
        } catch ( DAIException e ) {
            mLog.error("Unable to create service managed XML collection resource: " + e);
            throw new DataResourceUnavailableFaultType();
        }
        return addr;
    }

    /*
     * Determines whether a configuration document specifies that a
     * created resource should be writeable or not. The default, if
     * not specified, is that the resource is writeable.
     */
    private boolean isWriteable(ConfigurationDocumentType configDoc) {
        try {
            return configDoc.getWriteable().booleanValue();
        } catch ( NullPointerException e ) { return true; }
    }
    
    /**
     * Implementation of CollectionFactory::CollectionSelection
     */
    public DataResourceAddressType documentSelectionFactory(DocumentSelectionFactoryRequest documentSelectionFactoryRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, DocumentDoesNotExistFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, InvalidConfigurationDocumentFaultType, InvalidPortTypeQNameFaultType, DataResourceUnavailableFaultType {
        mLog.debug("Creating a XML document resource");
        ParentXMLResource dra = (ParentXMLResource) resource.getDataResourceAccessor();
        /*
         * Lookup the configuration map, which will give the port type QName
         * of the resource to be created
         */
        ConfigurationMapType configurationMap = 
            configurationMap = getConfigurationMap(dra,DAIXProperties.DocumentSelectionFactory,documentSelectionFactoryRequest);
        /* by default writeable = true */
        boolean writeable = true;
        /*
         * Some initial parameters may be specified in the configuration document
         * Of these, only writeable is supported as a configurable property by this
         * implementation of WS-DAIX.
         */
        if ( documentSelectionFactoryRequest.getConfigurationDocument() != null ) {
            try {
                ConfigurationDocumentType configDoc = (ConfigurationDocumentType) documentSelectionFactoryRequest.getConfigurationDocument();
                writeable = isWriteable(configDoc);
            } catch ( ClassCastException cce ) {
                throw new InvalidConfigurationDocumentFaultType();
            }
        }
        String documentName = documentSelectionFactoryRequest.getDocumentName();
        /*
         * Get the collection path
         */
        String collectionPath = null;
        String collectionURI = dra.getCollectionURI();
        if ( documentSelectionFactoryRequest.getCollectionName() != null ) {
            collectionPath = collectionAccessor.getCollectionMapping().URIToRelativePath(
                documentSelectionFactoryRequest.getCollectionName(),        
                ((DAIXXMLDBResource)dra).getCollectionName()
            );
            if ( collectionPath == null )
                throw new InvalidCollectionNameFaultType();
        }
        if ( collectionPath != null )
            mLog.debug("Path of subcollection relative to collection: " + collectionPath);
        if (! ((DAIXXMLDBResource)dra).resourceExists(collectionPath,documentName) ) {
            mLog.debug("Resource " + documentName + " does not exist!");
            throw new DocumentDoesNotExistFaultType();
        }
        
        if ( collectionPath != null )
            collectionURI = collectionURI + "/" + collectionPath;
        mLog.debug("Collection URI: " + collectionURI);
        /*
         * The created resource will be represented by the document service
         * - need to get the endpoint from the deployment properties 
         */
        String url = 
            deploymentProperties.getEndpoint( DAIXProperties.DOCUMENT_SERVICE, configurationMap.getPortTypeQName() );
        DataResourceAddressType addr = null;
        try {        
            TransientXMLDocumentResource childDRA = new TransientXMLDocumentResource(dra,collectionURI,documentName,dra.getRootCollectionURI());
            FileBasedActivitiesConfig activityConfig = DeploymentProperties.getActivityConfig(dra.getDirectory(),"");
            SessionConfigurationLoader sessionLoader = DeploymentProperties.getSessionConfig(dra.getDirectory(),"");
            DataResourceAddressType parent = DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.COLLECTION_COLLECTION_FACTORY_ENDPOINT),resource.getID().toString());
            String resourceId = this.exposeResource(childDRA, activityConfig, sessionLoader, deploymentProperties, parent);
            addr = DAIXUtil.constructDataResourceAddressType(url,resourceId);
        } catch ( DAIException e ) {
            mLog.error("Unable to create XML resource: " + e);
            throw new DataResourceUnavailableFaultType();
        }
        return addr;    
    }
    
}
