/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import java.io.File;

import org.apache.log4j.Logger;
import org.xmldb.api.base.Collection;
import uk.org.ogsadai.wsdaix.common.*;
import uk.org.ogsadai.common.SecurityContext;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.rolemap.DatabaseRole;
import uk.org.ogsadai.common.rolemap.RoleMapAuthorizationException;
import uk.org.ogsadai.common.rolemap.RoleMapCannotAuthorizeException;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DatasetMapType;
import uk.org.ogsadai.dais.xml.service.core.LanguageMapType;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dataresource.DataResourceAccessorConfigException;
import uk.org.ogsadai.dataresource.DataResourceAccessorMetaDataException;
import uk.org.ogsadai.dataresource.DataResourceAccessorSetupException;
import uk.org.ogsadai.dataresource.DataResourceAuthorizationException;
import uk.org.ogsadai.dataresource.DataResourcePropertyNames;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.dataresource.XMLDBCollectionUseException;
import uk.org.ogsadai.dataresource.XMLDBDataResourceAccessor;
import uk.org.ogsadai.dataresource.XMLDBUnknownCollectionException;
import uk.org.ogsadai.dataresource.XMLDBUnknownSubCollectionException;
import uk.org.ogsadai.exception.AuthorizationExecutionException;
import uk.org.ogsadai.wsdaix.common.CollectionAccessor;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;
import org.xmldb.api.base.XMLDBException;

/**
 * 
 * Encapsulates a WS-DAIX data resource which represents a
 * collection or document in an XML database.
 * 
 * @author Steven Lynden
 *
 */
public abstract class DAIXXMLDBResource extends XMLDBDataResourceAccessor implements DAIXDataResourceAccessor {

    private static Logger mLog = Logger.getLogger(DAIXXMLDBResource.class);

    /*
     * Deployment properties of WS-DAIX. All resources have a reference
     * to this object.
     */
    protected DeploymentProperties deploymentProperties;
    
    /*
     * Address of the parent data resource.
     */
    protected DataResourceAddressType parentAddress;
    
    /*
     * Indicates whether the resource is writeable or not.
     */
    protected boolean writeable;
    
    /*
     * Empty security context. OK to do this with OGSA-DAI WS-I 
     * but not with WSRF.
     */
    protected SecurityContext securityContext = new SecurityContext(null,null);
    
    /*
     * URI of the root collection.  
     */
    protected String rootCollectionURI;
    
    /*
     * Used to access the mapping between collection names
     * and URIs.
     */
    protected CollectionAccessor collectionAccessor;
    
    /**
     * Sets the collection accessor object which is used by collection resources
     * to access collection schema and the mapping between OGSA-DAI and WS-DAIX
     * collections.
     */
    public void setCollectionAccessor(CollectionAccessor collectionAccessor) {
        this.collectionAccessor = collectionAccessor;
    }
    
    public void setWriteable(boolean writeable) {
        this.writeable = writeable;   
    }
    
    public boolean isWriteable() {
        return this.writeable;   
    }
    
    public void setResourceName(String dataResourceName) {
        super.setResourceName(dataResourceName);
    }
    
    /**
     * Determines whether a specific collection exists within a
     * named collection 
     */
    public boolean subcollectionExists(String collectionName, String subCollection) {
        if ( collectionName == null ) return subcollectionExists(subCollection);
        else
            return subcollectionExists(collectionName+"/"+subCollection);
    }
    
    /**
     * Determines whether a specific collection exists within the
     * root collection 
     */
    public boolean subcollectionExists(String collectionName) {
        Collection c = null;
        try {
            getXMLDBCollection(collectionName,securityContext.getDN());
        } catch ( Exception e ) {
            mLog.debug("Sub collection " + collectionName + " does not exist");
            return false;
        }
        try {
            this.releaseXMLDBCollection(c);
        } catch ( Exception e ) {
            mLog.debug("Problem relasing subcollection " + collectionName);
        }
        return true;
    }
    
    /**
     * Determines whether a specific document exists within the
     * root collection 
     */
    public boolean resourceExists(String resourceName) {
        return resourceExists(null,resourceName);
    }
    
    /**
     * Determines whether a specific document exists within a
     * named collection 
     */
    public boolean resourceExists(String collectionName, String resourceName) {
        Collection c = null;
        try {
            c = getXMLDBCollection(collectionName,securityContext.getDN());
        } catch ( Exception e ) {
            mLog.debug("Sub collection " + collectionName + " does not exist");
            return false;
        }
        boolean b = false;
        try {
            if ( c.getResource(resourceName) != null ) b = true;
        } catch ( Exception e ) { }
        try {
            this.releaseXMLDBCollection(c);
        } catch ( Exception e ) {
            mLog.debug("Problem relasing subcollection " + collectionName);
        }
        return b;
    }
    
    /**
     * Restore from config files. Applicable to service managed resources only
     */
    public void restoreFromConfig(File directory) throws DataResourceAccessorConfigException, DataResourceAccessorMetaDataException, DataResourceAccessorSetupException {
        super.restoreFromConfig(directory);  
        /* ensure that the security context is set on all properties
         * NOTE: this is will only work for OGSA-DAI WS-I
         * where the security context is constructed using
         * null parameters
         */
        for(int i=0;i<mProperties.length;i++)
            mProperties[i].setSecurityContext(securityContext);
    }
    
    public void setDeploymentProperties(DeploymentProperties deploymentProperties) {
        this.deploymentProperties = deploymentProperties;    
    }
    
    public void setParentDataResource(DataResourceAddressType address) {
        this.parentAddress = address;   
    }
    
    /**
     * Used by subclasses to construct property docs
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = DAIXUtil.getPropertyDocument();
        //DATASET MAP
        DatasetMapType[] datasetMap = new DatasetMapType[2];
        //xpath
        datasetMap[0] = new DatasetMapType();
        datasetMap[0].setMessageQName(DAIXProperties.XPathExecute);
        datasetMap[0].setDatasetFormatURI(DAIXProperties.defaultDataSetFormatURI);
        //xquery
        datasetMap[1] = new DatasetMapType();
        datasetMap[1].setMessageQName(DAIXProperties.XQueryExecute);
        datasetMap[1].setDatasetFormatURI(DAIXProperties.defaultDataSetFormatURI);
        properties.setDatasetMap(datasetMap);
        //LANGAUGE MAP
        LanguageMapType[] languageMap = new LanguageMapType[3];
        //xpath
        languageMap[0] = new LanguageMapType();
        languageMap[0].setLanguageURI(DAIXProperties.XQueryXURI);
        languageMap[0].setMessageQName(DAIXProperties.XPathExecute);
        //xquery
        languageMap[1] = new LanguageMapType();
        languageMap[1].setLanguageURI(DAIXProperties.XQueryXURI);
        languageMap[1].setMessageQName(DAIXProperties.XQueryExecute);
        //xupdate
        languageMap[2] = new LanguageMapType();
        languageMap[2].setLanguageURI(DAIXProperties.XUpdateURI);
        languageMap[2].setMessageQName(DAIXProperties.XUpdateExecute);
        properties.setLanguageMap(languageMap);
        //misc
        properties.setConcurrentAccess(true);
        properties.setWriteable(writeable);
        return properties;
    }
    
    public Property getCollectionSchemaProperty() {
        for(int i=0;i<mProperties.length;i++) {
            if ( mProperties[i].getName().toString().equals("{"+DataResourcePropertyNames.NAMESPACE+"}collectionSchema")) {
                return mProperties[i];
            }
        }
        return null;
    }
    
    /**
     * Returns the name of the root collection. This is different to the URI,
     * e.g. URL = xmldb:exist://... , name = db/collection
     * If anything goes wrong then something has happened to the root collection
     * and a DataResourceUnavailableFault should be generated
     */
    public String getCollectionName() throws DataResourceUnavailableFaultType {
        try {
            Collection c = getXMLDBCollection(null,securityContext.getDN());
            return c.getName();
        } catch ( DAIException e ) {
            mLog.error("Unable to retrieve collection name " + e.toString() );
            throw new DataResourceUnavailableFaultType();
        } catch ( XMLDBException e ) {
            mLog.error("Unable to retrieve collection name " + e.toString() );
            throw new DataResourceUnavailableFaultType();
        }
    }
    
    /**
     * Overriden because RoleMaps must be obtained using the root collection,
     * not the collection of the created resource. This is an issue where 
     * child collection resources are created.
     */
    public Collection getXMLDBCollection(String subCollection, String userCredentials) throws DataResourceAuthorizationException, XMLDBUnknownSubCollectionException, XMLDBCollectionUseException, AuthorizationExecutionException {
        mLog.debug("Getting collection from XMLDBDataResource");
        String collectionURI = mCollectionURI;
        if (! (subCollection == null || subCollection.length() == 0)) {
            collectionURI = collectionURI + "/" + subCollection;
        }
        DatabaseRole role = null;
        try {
            if ( rootCollectionURI != null ) {
                /*
                 * This will be the case if the resource is a transient
                 * collection resource (i.e. a service managed child)
                 */
                role = (DatabaseRole)mRoleMapper.map(userCredentials,
                rootCollectionURI);
            } else {
                /*
                 * This will be the case if the resource is the externally
                 * mangaged resource
                 */
                role = (DatabaseRole)mRoleMapper.map(userCredentials,
                mCollectionURI);
            }
        } catch (RoleMapAuthorizationException e) {
            throw new DataResourceAuthorizationException(userCredentials, e);
        } catch (RoleMapCannotAuthorizeException e) {
            throw new AuthorizationExecutionException(e);
        }
        Collection collection = getCollection(collectionURI, 
                 role.getUserId(), 
                 role.getPassword());
        if (collection == null) {
            if ((subCollection == null) || (subCollection.length() == 0)) {
                //Could not find the base collection.
                XMLDBUnknownCollectionException ex =
                new XMLDBUnknownCollectionException(collectionURI, 
                           role.getUserId());
                throw new XMLDBCollectionUseException(mCollectionURI, ex);
            } else {
                //We were looking for a subcollection which did not exist. 
                throw new XMLDBUnknownSubCollectionException(subCollection);
            }
        }
        mLog.debug("Have collection from XMLDBDataResource");
        return collection;
    }
    
}
