/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import java.io.File;

import org.apache.log4j.Logger;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.rolemap.RoleMapper;
import uk.org.ogsadai.dais.xml.service.core.ChildSensitiveToParent;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceManagement;
import uk.org.ogsadai.dais.xml.service.core.ParentSensitiveToChild;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType;
import uk.org.ogsadai.dataresource.DataResourceAccessorConfigException;
import uk.org.ogsadai.dataresource.DataResourceAccessorMetaDataException;
import uk.org.ogsadai.dataresource.DataResourceAccessorSetupException;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;

/**
 * 
 * Encapsulates an externally managed XML document resource
 * 
 * @author Steven Lynden
 *
 */
public class XMLCollectionResource extends DAIXXMLDBResource implements ParentXMLResource {

    private static Logger mLog = Logger.getLogger(XMLCollectionResource.class);
    
    /*
     * OGSA-DAI resource properties.
     */
    private Property[] properties;
    
    /*
     * Name of the data resource. 
     */
    private String dataResourceName;
    
    /*
     * Directory in which OGSA-DAI resource specific
     * configuration files reside. 
     */
    private String directory;
    
    /**
     * Set the name of the resource.
     */
    public void setResourceName(String dataResourceName) {
        super.setResourceName(dataResourceName);
        this.dataResourceName = dataResourceName; 
    }
    
    /**
     * Returns the OGSA-DAI role mapper object.
     */
    public RoleMapper getRoleMapper() {
        return this.mRoleMapper;   
    }
    
    /**
     * Returns the classname of the driver
     * used to interface with the database.
     */
    public String getDriverClassName() {
        return this.mDriverClassName;   
    }
    
    /**
     * Returns the directory in which configuration
     * files reside.
     */
    public String getDirectory() {
        return this.directory;   
    }
    
    /**
     * Returns the collection URI used by OGSA-DAI. 
     */
    public String getCollectionURI() {
        return this.mCollectionURI;   
    }
    
    /**
     * The service managed resource maintains a
     * root collection URI
     */
    public String getRootCollectionURI() {
        return this.mCollectionURI;
    }
    
    /**
     * Initialise the resource by reading config files
     */
    public void restoreFromConfig(File directory) throws DataResourceAccessorConfigException, DataResourceAccessorMetaDataException, DataResourceAccessorSetupException {
        super.restoreFromConfig(directory);
        this.directory = directory.getAbsolutePath();
        mLog.debug("XMLCollection resource - read config/initialise");
        //a resource specific properties may exist specifying writeability only
        //an attempt to read this is made; default is writeable=true
        try {
            DeploymentProperties dp = new DeploymentProperties(directory+"/"+DAIXProperties.DEPLOYMENT_PROPERTIES_FILE);
            writeable = dp.getWriteAccess();
        } catch ( Exception e) {
            mLog.debug("No resource specific deployment properties");
            writeable = true;
        }
        mLog.debug("Writeable = " +  writeable);    
        this.properties = new Property[0];
    }
    
    /**
     * Returns the WS-DAI property document
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = super.getDataResourcePropertyDocument();
        ConfigurationMapType[] configurationMap = DAIXUtil.createCollectionResourceConfigurationMap();
        properties.setConfigurationMap(configurationMap);
        properties.setChildSensitiveToParent(ChildSensitiveToParent.Insensitive);
        properties.setParentSensitiveToChild(ParentSensitiveToChild.Insensitive);
        properties.setDataResourceManagement(DataResourceManagement.ExternallyManaged);
        return properties;
    }

    /**
     * Returns the WS-DAIX collection property document
     */
    public XMLCollectionPropertyDocumentType getCollectionPropertyDocument() {
        mLog.debug("Extracting property document");
        PropertyDocumentType properties = getDataResourcePropertyDocument();
        mLog.debug("Retrieving XML collection specific properties");
        return DAIXUtil.propertyDocumentToCollectionPropertyDocument(properties,getCollectionSchemaProperty(),collectionAccessor); 
    }

}
