/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.axis.AxisFault;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.dais.xml.service.core.ChildSensitiveToParent;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceManagement;
import uk.org.ogsadai.dais.xml.service.core.DatasetMapType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.ParentSensitiveToChild;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCountFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidStartPositionFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XMLSequencePropertyDocumentType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;

/**
 * 
 * Encapsulates a service managed XML sequence resource
 * Implements the SequenceAccess::GetItems operation
 * 
 * @author Steven Lynden
 *
 */
public class XMLSequenceResource implements DAIXDataResourceAccessor {

    private static Logger mLog = Logger.getLogger(XMLSequenceResource.class);
    
    /*
     * OGSA-DAI resource properties.
     */
    private Property[] properties;
    
    /*
     * Name of the data resource. 
     */
    private String dataResourceName;
    
    /*
     * Directory in which OGSA-DAI resource specific
     * configuration files reside. 
     */
    private String directory;
    
    /*
     * Deployment properties to which all data resources
     * have access.
     */
    private DeploymentProperties deploymentProperties;
    
    /*
     * Address of the parent data resource. 
     */
    private DataResourceAddressType parentAddress;
    
    /*
     * The XML document contaning a sequence of elements
     * that the resource is representing 
     */
    private Document resourceSet;
    
    /*
     * Number of items in the XML sequence.
     */
    int numItems;
    
    /*
     * (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#setResourceName(java.lang.String)
     */
    public void setResourceName(String dataResourceName) {
        this.dataResourceName = dataResourceName; 
    }
    
    /*
     *  (non-Javadoc)
     * @see uk.org.ogsadai.wsdaix.resource.DAIXDataResourceAccessor#setDeploymentProperties(uk.org.ogsadai.wsdaix.common.DeploymentProperties)
     */
    public void setDeploymentProperties(DeploymentProperties deploymentProperties) {
        this.deploymentProperties = deploymentProperties;    
    }
    
    /*
     *  (non-Javadoc)
     * @see uk.org.ogsadai.wsdaix.resource.DAIXDataResourceAccessor#setParentDataResource(uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType)
     */
    public void setParentDataResource(DataResourceAddressType address) {
        this.parentAddress = address;   
    }

    /**
     * Initialise the data resource with an XML sequence resource set
     */
    public XMLSequenceResource(ParentXMLResource parent, Document resourceSet) throws DAIException {
        mLog.debug("Creating XML sequence resource");
        NodeList nodes = resourceSet.getDocumentElement().getChildNodes();
        numItems = 0;
        for(int i=0;i<nodes.getLength();i++)
            if ( nodes.item(i).getNodeName().equals("resource") ) numItems++;
        mLog.debug("Sequence has " + numItems + " items:");
        this.directory = parent.getDirectory();
        this.resourceSet = resourceSet;
        this.properties = new Property[0];
    }
    
    /*
     * (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#getProperties()
     */
    public Property[] getProperties() {
        return this.properties;   
    }
    
    /**
     * Implements XMLSequenceAccess::GetItems
     */
    public Document getItems(int startPos, int count) throws AxisFault, NotAuthorizedFaultType, InvalidStartPositionFaultType, InvalidCountFaultType {
        if ( startPos < 0 ) throw new InvalidStartPositionFaultType();
        if ( count < 1 ) throw new InvalidCountFaultType();
        Document doc = null;
        try {
            doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch ( Exception e ) {
            mLog.error("Exception creating results document: " + e );
            throw new AxisFault();
        }
        Element root = doc.createElement("resourceSet");
        doc.appendChild(root);
        NodeList nodes = resourceSet.getDocumentElement().getChildNodes();
        if ( startPos > numItems ) throw new InvalidStartPositionFaultType();
        if ( (startPos+count) > numItems ) throw new InvalidCountFaultType();
        //move to the desired item
        int c = 0;
        int actualStartPos = 0;
        for(int i=0;c<startPos;i++) {
            if ( nodes.item(i).getNodeName().equals("resource") ) c++;
            actualStartPos = i+1;
        }
        for(int i=actualStartPos;c<(startPos+count);i++) {
            if ( nodes.item(i).getNodeName().equals("resource") ) {
                Node n = doc.importNode(nodes.item(i),true);
                root.appendChild(n);
                c++;
            }
        }
        return doc;
    }
    
    /**
     * Returns the WS-DAI property document
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = DAIXUtil.getPropertyDocument();
        DatasetMapType[] datasetMap = new DatasetMapType[1];
        datasetMap[0] = new DatasetMapType();
        datasetMap[0].setMessageQName(DAIXProperties.GetItems);
        datasetMap[0].setDatasetFormatURI(DAIXProperties.defaultDataSetFormatURI);
        properties.setDatasetMap(datasetMap);
        properties.setWriteable(false);
        properties.setConcurrentAccess(true);
        properties.setChildSensitiveToParent(ChildSensitiveToParent.Insensitive);
        properties.setParentSensitiveToChild(ParentSensitiveToChild.Sensitive);
        properties.setConcurrentAccess(true);
        properties.setDataResourceManagement(DataResourceManagement.ServiceManaged);
        properties.setParentDataResource(parentAddress);
        return properties;
    }
    
    /**
     * Returns the WS-DAIX sequence property document
     */
    public XMLSequencePropertyDocumentType getXMLSequencePropertyDocument() {
        PropertyDocumentType properties = getDataResourcePropertyDocument();
        XMLSequencePropertyDocumentType xmlProperties = new XMLSequencePropertyDocumentType();
        xmlProperties.setChildSensitiveToParent(properties.getChildSensitiveToParent());
        xmlProperties.setConcurrentAccess(properties.isConcurrentAccess());
        xmlProperties.setConfigurationMap(properties.getConfigurationMap());
        xmlProperties.setDataResourceManagement(properties.getDataResourceManagement());
        xmlProperties.setDatasetMap(properties.getDatasetMap());
        xmlProperties.setLanguageMap(properties.getLanguageMap());
        xmlProperties.setNumberOfItems(numItems);
        xmlProperties.setParentDataResource(properties.getParentDataResource());
        xmlProperties.setParentSensitiveToChild(properties.getParentSensitiveToChild());
        xmlProperties.setReadable(properties.isReadable());
        xmlProperties.setTransactionInitiation(properties.getTransactionInitiation());
        xmlProperties.setTransactionIsolation(properties.getTransactionIsolation());
        xmlProperties.setWriteable(properties.isWriteable());
        return xmlProperties;
    }
    
}
